﻿using System;
using System.Collections.Generic;
using System.Web;
using Xunit;
using Moq;
using Weapsy.Apps.Forum.Services;
using Weapsy.Apps.Forum.Entities;
using Weapsy.Apps.Forum.Tests.Stubs;
using Weapsy.Data;
using Weapsy.Core.Caching;
using System.Linq;

namespace Weapsy.Apps.Forum.Tests.Services
{
    public class CategoryServiceTest : IDisposable
    {
        private Mock<IRepository<ForumCategory>> mockCategoryRepository;
        private Mock<ICacheManager> mockCacheManager;

        public CategoryServiceTest()
        {
            mockCategoryRepository = new Mock<IRepository<ForumCategory>>();
            mockCacheManager = new Mock<ICacheManager>();
        }

        [Fact]
        public void CategoryServiceConstructorTest()
        {
            var result = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);

            Assert.NotNull(result);
            Assert.IsType<CategoryService>(result);
        }

        [Fact]
        public void Create_GivenCategory_CreatesCategoryInRepository()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            ForumCategory entity = new ForumCategory();

            target.Create(entity);

            mockCategoryRepository.Verify(x => x.Create(It.IsAny<ForumCategory>()), Times.Once());
        }
        
        [Fact]
        public void Delete_DeletesCategoryFromRepository()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            ForumCategory entity = new ForumCategory();

            target.Delete(entity);

            mockCategoryRepository.Verify(x => x.Delete(It.IsAny<ForumCategory>()), Times.Once());
        }

        
        [Fact]
        public void GetById_UsingCacheByDefault_ReturnsCategory()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(CategoryStubs.GetCategoryListStub());
            int ConfigurationId = 1;
            int Id = 1;
            
            var result = target.GetById(ConfigurationId, Id) as ForumCategory;
            
            Assert.Equal(1, result.Id);
        }
        
        [Fact]
        public void GetById_NotUsingCache_ReturnsCategory()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCategoryRepository.SetupGet<IQueryable<ForumCategory>>(x => x.Table).Returns(CategoryStubs.GetCategoryListStub().AsQueryable<ForumCategory>());
            int ConfigurationId = 1;
            int Id = 1;

            var result = target.GetById(ConfigurationId, Id, false) as ForumCategory;

            Assert.Equal(Id, result.Id);
        }

        [Fact]
        public void GetById_IdDoesntExist_ReturnsNull()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCategoryRepository.SetupGet<IQueryable<ForumCategory>>(x => x.Table).Returns(CategoryStubs.GetCategoryListStub().AsQueryable<ForumCategory>());
            int ConfigurationId = 0;
            int Id = 0;

            var result = target.GetById(ConfigurationId, Id, false) as ForumCategory;

            Assert.Null(result);
        }
       
        [Fact]
        public void GetByName_UsingCacheByDefault_ReturnsCategory()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(CategoryStubs.GetCategoryListStub());
            int ConfigurationId = 1;
            string Name = "TestCategory1";

            var result = target.GetByName(ConfigurationId, Name) as ForumCategory;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByName_NotUsingCache_ReturnsCategory()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCategoryRepository.SetupGet<IQueryable<ForumCategory>>(x => x.Table).Returns(CategoryStubs.GetCategoryListStub().AsQueryable<ForumCategory>());
            int ConfigurationId = 1;
            string Name = "TestCategory1";

            var result = target.GetByName(ConfigurationId, Name, false) as ForumCategory;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByUrl_UsingCacheByDefault_ReturnsCategory()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(CategoryStubs.GetCategoryListStub());
            int ConfigurationId = 1;
            string Url = "test-category-1";

            var result = target.GetByUrl(ConfigurationId, Url) as ForumCategory;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByUrl_NotUsingCache_ReturnsCategory()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCategoryRepository.SetupGet<IQueryable<ForumCategory>>(x => x.Table).Returns(CategoryStubs.GetCategoryListStub().AsQueryable<ForumCategory>());
            int ConfigurationId = 1;
            string Url = "test-category-1";

            var result = target.GetByUrl(ConfigurationId, Url, false) as ForumCategory;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetList_UsingCacheByDefault_ReturnsListOfCategories()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(CategoryStubs.GetCategoryListStub());
            int ConfigurationId = 1;

            var result = target.GetList(ConfigurationId) as List<ForumCategory>;
            ForumCategory resultCategory = result.Find(a => a.Id == 1);

            Assert.Equal("TestCategory1", resultCategory.Name);
        }

        [Fact]
        public void GetList_NotUsingCache_ReturnsListOfCategories()
        {
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            mockCategoryRepository.SetupGet<IQueryable<ForumCategory>>(x => x.Table).Returns(CategoryStubs.GetCategoryListStub().AsQueryable<ForumCategory>());
            int ConfigurationId = 1;

            var result = target.GetList(ConfigurationId, false) as List<ForumCategory>;
            ForumCategory resultCategory = result.Find(a => a.Id == 1);

            Assert.Equal("TestCategory1", resultCategory.Name);
        }

        [Fact]
        public void Update_GivenAnyCategory_UpdatesRepository()
        {
            mockCategoryRepository = new Mock<IRepository<ForumCategory>>();
            mockCacheManager = new Mock<ICacheManager>();
            CategoryService target = new CategoryService(mockCategoryRepository.Object, mockCacheManager.Object);
            ForumCategory entity = new ForumCategory();

            target.Update(entity);

            mockCategoryRepository.Verify(d => d.Update(It.IsAny<ForumCategory>()), Times.Once());
        }

        public void Dispose()
        {
            // cleanup any resources
        }
    }
}
