#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Xml;
using Weapsy.Core.Caching;
using Weapsy.Entities;
using Weapsy.Services;

namespace Weapsy.Services
{
	public class InstallService : IInstallService
	{

        #region Private Members

        private readonly IAppService _appService;
        private readonly IEmailAccountService _emailAccountService;
        private readonly ILanguageService _languageService;
        private readonly IMailTemplateService _mailTemplateService;
        private readonly IMailTemplateTypeService _mailTemplateTypeService;
        private readonly IModuleDefinitionService _moduleDefinitionService;
        private readonly IModuleService _moduleService;
        private readonly IModuleTemplateService _moduleTemplateService;
        private readonly IPageService _pageService;
        private readonly IPageTemplateService _pageTemplateService;
        private readonly IPluginDefinitionService _pluginDefinitionService;
        private readonly IPluginService _pluginService;
        private readonly IRoleService _roleService;
        private readonly ISettingService _settingService;
        private readonly ISiteService _siteService;
        private readonly ITextService _textService;
        private readonly IThemeService _themeService;
        private readonly IUserService _userService;
        private readonly IWidgetService _widgetService;
        private readonly IWidgetDefinitionService _widgetDefinitionService;
        private readonly ICacheManager _cacheManager;

        private string _IpAddress = string.Empty;

        private string GetIpAddress()
        {
            if (_IpAddress == string.Empty)
            {
                if (HttpContext.Current != null)
                {
                    _IpAddress = HttpContext.Current.Request.UserHostAddress;
                }
                else
                {
                    _IpAddress = Consts.NullIpAddress;
                }
            }
            return _IpAddress;
        }

        #endregion

        #region Constructors

        public InstallService(IAppService appService,
            IEmailAccountService emailAccountService,
            ILanguageService languageService,
            IMailTemplateService mailTemplateService,
            IMailTemplateTypeService mailTemplateTypeService,
            IModuleDefinitionService moduleDefinitionService,
            IModuleService moduleService,
            IModuleTemplateService moduleTemplateService,
            IPageService pageService,
            IPageTemplateService pageTemplateService,
            IPluginDefinitionService pluginDefinitionService,
            IPluginService pluginService,
            IRoleService roleService,
            ISettingService settingService,
            ISiteService siteService,
            ITextService textService,
            IThemeService themeService,
            IUserService userService,
            IWidgetService widgetService,
            IWidgetDefinitionService widgetDefinitionService,
            ICacheManager cacheManager)
        {
            _appService = appService;
            _emailAccountService = emailAccountService;
            _languageService = languageService;
            _mailTemplateService = mailTemplateService;
            _mailTemplateTypeService = mailTemplateTypeService;
            _moduleDefinitionService = moduleDefinitionService;
            _moduleService = moduleService;
            _moduleTemplateService = moduleTemplateService;
            _pageService = pageService;
            _pageTemplateService = pageTemplateService;
            _pluginService = pluginService;
            _pluginDefinitionService = pluginDefinitionService;
            _roleService = roleService;
            _settingService = settingService;
            _siteService = siteService;
            _textService = textService;
            _themeService = themeService;
            _userService = userService;
            _widgetService = widgetService;
            _widgetDefinitionService = widgetDefinitionService;
            _cacheManager = cacheManager;
        }

        #endregion

        public void EnsureInstalled()
        {
            if (_siteService.Get() == null)
            {
                Initialize();
            }
        }

        public App EnsureAppInstalled(App app)
        {
            App entity = _appService.GetByName(app.Name);

            if (entity == null)
            {
                entity = new App();

                app.CreatedByUserId = Consts.NullId;
                app.CreatedByIpAddress = GetIpAddress();
                app.CreatedOnDate = Utils.GetDateForDB();

                app.ModifiedByUserId = Consts.NullId;
                app.ModifiedByIpAddress = GetIpAddress();
                app.ModifiedOnDate = Utils.GetDateForDB();

                _appService.Create(app);

                entity = _appService.GetByName(app.Name);
            }

            return entity;
        }

        public void EnsureModuleDefinitionInstalled(ModuleDefinition moduleDefinition)
        {
            ModuleDefinition entity = _moduleDefinitionService.GetByName(moduleDefinition.Name);

            if (entity == null)
            {
                moduleDefinition.CreatedByUserId = Consts.NullId;
                moduleDefinition.CreatedByIpAddress = GetIpAddress();
                moduleDefinition.CreatedOnDate = Utils.GetDateForDB();

                moduleDefinition.ModifiedByUserId = Consts.NullId;
                moduleDefinition.ModifiedByIpAddress = GetIpAddress();
                moduleDefinition.ModifiedOnDate = Utils.GetDateForDB();

                _moduleDefinitionService.Create(moduleDefinition);
            }
        }

        public void EnsurePluginDefinitionInstalled(PluginDefinition pluginDefinition)
        {
            PluginDefinition entity = _pluginDefinitionService.GetByName(pluginDefinition.Name);

            if (entity == null)
            {
                pluginDefinition.CreatedByUserId = Consts.NullId;
                pluginDefinition.CreatedByIpAddress = GetIpAddress();
                pluginDefinition.CreatedOnDate = Utils.GetDateForDB();

                pluginDefinition.ModifiedByUserId = Consts.NullId;
                pluginDefinition.ModifiedByIpAddress = GetIpAddress();
                pluginDefinition.ModifiedOnDate = Utils.GetDateForDB();

                _pluginDefinitionService.Create(pluginDefinition);
            }
        }

        public void EnsureWidgetDefinitionInstalled(WidgetDefinition widgetDefinition)
        {
            WidgetDefinition entity = _widgetDefinitionService.GetByName(widgetDefinition.Name);

            if (entity == null)
            {
                widgetDefinition.CreatedByUserId = Consts.NullId;
                widgetDefinition.CreatedByIpAddress = GetIpAddress();
                widgetDefinition.CreatedOnDate = Utils.GetDateForDB();

                widgetDefinition.ModifiedByUserId = Consts.NullId;
                widgetDefinition.ModifiedByIpAddress = GetIpAddress();
                widgetDefinition.ModifiedOnDate = Utils.GetDateForDB();

                _widgetDefinitionService.Create(widgetDefinition);
            }
        }

        public void Initialize()
        {
            // ==============================
            //  USERS AND ROLES
            // ==============================

            string superAdminName = AppSettings.UserSuperAdminName;
            string superAdminEmail = AppSettings.UserSuperAdminEmail;
            string adminName = AppSettings.UserAdminName;
            string adminEmail = AppSettings.UserAdminEmail;
            string roleSuperAdmin = AppSettings.RoleSuperAdmin;
            string roleAdmin = AppSettings.RoleAdmin;

            string superAdminMembershipName = "";
            string adminMembershipName = "";

            if (AppSettings.DefaultMembershipName == "Email")
            {
                superAdminMembershipName = AppSettings.UserSuperAdminEmail;
                adminMembershipName = AppSettings.UserAdminEmail;
            }
            else
            {
                superAdminMembershipName = AppSettings.UserSuperAdminName;
                adminMembershipName = AppSettings.UserAdminName;
            }

            if (Membership.GetUser(superAdminMembershipName) == null)
            {
                Membership.CreateUser(superAdminMembershipName, "123456", superAdminEmail);
            }

            if (Membership.GetUser(adminMembershipName) == null)
            {
                Membership.CreateUser(adminMembershipName, "123456", adminEmail);
            }

            if (!Roles.RoleExists(roleSuperAdmin))
            {
                Roles.CreateRole(roleSuperAdmin);
            }

            if (!Roles.RoleExists(roleAdmin))
            {
                Roles.CreateRole(roleAdmin);
            }

            if (!Roles.IsUserInRole(superAdminMembershipName, roleSuperAdmin))
            {
                Roles.AddUserToRole(superAdminMembershipName, roleSuperAdmin);
            }

            if (!Roles.IsUserInRole(adminMembershipName, roleAdmin))
            {
                Roles.AddUserToRole(adminMembershipName, roleAdmin);
            }

            MembershipUser mSuperAdmin = Membership.GetUser(superAdminMembershipName);

            if (_userService.GetByMembershipName(mSuperAdmin.UserName, false) == null)
            {
                User SuperAdmin = new User();

                SuperAdmin.MembershipName = mSuperAdmin.UserName;
                SuperAdmin.MembershipEmail = mSuperAdmin.Email;
                SuperAdmin.MembershipDate = mSuperAdmin.CreationDate;

                SuperAdmin.DisplayName = superAdminName;
                SuperAdmin.ProfileName = superAdminName;

                SuperAdmin.CreatedByUserId = Consts.NullId;
                SuperAdmin.CreatedByIpAddress = GetIpAddress();
                SuperAdmin.CreatedOnDate = Utils.GetDateForDB();

                SuperAdmin.ModifiedByUserId = Consts.NullId;
                SuperAdmin.ModifiedByIpAddress = GetIpAddress();
                SuperAdmin.ModifiedOnDate = Utils.GetDateForDB();

                _userService.Create(SuperAdmin);
            }

            MembershipUser mAdmin = Membership.GetUser(adminMembershipName);

            if (_userService.GetByMembershipName(mAdmin.UserName, false) == null)
            {
                User Admin = new User();

                Admin.MembershipName = mAdmin.UserName;
                Admin.MembershipEmail = mAdmin.Email;
                Admin.MembershipDate = mAdmin.CreationDate;

                Admin.DisplayName = adminName;
                Admin.ProfileName = adminName;

                Admin.CreatedByUserId = Consts.NullId;
                Admin.CreatedByIpAddress = GetIpAddress();
                Admin.CreatedOnDate = Utils.GetDateForDB();

                Admin.ModifiedByUserId = Consts.NullId;
                Admin.ModifiedByIpAddress = GetIpAddress();
                Admin.ModifiedOnDate = Utils.GetDateForDB();

                _userService.Create(Admin);
            }

            // ==============================
            //  SETTINGS
            // ==============================

            InstallSettings();

            // ==============================
            //  MODULE DEFINITIONS
            // ==============================

            ModuleDefinition TextModuleDef = InstallModuleDefinition("Text", "Module for Html content", "Text", "Text", "Module", "Text", "Edit");

            // ==============================
            //  PLUGIN DEFINITIONS
            // ==============================

            PluginDefinition EditProfilePluginDef = InstallPluginDefinition(PluginType.Account, "EditProfile", "Edit Profile", "Edit Profile", "Edit Profile", "Edit Profile", "Edit Profile", "Account", "EditProfile", "", "", "account-dashboard-profile.png", "account-menu-profile.png");
            PluginDefinition ChangePasswordPluginDef = InstallPluginDefinition(PluginType.Account, "ChangePassword", "Change Password", "Change Password", "Change Password", "Change Password", "Change Password", "Account", "ChangePassword", "", "", "account-dashboard-password.png", "account-menu-password.png");
            PluginDefinition ChangeEmailPluginDef = InstallPluginDefinition(PluginType.Account, "ChangeEmail", "Change Email", "Change Email", "Change Email", "Change Email", "Change Email", "Account", "ChangeEmail", "", "", "account-dashboard-email.png", "account-menu-email.png");

            // ==============================
            //  WIDGET DEFINITIONS
            // ==============================

            // -----

            // ==============================
            //  THEMES
            // ==============================

            List<Theme> themes = new List<Theme>();

            DirectoryInfo themeDir = new DirectoryInfo(HttpContext.Current.Request.MapPath("~/Themes/"));

            foreach (var dir in themeDir.GetDirectories())
            {
                themes.Add(InstallTheme(dir.Name, themes.Count + 1));
            }

            Theme defaultTheme = new Theme();

            var query = from item in themes
                           where item.Name == AppSettings.DefaultTheme
                           select item;

            defaultTheme = query.FirstOrDefault();

            if (defaultTheme == null)
            {
                defaultTheme = themes[0];
            }

            // ==============================
            //  PAGE TEMPLATES
            // ==============================

            PageTemplate PageTemplate1 = InstallPageTemplate("Header-Left-Content-Right-Footer", "Header,Left,Content,Right,Footer");
            PageTemplate PageTemplate2 = InstallPageTemplate("Header-Content-Right-Footer", "Header,Content,Right,Footer");
            PageTemplate PageTemplate3 = InstallPageTemplate("Header-Left-Content-Footer", "Header,Left,Content,Footer");
            PageTemplate PageTemplate4 = InstallPageTemplate("Header-Left-Right-Footer", "Header,Left,Right,Footer");
            PageTemplate PageTemplate5 = InstallPageTemplate("Header-Content-Footer", "Header,Content,Footer");

            // ==============================
            //  MODULE TEMPLATES
            // ==============================

            ModuleTemplate ModuleTemplate1 = InstallModuleTemplate("Standard");
            ModuleTemplate ModuleTemplate2 = InstallModuleTemplate("Empty");

            // ==============================
            //  SITE
            // ==============================

            Site site = InstallSite(defaultTheme.Id, PageTemplate1.Id, ModuleTemplate1.Id);

            // ==============================
            //  LANGUAGES
            // ==============================

            List<Language> Languages = new List<Language>();

            Language defaultLanguage = InstallLanguage(site.Id, AppSettings.DefaultLanguage, 1);

            Languages.Add(defaultLanguage);

            // ==============================
            //  MAIL TEMPLATE TYPES
            // ==============================

            MailTemplateType registerMailTemplateType = InstallMailTemplateType(new MailTemplateType()
            {
                AppId = Consts.NullId,
                Name = "Register",
                Description = "Register",
                TokenController = "MailTemplateType",
                TokenAction = "RegisterTokens"
            });

            MailTemplateType passwordRecoveryMailTemplateType = InstallMailTemplateType(new MailTemplateType()
            {
                AppId = Consts.NullId,
                Name = "PasswordRecovery",
                Description = "Password Recovery",
                TokenController = "MailTemplateType",
                TokenAction = "PasswordRecoveryTokens"
            });

            MailTemplateType changePasswordMailTemplateType = InstallMailTemplateType(new MailTemplateType()
            {
                AppId = Consts.NullId,
                Name = "ChangePassword",
                Description = "Change Password",
                TokenController = "MailTemplateType",
                TokenAction = "ChangePasswordTokens"
            });

            // ==============================
            //  MAIL TEMPLATES
            // ==============================

            MailTemplate registerMailTemplate = InstallMailTemplate(new MailTemplate() 
            {
                MailTemplateTypeId = registerMailTemplateType.Id,
                SiteId = site.Id,
                Name = "Register",
                Description = "Register Default Template",
                Subject = "Welcome to {SiteTitle}!",
                BodyHtml = "Thanks for joining our site.<br /><br /><strong>{SiteTitle}</strong>",
                BodyPlain = "Thanks for joining our site.\r\n\r\n{SiteTitle}",
            });

            MailTemplate passwordRecoveryMailTemplate = InstallMailTemplate(new MailTemplate()
            {
                MailTemplateTypeId = passwordRecoveryMailTemplateType.Id,
                SiteId = site.Id,
                Name = "PasswordRecovery",
                Description = "Password Recovery Default Template",
                Subject = "{SiteTitle} - Password Recovery",
                BodyHtml = "Password Recovered<br /><br />This is your New Password: <strong>{Password}</strong><br /><br />{SiteTitle}",
                BodyPlain = "Password Recovered\r\n\r\nThis is your New Password: {Password}\r\n\r\n{SiteTitle}",
            });

            MailTemplate changePasswordMailTemplate = InstallMailTemplate(new MailTemplate()
            {
                MailTemplateTypeId = changePasswordMailTemplateType.Id,
                SiteId = site.Id,
                Name = "ChangePassword",
                Description = "Change Password Default Template",
                Subject = "{SiteTitle} - Change Password",
                BodyHtml = "Password Changed<br /><br />This is your New Password: <strong>{Password}</strong><br /><br />{SiteTitle}",
                BodyPlain = "Password Changed\r\n\r\nThis is your New Password: {Password}\r\n\r\n{SiteTitle}",
            });

            // ==============================
            //  EMAIL ACCOUNTS
            // ==============================

            EmailAccount defaultEmailAccount = InstallEmailAccount(new EmailAccount()
            {
                SiteId = site.Id,
                Address = "info@mail.com",
                DisplayName = site.Title,
                Host = "smtp.mail.com",
                Port = 25,
                UserName = "",
                Password = "",
                DefaultCredentials = false,
                Ssl = false
            });

            // ==============================
            //  PAGES
            // ==============================

            Page Page1 = InstallPage(site.Id, "Home", "Welcome!"); 

            // ==============================
            //  MODULES
            // ==============================

            InstallTextModule(site.Id, Page1.Id, TextModuleDef.Id, "Left", 1, "Left Zone - Module 1", "First Text Module in Left Zone");
            InstallTextModule(site.Id, Page1.Id, TextModuleDef.Id, "Left", 2, "Left Zone - Module 2", "Second Text Module in Left Zone");
            InstallTextModule(site.Id, Page1.Id, TextModuleDef.Id, "Content", 1, "Content Zone - Module 1", "Weapsy " + Consts.Version + "<br /><br /><strong>Default Credentials</strong><br />login name: " + superAdminMembershipName +" <br />password: 123456");
            InstallTextModule(site.Id, Page1.Id, TextModuleDef.Id, "Content", 2, "Content Zone - Module 2", "Second Text Module in Content Zone");
            InstallTextModule(site.Id, Page1.Id, TextModuleDef.Id, "Right", 1, "Right Zone - Module 1", "First Text Module in Right Zone");
            InstallTextModule(site.Id, Page1.Id, TextModuleDef.Id, "Right", 2,  "Right Zone - Module 2","Second Text Module in Right Zone");

            // ==============================
            //  ACCOUNT PLUGINS
            // ==============================

            Plugin editProfilePlugin = InstallPlugin(site.Id, 
                EditProfilePluginDef.Id, 
                EditProfilePluginDef.DashboardText,
                EditProfilePluginDef.DashboardTitle, 
                EditProfilePluginDef.MenuText,
                EditProfilePluginDef.MenuTitle,
                1);
            Plugin changePasswordPlugin = InstallPlugin(site.Id, 
                ChangePasswordPluginDef.Id, 
                ChangePasswordPluginDef.DashboardText,
                ChangePasswordPluginDef.DashboardTitle,
                ChangePasswordPluginDef.MenuText,
                ChangePasswordPluginDef.MenuTitle, 
                2);
            Plugin changeEmailPlugin = InstallPlugin(site.Id, 
                ChangeEmailPluginDef.Id, 
                ChangeEmailPluginDef.DashboardText,
                ChangeEmailPluginDef.DashboardTitle,
                ChangeEmailPluginDef.MenuText,
                ChangeEmailPluginDef.MenuTitle, 
                3);

            // ==============================
            //  UPDATE SITE
            // ==============================

            Site siteToUpdate = new Site();

            siteToUpdate = _siteService.GetById(site.Id);

            siteToUpdate.LanguageId = defaultLanguage.Id;
            siteToUpdate.HomePageId = Page1.Id;

            siteToUpdate.RegisterMailTemplateId = registerMailTemplate.Id;
            siteToUpdate.PasswordRecoveryMailTemplateId = passwordRecoveryMailTemplate.Id;
            siteToUpdate.ChangePasswordMailTemplateId = changePasswordMailTemplate.Id;

            siteToUpdate.RegisterEmailAccountId = defaultEmailAccount.Id;
            siteToUpdate.PasswordRecoveryEmailAccountId = defaultEmailAccount.Id;
            siteToUpdate.ChangePasswordEmailAccountId = defaultEmailAccount.Id;

            _siteService.Update(siteToUpdate, new List<LocalizedSite>());

            // ==============================
            //  RESTART APPLICATION
            // ==============================

            Utils.RestartApplication();

            // ==============================
            //  REDIRECT TO HOME PAGE
            // ==============================

            //HttpContext.Current.Response.Redirect("");
        }

        private void InstallSettings()
        {
            LoginNameType loginNameType = LoginNameType.UserName;

            if (AppSettings.DefaultMembershipName == "Email")
            {
                loginNameType = LoginNameType.Email;
            }

            _settingService.Save<MemberSettings>(new MemberSettings()
            {
                RegistrationType = RegistrationType.Public, // 1 = Public | 2 = Private
                LoginName = loginNameType, // 1 = Email | 2 = UserName
                ChangeLoginName = false
            });

            _settingService.Save<SecuritySettings>(new SecuritySettings()
            {
                RegisterCaptcha = false,
                LoginCaptcha = false,
                PasswordRecoveryCaptcha = false,
                ReCaptchaPublicKey = "",
                ReCaptchaPrivateKey = "",
            });
        }

        private ModuleDefinition InstallModuleDefinition(string Name, string Description, string Title, string Controller, string Action, string AdminController, string AdminAction)
        {
            ModuleDefinition def = new ModuleDefinition();

            def = _moduleDefinitionService.GetByName(Name);

            if (def == null)
            {
                def = new ModuleDefinition();

                def.AppId = Consts.NullId;
                def.Name = Name;
                def.Description = Description;
                def.Title = Title;
                def.Controller = Controller;
                def.Action = Action;
                def.AdminController = AdminController;
                def.AdminAction = AdminAction;

                def.CreatedByUserId = Consts.NullId;
                def.CreatedByIpAddress = GetIpAddress();
                def.CreatedOnDate = Utils.GetDateForDB();

                def.ModifiedByUserId = Consts.NullId;
                def.ModifiedByIpAddress = GetIpAddress();
                def.ModifiedOnDate = Utils.GetDateForDB();

                _moduleDefinitionService.Create(def);
            }

            return def;
        }

        private PluginDefinition InstallPluginDefinition(PluginType pluginType, string Name, string Description, string DashboardText, string DashboardTitle, string MenuText, string MenuTitle, string Controller, string Action, string AdminController, string AdminAction, string dashboardIcon, string menuIcon)
        {
            PluginDefinition def = new PluginDefinition();

            def = _pluginDefinitionService.GetByName(Name);

            if (def == null)
            {
                def = new PluginDefinition();

                def.AppId = Consts.NullId;
                def.Name = Name;
                def.Description = Description;
                def.DashboardText = DashboardText;
                def.DashboardTitle = DashboardTitle;
                def.MenuText = MenuText;
                def.MenuTitle = MenuTitle;
                def.Controller = Controller;
                def.Action = Action;
                def.AdminController = AdminController;
                def.AdminAction = AdminAction;
                def.TypeId = (byte)pluginType;
                def.DashboardIcon = dashboardIcon;
                def.MenuIcon = menuIcon;
                def.CreatedByUserId = Consts.NullId;
                def.CreatedByIpAddress = GetIpAddress();
                def.CreatedOnDate = Utils.GetDateForDB();

                def.ModifiedByUserId = Consts.NullId;
                def.ModifiedByIpAddress = GetIpAddress();
                def.ModifiedOnDate = Utils.GetDateForDB();

                _pluginDefinitionService.Create(def);
            }

            return def;
        }

        private WidgetDefinition InstallWidgetDefinition(WidgetType widgetType, string Name, string Description, string Title, string Controller, string Action, string AdminController, string AdminAction, string icon)
        {
            WidgetDefinition def = new WidgetDefinition();

            def = _widgetDefinitionService.GetByName(Name);

            if (def == null)
            {
                def = new WidgetDefinition();

                def.AppId = Consts.NullId;
                def.Name = Name;
                def.Description = Description;
                def.Title = Title;
                def.Controller = Controller;
                def.Action = Action;
                def.AdminController = AdminController;
                def.AdminAction = AdminAction;
                def.TypeId = (byte)widgetType;
                def.Icon = icon;

                def.CreatedByUserId = Consts.NullId;
                def.CreatedByIpAddress = GetIpAddress();
                def.CreatedOnDate = Utils.GetDateForDB();

                def.ModifiedByUserId = Consts.NullId;
                def.ModifiedByIpAddress = GetIpAddress();
                def.ModifiedOnDate = Utils.GetDateForDB();

                _widgetDefinitionService.Create(def);
            }

            return def;
        }

        private Theme InstallTheme(string Name, int SortOrder)
        {
            Theme theme = new Theme();

            theme = _themeService.GetByName(Name);

            if (theme == null)
            {
                theme = new Theme();

                theme.Name = Name;
                theme.Description = Name;
                theme.Folder = Name;
                theme.PreviewImage = "theme.png";
                theme.SortOrder = SortOrder;

                theme.CreatedByUserId = Consts.NullId;
                theme.CreatedByIpAddress = GetIpAddress();
                theme.CreatedOnDate = Utils.GetDateForDB();

                theme.ModifiedByUserId = Consts.NullId;
                theme.ModifiedByIpAddress = GetIpAddress();
                theme.ModifiedOnDate = Utils.GetDateForDB();

                _themeService.Create(theme);
            }

            return theme;
        }

        private PageTemplate InstallPageTemplate(string Name, string Zones)
        {
            PageTemplate template = new PageTemplate();

            template = _pageTemplateService.GetByName(Name);

            if (template == null)
            {
                template = new PageTemplate();

                template.Name = Name;
                template.Zones = Zones;
                template.ViewName = template.Name;
                template.PreviewImage = Name + ".png";

                template.CreatedByUserId = Consts.NullId;
                template.CreatedByIpAddress = GetIpAddress();
                template.CreatedOnDate = Utils.GetDateForDB();

                template.ModifiedByUserId = Consts.NullId;
                template.ModifiedByIpAddress = GetIpAddress();
                template.ModifiedOnDate = Utils.GetDateForDB();

                _pageTemplateService.Create(template);
            }

            return template;
        }

        private ModuleTemplate InstallModuleTemplate(string Name)
        {
            ModuleTemplate template = new ModuleTemplate();

            template = _moduleTemplateService.GetByName(Name);

            if (template == null)
            {
                template = new ModuleTemplate();

                template.Name = Name;
                template.ViewName = template.Name;
                template.PreviewImage = Name + ".png";

                template.CreatedByUserId = Consts.NullId;
                template.CreatedByIpAddress = GetIpAddress();
                template.CreatedOnDate = Utils.GetDateForDB();

                template.ModifiedByUserId = Consts.NullId;
                template.ModifiedByIpAddress = GetIpAddress();
                template.ModifiedOnDate = Utils.GetDateForDB();

                _moduleTemplateService.Create(template);
            }

            return template;
        }

        private Site InstallSite(int themeId, int pageTemplateId, int moduleTemplateId)
        {
            Site site = new Site();

            site.Name = AppSettings.SiteName;
            site.Title = "My Website";
            site.ThemeId = themeId;
            site.PageTemplateId = pageTemplateId;
            site.ModuleTemplateId = moduleTemplateId;
            site.LanguageSelectionEnabled = true;
            site.LanguageUrlEnabled = true;
            site.Copyright = "Copyright &copy; " + DateTime.Now.Year + " by " + site.Title;

            //site.AccountLayout = "_AccountLayout";
            //site.AdminLayout = "_AdminLayout";
            //site.AuthLayout = "_AuthLayout";
            //site.GroupLayout = "_GroupLayout";
            //site.ProfileLayout = "_ProfileLayout";
            //site.SiteLayout = "_SiteLayout";

            site.CreatedByUserId = Consts.NullId;
            site.CreatedByIpAddress = GetIpAddress();
            site.CreatedOnDate = Utils.GetDateForDB();

            site.ModifiedByUserId = Consts.NullId;
            site.ModifiedByIpAddress = GetIpAddress();
            site.ModifiedOnDate = Utils.GetDateForDB();

            _siteService.Create(site);

            return site;
        }

        private Language InstallLanguage(int SiteId, string lang, int sortOrder)
        {
            Language language = new Language();

            CultureInfo culture = CultureInfo.GetCultureInfo(lang);

            language.CultureName = culture.Name;
            language.CultureNativeName = culture.NativeName;
            language.CultureEnglishName = culture.EnglishName;
            language.Url = lang;

            language.SiteId = SiteId;
            language.Name = culture.NativeName;
            language.SortOrder = sortOrder;
            language.Active = true;

            language.CreatedByUserId = Consts.NullId;
            language.CreatedByIpAddress = GetIpAddress();
            language.CreatedOnDate = Utils.GetDateForDB();

            language.ModifiedByUserId = Consts.NullId;
            language.ModifiedByIpAddress = GetIpAddress();
            language.ModifiedOnDate = Utils.GetDateForDB();

            _languageService.Create(language);

            return language;
        }

        private MailTemplateType InstallMailTemplateType(MailTemplateType mtt)
        {
            mtt.CreatedByUserId = Consts.NullId;
            mtt.CreatedByIpAddress = GetIpAddress();
            mtt.CreatedOnDate = Utils.GetDateForDB(); 

            mtt.ModifiedByUserId = Consts.NullId;
            mtt.ModifiedByIpAddress = GetIpAddress();
            mtt.ModifiedOnDate = Utils.GetDateForDB();

            _mailTemplateTypeService.Create(mtt);

            return mtt;
        }

        private MailTemplate InstallMailTemplate(MailTemplate template)
        {
            template.CreatedByUserId = Consts.NullId;
            template.CreatedByIpAddress = GetIpAddress();
            template.CreatedOnDate = Utils.GetDateForDB();

            template.ModifiedByUserId = Consts.NullId;
            template.ModifiedByIpAddress = GetIpAddress();
            template.ModifiedOnDate = Utils.GetDateForDB();

            _mailTemplateService.Create(template, new List<LocalizedMailTemplate>());

            return template;
        }

        private EmailAccount InstallEmailAccount(EmailAccount account)
        {
            account.CreatedByUserId = Consts.NullId;
            account.CreatedByIpAddress = GetIpAddress();
            account.CreatedOnDate = Utils.GetDateForDB();

            account.ModifiedByUserId = Consts.NullId;
            account.ModifiedByIpAddress = GetIpAddress();
            account.ModifiedOnDate = Utils.GetDateForDB();

            _emailAccountService.Create(account);

            return account;
        }

        private Page InstallPage(int SiteId, string Name, string Title)
        {
            Page page = new Page();

            page.SiteId = SiteId;
            page.Name = Name;
            page.Url = Utils.ParseTextForUrl(Name);
            page.PageTitle = Title;
            page.MainMenu = true;
            page.SortOrder = 1;
            page.Active = true;
            page.LanguageUrlEnabled = true;
            page.Authorized = _roleService.GetDefaultRoles(true, false, false);
            page.DisplayPageTitle = true;
            page.SiteMap = true;
            page.PageTemplateId = Consts.NullId;
            page.ModuleTemplateId = Consts.NullId;

            page.CreatedByUserId = Consts.NullId;
            page.CreatedByIpAddress = GetIpAddress();
            page.CreatedOnDate = Utils.GetDateForDB();

            page.ModifiedByUserId = Consts.NullId;
            page.ModifiedByIpAddress = GetIpAddress();
            page.ModifiedOnDate = Utils.GetDateForDB();

            _pageService.Create(page, new List<LocalizedPage>());

            return page;
        }

        private Plugin InstallPlugin(int SiteId, int defId, string DashboardText, string DashboardTitle, string MenuText, string MenuTitle, int order)
        {
            Plugin plugin = new Plugin();

            plugin.SiteId = SiteId;
            plugin.PluginDefinitionId = defId;
            plugin.DashboardText = DashboardText;
            plugin.DashboardTitle = DashboardTitle;
            plugin.MenuText = MenuText;
            plugin.MenuTitle = MenuTitle;
            plugin.SortOrder = order;
            plugin.Active = true;
            plugin.Authorized = _roleService.GetDefaultRoles(false, true, false);

            plugin.CreatedByUserId = Consts.NullId;
            plugin.CreatedByIpAddress = GetIpAddress();
            plugin.CreatedOnDate = Utils.GetDateForDB();

            plugin.ModifiedByUserId = Consts.NullId;
            plugin.ModifiedByIpAddress = GetIpAddress();
            plugin.ModifiedOnDate = Utils.GetDateForDB();

            _pluginService.Create(plugin, new List<LocalizedPlugin>());

            return plugin;
        }

        private Widget InstallWidget(int SiteId, int defId, string Title, int order)
        {
            Widget widget = new Widget();

            widget.SiteId = SiteId;
            widget.WidgetDefinitionId = defId;
            widget.Title = Title;
            widget.SortOrder = order;
            widget.Active = true;
            widget.Authorized = _roleService.GetDefaultRoles(true, false, false);

            widget.CreatedByUserId = Consts.NullId;
            widget.CreatedByIpAddress = GetIpAddress();
            widget.CreatedOnDate = Utils.GetDateForDB();

            widget.ModifiedByUserId = Consts.NullId;
            widget.ModifiedByIpAddress = GetIpAddress();
            widget.ModifiedOnDate = Utils.GetDateForDB();

            _widgetService.Create(widget, new List<LocalizedWidget>());

            return widget;
        }

        private void InstallTextModule(int SiteId, int PageId, int ModuleDefId, string Zone, int SortOrder, string Title, string ModuleText)
        {
            Module module = new Module();

            module.SiteId = SiteId;
            module.ModuleDefinitionId = ModuleDefId;
            module.Title = Title;
            module.InheritPermissions = true;
            module.Authorized = _roleService.GetDefaultRoles(true, false, false);
            module.AllPages = false;
            module.DisplayTitle = true;
            module.Zone = "Content";

            module.CreatedByUserId = Consts.NullId;
            module.CreatedByIpAddress = GetIpAddress();
            module.CreatedOnDate = Utils.GetDateForDB();

            module.ModifiedByUserId = Consts.NullId;
            module.ModifiedByIpAddress = GetIpAddress();
            module.ModifiedOnDate = Utils.GetDateForDB();

            PageModule pageModule = new PageModule();

            pageModule.PageId = PageId;
            pageModule.ModuleId = module.Id;
            pageModule.Zone = Zone;
            pageModule.SortOrder = SortOrder;
            pageModule.ModuleTemplateId = Consts.NullId;
            
            pageModule.CreatedByUserId = Consts.NullId;
            pageModule.CreatedByIpAddress = GetIpAddress();
            pageModule.CreatedOnDate = Utils.GetDateForDB();

            pageModule.ModifiedByUserId = Consts.NullId;
            pageModule.ModifiedByIpAddress = GetIpAddress();
            pageModule.ModifiedOnDate = Utils.GetDateForDB();

            _moduleService.AddNew(module, new List<LocalizedModule>(), pageModule, new List<LocalizedPageModule>());

            Text text = new Text();

            text.ModuleId = module.Id;
            text.TextVersionId = 0;

            text.CreatedByUserId = Consts.NullId;
            text.CreatedByIpAddress = GetIpAddress();
            text.CreatedOnDate = Utils.GetDateForDB();

            text.ModifiedByUserId = Consts.NullId;
            text.ModifiedByIpAddress = GetIpAddress();
            text.ModifiedOnDate = Utils.GetDateForDB();

            TextVersion firstVersion = new TextVersion();

            firstVersion.Body = ModuleText;

            firstVersion.CreatedByUserId = Consts.NullId;
            firstVersion.CreatedByIpAddress = GetIpAddress();
            firstVersion.CreatedOnDate = Utils.GetDateForDB();

            firstVersion.ModifiedByUserId = Consts.NullId;
            firstVersion.ModifiedByIpAddress = GetIpAddress();
            firstVersion.ModifiedOnDate = Utils.GetDateForDB();

            _textService.Create(text, firstVersion, new List<LocalizedTextVersion>());

            //text.VersionId = firstVersion.Id;

            //db.SaveChanges();
        }

	}
}