#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class LanguageService : ILanguageService
    {

        #region Private Members

        private readonly IRepository<Language> _languageRepository;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        public const string Key = Consts.GlobalKey + "|Languages|SiteId:{0}";

        #endregion

        #region Constructors

        public LanguageService(IRepository<Language> languageRepository,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _languageRepository = languageRepository;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual Language Create(Language entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Language");
            }

            _languageRepository.Create(entity);

            _cacheManager.Remove(string.Format(Key, entity.SiteId));

            return entity;
        }

        public virtual void Update(Language entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Language");
            }

            _languageRepository.Update(entity);

            _cacheManager.Remove(string.Format(Key, entity.SiteId));
        }

        public virtual void Delete(Language entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Language");
            }

            _languageRepository.Delete(entity);

            _cacheManager.Remove(string.Format(Key, entity.SiteId));
        }

        public virtual Language GetById(int SiteId, int Id)
        {
            return GetById(SiteId, Id, true);
        }

        public virtual Language GetById(int SiteId, int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(SiteId, true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(SiteId, Id);
            }
        }

        public virtual Language GetByName(int SiteId, string Name)
        {
            return GetByName(SiteId, Name, true);
        }

        public virtual Language GetByName(int SiteId, string Name, bool UseCache = true)
        {
            if (UseCache)
            {
                return GetList(SiteId, true).Find(o => o.Name == Name);
            }
            else
            {
                return _GetByName(SiteId, Name);
            }
        }

        public virtual Language GetByCultureName(int SiteId, string CultureName)
        {
            return GetByCultureName(SiteId, CultureName, true);
        }

        public virtual Language GetByCultureName(int SiteId, string CultureName, bool UseCache = true)
        {
            if (UseCache)
            {
                return GetList(SiteId, true).Find(o => o.CultureName == CultureName);
            }
            else
            {
                return _GetByCultureName(SiteId, CultureName);
            }
        }

        public virtual Language GetByUrl(int SiteId, string Url)
        {
            return GetByUrl(SiteId, Url, true);
        }

        public virtual Language GetByUrl(int SiteId, string Url, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(SiteId, true).Find(o => o.Url == Url);
            }
            else
            {
                return _GetByUrl(SiteId, Url);
            }
        }

        public virtual List<Language> GetList(int SiteId)
        {
            return GetList(SiteId, true);
        }

        public virtual List<Language> GetList(int SiteId, bool UseCache)
        {
            List<Language> list;
            if (UseCache)
            {
                string key = string.Format(Key, SiteId);
                object obj = _cacheManager.Get(key);
                if (obj != null)
                {
                    list = (List<Language>)obj;
                }
                else
                {
                    list = _GetList(SiteId);
                    _cacheManager.Insert(key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList(SiteId);
            }
            return list;
        }

        public virtual Dictionary<string, string> GetDictionary(int SiteId)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            foreach (var item in GetList(SiteId))
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }
            return dic;
        }

        public void Move(int SiteId, int Id, string Action)
        {
            Language entity1 = _GetById(SiteId, Id);
            int entity1NewSortOrder;

            Language entity2 = new Language();
            int entity2NewSortOrder;

            if (Action == "UP")
            {
                entity2 = _GetBySortOrder(SiteId, entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(SiteId, entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _languageRepository.Update(entity1);
            _languageRepository.Update(entity2);

            _cacheManager.RemoveByPattern(string.Format(Key, SiteId));
        }

        #endregion

        #region Private Methods

        private Language _GetById(int siteId, int id)
        {
            var query = from item in _languageRepository.Table
                        where item.SiteId == siteId & item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private Language _GetByName(int siteId, string name)
        {
            var query = from item in _languageRepository.Table
                        where item.SiteId == siteId & item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private Language _GetByCultureName(int siteId, string cultureName)
        {
            var query = from item in _languageRepository.Table
                        where item.SiteId == siteId & item.CultureName == cultureName
                        select item;
            return query.FirstOrDefault();
        }

        private Language _GetByUrl(int siteId, string url)
        {
            var query = from item in _languageRepository.Table
                        where item.SiteId == siteId & item.Url == url
                        select item;
            return query.FirstOrDefault();
        }

        private Language _GetBySortOrder(int siteId, int sortOrder)
        {
            var query = from item in _languageRepository.Table
                        where item.SiteId == siteId & item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        private List<Language> _GetList(int siteId)
        {
            var query = from item in _languageRepository.Table
                        where item.SiteId == siteId
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        #endregion

    }

}