#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class MailTemplateService : IMailTemplateService
    {

        #region Private Members

        private readonly IRepository<MailTemplate> _mailTemplateRepository;
        private readonly IRepository<LocalizedMailTemplate> _localizedMailTemplateRepository;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        private const string CacheKey = Consts.GlobalKey + "|MailTemplates|SiteId:{0}|LanguageId:{1}";
        private const string CacheKeyPattern = Consts.GlobalKey + "|MailTemplates|SiteId:{0}";

        #endregion

        #region Constructors

        public MailTemplateService(IRepository<MailTemplate> mailTemplateRepository,
            IRepository<LocalizedMailTemplate> localizedMailTemplateRepository,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _mailTemplateRepository = mailTemplateRepository;
            _localizedMailTemplateRepository = localizedMailTemplateRepository;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual void Create(MailTemplate entity, List<LocalizedMailTemplate> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("MailTemplate");
            }

            _mailTemplateRepository.Create(entity);

            foreach (var item in localizedEntities)
            {
                item.MailTemplateId = entity.Id;
                _localizedMailTemplateRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual LocalizedMailTemplate CreateLocalized(LocalizedMailTemplate entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedMailTemplate");
            }

            _localizedMailTemplateRepository.Create(entity);

            return entity;
        }

        public virtual void Update(MailTemplate entity, List<LocalizedMailTemplate> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("MailTemplate");
            }

            _mailTemplateRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedMailTemplateRepository.Update(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual void Delete(MailTemplate entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("MailTemplate");
            }

            foreach (var item in GetLocalizedList(entity.Id))
            {
                _localizedMailTemplateRepository.Delete(item);
            }

            _mailTemplateRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual MailTemplate GetById(int SiteId, int Id)
        {
            var query = from item in _mailTemplateRepository.Table
                        .Include("MailTemplateType")
                        where item.SiteId == SiteId & item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual MailTemplate GetSiteMailTemplate(int SiteId, int Id, int LanguageId)
        {
            foreach (var item in GetSiteMailTemplates(SiteId, LanguageId))
            {
                if (item.Id == Id) return item;
            }
            return null;
        }

        public virtual LocalizedMailTemplate GetLocalizedByMailTemplateId(int MailTemplateId, int LanguageId)
        {
            var query = from item in _localizedMailTemplateRepository.Table
                        where item.MailTemplateId == MailTemplateId
                        & item.LanguageId == LanguageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedMailTemplate GetLocalizedById(int Id)
        {
            var query = from item in _localizedMailTemplateRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<MailTemplate> GetList(int siteId)
        {
            var query = from item in _mailTemplateRepository.Table
                        where item.SiteId == siteId
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        public virtual List<LocalizedMailTemplate> GetLocalizedList(int mailTemplateId)
        {
            var query = from item in _localizedMailTemplateRepository.Table
                        where item.MailTemplateId == mailTemplateId
                        select item;
            return query.ToList();
        }

        public virtual List<MailTemplate> GetSiteMailTemplates(int SiteId, int LanguageId)
        {
            List<MailTemplate> list = new List<MailTemplate>();
            string key = string.Format(CacheKey, SiteId, LanguageId);
            object obj = _cacheManager.Get(key);
            if (obj != null)
            {
                list = (List<MailTemplate>)obj;
            }
            else
            {
                list = _GetSiteMailTemplates(SiteId, LanguageId);
                _cacheManager.Insert(key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
            }
            return list;
        }

        #endregion

        #region Private Methods

        private List<MailTemplate> _GetSiteMailTemplates(int siteId, int languageId)
        {
            List<MailTemplate> list = new List<MailTemplate>();

            var query = from item in _mailTemplateRepository.Table
                        .Include("LocalizedMailTemplates")
                        where item.SiteId == siteId
                        orderby item.Name
                        select item;

            list = query.ToList();

            LocalizedMailTemplate LocalizedMailTemplate = new LocalizedMailTemplate();

            string LocalizedSubject = "";
            string LocalizedBodyHtml = "";
            string LocalizedBodyPlain = "";

            foreach (var item in list)
            {
                LocalizedMailTemplate = null;

                LocalizedSubject = "";
                LocalizedBodyHtml = "";
                LocalizedBodyPlain = "";

                foreach (var lp in item.LocalizedMailTemplates)
                {
                    if (lp.LanguageId == languageId)
                    {
                        LocalizedMailTemplate = lp;
                        break;
                    }
                }

                if (LocalizedMailTemplate == null)
                {
                    LocalizedSubject = item.Subject;
                    LocalizedBodyHtml = item.BodyHtml;
                    LocalizedBodyPlain = item.BodyPlain;
                }
                else
                {
                    LocalizedSubject = Utils.Choose(LocalizedMailTemplate.Subject, item.Subject);
                    LocalizedBodyHtml = Utils.Choose(LocalizedMailTemplate.BodyHtml, item.BodyHtml);
                    LocalizedBodyPlain = Utils.Choose(LocalizedMailTemplate.BodyPlain, item.BodyPlain);
                }

                item.LocalizedSubject = LocalizedSubject;
                item.LocalizedBodyHtml = LocalizedBodyHtml;
                item.LocalizedBodyPlain = LocalizedBodyPlain;
            }

            return list;
        }

        #endregion

	}

}