#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class ModuleService : IModuleService
    {

        #region Private Members

        private readonly IRepository<Module> _moduleRepository;
        private readonly IRepository<LocalizedModule> _localizedModuleRepository;
        private readonly IRepository<PageModule> _pageModuleRepository;
        private readonly IRepository<LocalizedPageModule> _localizedPageModuleRepository;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        private const string CacheKey = Consts.GlobalKey + "|PageModules|PageId:{0}|LanguageId:{1}";
        private const string CacheKeyPattern = Consts.GlobalKey + "|PageModules|PageId:{0}";

        #endregion

        #region Constructors

        public ModuleService(IRepository<Module> moduleRepository,
            IRepository<LocalizedModule> localizedModuleRepository,
            IRepository<PageModule> pageModuleRepository,
            IRepository<LocalizedPageModule> localizedPageModuleRepository,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _moduleRepository = moduleRepository;
            _localizedModuleRepository = localizedModuleRepository;
            _pageModuleRepository = pageModuleRepository;
            _localizedPageModuleRepository = localizedPageModuleRepository;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Module entity, List<LocalizedModule> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Module");
            }

            _moduleRepository.Create(entity);

            foreach (var item in localizedEntities)
            {
                item.ModuleId = entity.Id;
                _localizedModuleRepository.Create(item);
            }
        }

        public virtual void CreateForPage(PageModule entity, List<LocalizedPageModule> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Page Module");
            }

            _pageModuleRepository.Create(entity);

            foreach (var item in localizedEntities)
            {
                item.PageModuleId = entity.Id;
                _localizedPageModuleRepository.Create(item);
            }
        }

        public virtual void CreateLocalized(LocalizedModule entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedModule");
            }
            _localizedModuleRepository.Create(entity);
        }

        public virtual void CreateLocalizedForPage(LocalizedPageModule entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedPageModule");
            }
            _localizedPageModuleRepository.Create(entity);
        }

        public virtual void AddNew(Module module,
            List<LocalizedModule> localizedModules,
            PageModule pageModule,
            List<LocalizedPageModule> localizedPageModules)
        {
            if (module == null)
            {
                throw new ArgumentNullException("Module");
            }

            if (pageModule == null)
            {
                throw new ArgumentNullException("PageModule");
            }

            _moduleRepository.Create(module);

            foreach (var item in localizedModules)
            {
                item.ModuleId = module.Id;
                _localizedModuleRepository.Create(item);
            }

            pageModule.ModuleId = module.Id;
            _pageModuleRepository.Create(pageModule);

            foreach (var item in localizedPageModules)
            {
                item.PageModuleId = pageModule.Id;
                _localizedPageModuleRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, pageModule.PageId));
        }

        public virtual void AddExisting(PageModule pageModule, List<LocalizedPageModule> localizedPageModules)
        {
            if (pageModule == null)
            {
                throw new ArgumentNullException("PageModule");
            }

            _pageModuleRepository.Create(pageModule);

            foreach (var item in localizedPageModules)
            {
                item.PageModuleId = pageModule.Id;
                _localizedPageModuleRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, pageModule.PageId));
        }

        public virtual void Update(Module entity, List<LocalizedModule> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Module");
            }

            _moduleRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedModuleRepository.Update(item);
            }
        }

        public virtual void Update(PageModule entity, List<LocalizedPageModule> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PageModule");
            }

            _pageModuleRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedPageModuleRepository.Update(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.PageId));
        }

        public virtual void Delete(Module entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Module");
            }

            _moduleRepository.Delete(entity);
        }

        public virtual void DeleteForPage(PageModule entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PageModule");
            }

            _pageModuleRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.PageId));
        }

        public virtual Module GetById(int siteId, int id)
        {
            var query = from item in _moduleRepository.Table
                        .Include("ModuleDefinition")
                        where item.SiteId == siteId
                        & item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual PageModule GetForPageById(int siteId, int id)
        {
            var query = from item in _pageModuleRepository.Table
                        where item.Module.SiteId == siteId
                        & item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedModule GetLocalizedByModuleId(int moduleId, int languageId)
        {
            var query = from item in _localizedModuleRepository.Table
                        where item.ModuleId == moduleId
                        & item.LanguageId == languageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedModule GetLocalizedById(int Id)
        {
            var query = from item in _localizedModuleRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedPageModule GetLocalizedForPageByPageModuleId(int pageModuleId, int languageId)
        {
            var query = from item in _localizedPageModuleRepository.Table
                        where item.PageModuleId == pageModuleId
                        & item.LanguageId == languageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedPageModule GetLocalizedForPageById(int Id)
        {
            var query = from item in _localizedPageModuleRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<Module> GetList(int siteId)
        {
            var query = from item in _moduleRepository.Table
                        .Include("ModuleDefinition")
                        .Include("PageModules")
                        where item.SiteId == siteId
                        select item;
            return query.ToList();
        }

        public virtual List<Module> GetListForAllPages(int siteId)
        {
            var query = from item in _moduleRepository.Table
                        where item.SiteId == siteId
                        & item.AllPages == true
                        select item;
            return query.ToList();
        }

        public virtual List<PageModule> GetListForModule(int moduleId)
        {
            var query = from item in _pageModuleRepository.Table
                        .Include("Page")
                        where item.ModuleId == moduleId
                        select item;
            return query.ToList();
        }

        public virtual List<PageModule> GetListForPage(int siteId, int pageId)
        {
            var query = from item in _pageModuleRepository.Table
                        .Include("Module.ModuleDefinition")
                        where item.Module.SiteId == siteId
                        & item.PageId == pageId
                        select item;
            return query.ToList();
        }

        public virtual List<LocalizedPageModule> GetLocalizedListForPage(int pageModuleId)
        {
            var query = from item in _localizedPageModuleRepository.Table
                        where item.PageModuleId == pageModuleId
                        select item;
            return query.ToList();
        }

        public virtual List<Module> GetListForExisting(int SiteId, int ExcludePageId)
        {
            List<Module> list = new List<Module>();

            foreach (var module in GetList(SiteId))
            {
                bool exists = false;
                foreach (var item in module.PageModules)
                {
                    if (item.PageId == ExcludePageId)
                    {
                        exists = true;
                        break;
                    }
                }
                if (exists == false)
                {
                    list.Add(module);
                }
            }

            return list;
        }

        public virtual List<PageModule> GetPageModules(int SiteId, int PageId, int LanguageId)
        {
            List<PageModule> list = new List<PageModule>();
            string key = string.Format(CacheKey, PageId, LanguageId);
            object obj = _cacheManager.Get(key);
            if (obj != null)
            {
                list = (List<PageModule>)obj;
            }
            else
            {
                list = _GetPageModules(SiteId, PageId, LanguageId);
                _cacheManager.Insert(key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
            }
            return list;
        }

        #endregion

        #region Private Methods

        private Module _GetById(int id)
        {
            var query = from item in _moduleRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private List<PageModule> _GetPageModules(int siteId, int pageId, int languageId)
        {
            List<PageModule> list = new List<PageModule>();

            var query = from item in _pageModuleRepository.Table
                        .Include("Module.ModuleDefinition")
                        .Include("Module.LocalizedModules")
                        .Include("LocalizedPageModules")
                        where item.Module.SiteId == siteId
                        & item.PageId == pageId
                        orderby item.SortOrder
                        select item;

            list = query.ToList();

            LocalizedModule localizedModule = new LocalizedModule();
            LocalizedPageModule localizedPageModule = new LocalizedPageModule();

            string LocalizedTitle = "";

            foreach (var item in list)
            {
                localizedModule = null;
                localizedPageModule = null;

                LocalizedTitle = "";

                foreach (var lm in item.Module.LocalizedModules)
                {
                    if (lm.LanguageId == languageId)
                    {
                        localizedModule = lm;
                        break;
                    }
                }

                foreach (var lpm in item.LocalizedPageModules)
                {
                    if (lpm.LanguageId == languageId)
                    {
                        localizedPageModule = lpm;
                        break;
                    }
                }

                if (localizedModule == null || localizedPageModule == null)
                {
                    LocalizedTitle = Utils.Choose(item.Title, item.Module.Title);
                }
                else
                {
                    LocalizedTitle = Utils.Choose(localizedPageModule.Title, item.Title, localizedModule.Title, item.Module.Title);
                }

                item.LocalizedTitle = LocalizedTitle;
            }

            return list;
        }

        #endregion

    }

}