#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Caching;
using System.Web.UI.WebControls;
using Weapsy.Core;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{

    public class PageService : IPageService
    {

        #region Private Members

        private readonly IRepository<Page> _pageRepository;
        private readonly IRepository<LocalizedPage> _localizedPageRepository;
        private readonly ISettingService _settingService;
        private readonly ICacheManager _cacheManager;

        private const string CacheKeyPages = Consts.GlobalKey + "|Pages|SiteId:{0}|LanguageId:{1}";
        private const string CacheKeyPagesPattern = Consts.GlobalKey + "|Pages|SiteId:{0}";

        #endregion

        #region Constructors

        public PageService(IRepository<Page> pageRepository,
            IRepository<LocalizedPage> localizedPageRepository,
            ISettingService settingService,
            ICacheManager cacheManager)
        {
            _pageRepository = pageRepository;
            _localizedPageRepository = localizedPageRepository;
            _settingService = settingService;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Page entity, List<LocalizedPage> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Page");
            }

            _pageRepository.Create(entity);

            foreach (var item in localizedEntities)
            {
                item.PageId = entity.Id;
                _localizedPageRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPagesPattern, entity.SiteId));
        }

        public virtual LocalizedPage CreateLocalized(LocalizedPage entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedPage");
            }

            _localizedPageRepository.Create(entity);

            return entity;
        }

        public virtual void Update(Page entity, List<LocalizedPage> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Page");
            }

            _pageRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedPageRepository.Update(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPagesPattern, entity.SiteId));
        }

        public virtual void Delete(Page entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Page");
            }

            _pageRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPagesPattern, entity.SiteId));
        }

        public virtual Page GetById(int SiteId, int PageId)
        {
            return _GetById(SiteId, PageId);
        }

        public virtual Page GetByName(int SiteId, string Name)
        {
            return _GetByName(SiteId, Name);
        }

        public virtual Page GetByUrl(int SiteId, string Url)
        {
            return _GetByUrl(SiteId, Url);
        }

        public virtual LocalizedPage GetLocalizedByPageId(int pageId, int languageId)
        {
            var query = from item in _localizedPageRepository.Table
                        where item.PageId == pageId
                        & item.LanguageId == languageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedPage GetLocalizedById(int id)
        {
            var query = from item in _localizedPageRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<Page> GetList(int siteId)
        {
            var query = from item in _pageRepository.Table
                        where item.SiteId == siteId & item.Deleted == false
                        orderby item.ParentId, item.SortOrder
                        select item;
            return query.ToList();
        }

        public virtual List<Page> GetSitePages(int SiteId, int LanguageId)
        {
            List<Page> list = new List<Page>();
            string key = string.Format(CacheKeyPages, SiteId, LanguageId);
            object obj = _cacheManager.Get(key);
            if (obj != null)
            {
                list = (List<Page>)obj;
            }
            else
            {
                list = _GetSitePages(SiteId, LanguageId);
                _cacheManager.Insert(key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
            }
            return list;
        }

        public virtual List<Page> GetSitePagesForTree(int SiteId, int LanguageId)
        {
            return SortSitePagesForTree(GetSitePages(SiteId, LanguageId), 0);
        }

        public virtual Dictionary<string, string> GetDictionary(int SiteId, int LanguageId)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            foreach (var item in GetSitePagesForTree(SiteId, LanguageId))
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }
            return dic;
        }

        public virtual Dictionary<string, string> GetDictionaryForParent(int SiteId, int LanguageId, int ExcludePageId)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("0", "");
            foreach (var item in GetSitePagesForTree(SiteId, LanguageId))
            {
                if (item.Id != ExcludePageId)
                {
                    dic.Add(item.Id.ToString(), item.Name.ToString());
                }
            }
            return dic;
        }

        public virtual void Move(int SiteId, int Id, string Action)
        {
            Page entity1 = _GetById(SiteId, Id);
            int entity1NewSortOrder;

            Page entity2 = new Page();
            int entity2NewSortOrder;

            if (Action == "UP")
            {
                entity2 = _GetBySortOrder(SiteId, entity1.ParentId, entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(SiteId, entity1.ParentId, entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _pageRepository.Update(entity1);
            _pageRepository.Update(entity2);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPagesPattern, SiteId));
        }

        #endregion
        
        #region Private Methods

        private Page _GetById(int siteId, int id)
        {
            var query = from item in _pageRepository.Table
                        where item.SiteId == siteId
                        & item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private Page _GetByName(int siteId, string name)
        {
            var query = from item in _pageRepository.Table
                        where item.SiteId == siteId
                        & item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private Page _GetByUrl(int siteId, string url)
        {
            var query = from item in _pageRepository.Table
                        where item.SiteId == siteId
                        & item.Url == url
                        select item;
            return query.FirstOrDefault();
        }

        private Page _GetBySortOrder(int siteId, int parentId, int sortOrder)
        {
            var query = from item in _pageRepository.Table
                        where item.SiteId == siteId
                        & item.ParentId == parentId
                        & item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        private List<Page> _GetListByParent(int siteId, int parentId)
        {
            var query = from item in _pageRepository.Table
                        where item.SiteId == siteId
                        & item.ParentId == parentId
                        & item.Deleted == false
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        private List<Page> _GetSitePages(int siteId, int languageId)
        {
            List<Page> list = new List<Page>();

            var query = from item in _pageRepository.Table
                            .Include("LocalizedPages")
                            .Include("PageModules")
                        where item.SiteId == siteId & item.Deleted == false
                        orderby item.ParentId, item.SortOrder
                        select item;

            LocalizedPage LocalizedPage = new LocalizedPage();

            string LocalizedUrl = "";
            string LocalizedPageTitle = "";
            string LocalizedMenuText = "";
            string LocalizedMenuTitle = "";
            string LocalizedMetaTitle = "";
            string LocalizedMetaDescription = "";
            string LocalizedMetaKeywords = "";

            foreach (var item in query.ToList())
            {
                Page sitePage = item;

                LocalizedUrl = "";
                LocalizedPageTitle = "";
                LocalizedMenuText = "";
                LocalizedMenuTitle = "";
                LocalizedMetaTitle = "";
                LocalizedMetaDescription = "";
                LocalizedMetaKeywords = "";

                LocalizedPage = null;

                foreach (var lp in item.LocalizedPages)
                {
                    if (lp.LanguageId == languageId)
                    {
                        LocalizedPage = lp;
                        break;
                    }
                }

                if (LocalizedPage == null)
                {
                    LocalizedUrl = item.Url;
                    LocalizedPageTitle = Utils.Choose(item.PageTitle, item.Name);
                    LocalizedMenuText = Utils.Choose(item.MenuText, item.Name);
                    LocalizedMenuTitle = Utils.Choose(item.MenuTitle, LocalizedMenuText);
                    LocalizedMetaTitle = item.MetaTitle;
                    LocalizedMetaDescription = item.MetaDescription;
                    LocalizedMetaKeywords = item.MetaKeywords;
                }
                else
                {
                    LocalizedUrl = Utils.Choose(LocalizedPage.Url, item.Url);
                    LocalizedPageTitle = Utils.Choose(LocalizedPage.PageTitle, item.PageTitle, item.Name);
                    LocalizedMenuText = Utils.Choose(LocalizedPage.MenuText, item.MenuText, item.Name);
                    LocalizedMenuTitle = Utils.Choose(LocalizedPage.MenuTitle, item.MenuTitle, LocalizedMenuText);
                    LocalizedMetaTitle = Utils.Choose(LocalizedPage.MetaTitle, item.MetaTitle);
                    LocalizedMetaDescription = Utils.Choose(LocalizedPage.MetaDescription, item.MetaDescription);
                    LocalizedMetaKeywords = Utils.Choose(LocalizedPage.MetaKeywords, item.MetaKeywords);
                }

                sitePage.LocalizedUrl = LocalizedUrl;
                sitePage.LocalizedPageTitle = LocalizedPageTitle;
                sitePage.LocalizedMenuText = LocalizedMenuText;
                sitePage.LocalizedMenuTitle = LocalizedMenuTitle;
                sitePage.LocalizedMetaTitle = LocalizedMetaTitle;
                sitePage.LocalizedMetaDescription = LocalizedMetaDescription;
                sitePage.LocalizedMetaKeywords = LocalizedMetaKeywords;

                sitePage.PageModuleCount = item.PageModules.Count;

                list.Add(sitePage);
            }

            return list;
        }

        private static List<Page> SortSitePagesForTree(List<Page> Source, int ParentId)
        {
            List<Page> Result = new List<Page>();

            List<Page> Temp = Source.FindAll(o => o.ParentId == ParentId);

            foreach (Page Page in Temp)
            {
                Page.NameForTree = GetPageNameForTree(Page, Source);
                Result.Add(Page);
                Result.AddRange(SortSitePagesForTree(Source, Page.Id));
            }

            return Result;
        }

        private static string GetPageNameForTree(Page Page, List<Page> Source)
        {
            string Result = string.Empty;
            while (Page != null && !Page.Deleted)
            {
                if (string.IsNullOrEmpty(Result))
                {
                    Result = Page.Name;
                }
                else
                {
                    Result = "---" + Result;
                }
                Page = Source.Find(x => x.Id == Page.ParentId);
            }
            return Result;
        }

        #endregion

    }
}