#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class PluginService : IPluginService
    {

        #region Private Members

        private readonly IRepository<Plugin> _pluginRepository;
        private readonly IRepository<LocalizedPlugin> _localizedPluginRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|Plugins|SiteId:{0}|Type:{1}|LanguageId:{2}";
        private const string CacheKeyPattern = Consts.GlobalKey + "|Plugins|SiteId:{0}";

        #endregion

        #region Constructors

        public PluginService(IRepository<Plugin> pluginRepository,
            IRepository<LocalizedPlugin> localizedPluginRepository,
            ICacheManager cacheManager)
        {
            _pluginRepository = pluginRepository;
            _localizedPluginRepository = localizedPluginRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Plugin entity, List<LocalizedPlugin> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Plugin");
            }

            _pluginRepository.Create(entity);

            foreach (var item in localizedEntities)
            {
                item.PluginId = entity.Id;
                _localizedPluginRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual LocalizedPlugin CreateLocalized(LocalizedPlugin entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedPlugin");
            }

            _localizedPluginRepository.Create(entity);

            return entity;
        }

        public virtual void Update(Plugin entity, List<LocalizedPlugin> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Plugin");
            }

            _pluginRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedPluginRepository.Update(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual void Delete(Plugin entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Plugin");
            }

            foreach (var item in GetLocalizedList(entity.Id))
            {
                _localizedPluginRepository.Delete(item);
            }

            _pluginRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual Plugin GetById(int SiteId, int Id)
        {
            var query = from item in _pluginRepository.Table
                        where item.SiteId == SiteId & item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedPlugin GetLocalizedByPluginId(int PluginId, int LanguageId)
        {
            var query = from item in _localizedPluginRepository.Table
                        where item.PluginId == PluginId
                        & item.LanguageId == LanguageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedPlugin GetLocalizedById(int Id)
        {
            var query = from item in _localizedPluginRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<Plugin> GetList(int siteId)
        {
            var query = from item in _pluginRepository.Table
                        where item.SiteId == siteId
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        public virtual List<Plugin> GetList(int siteId, PluginType pluginType)
        {
            var query = from item in _pluginRepository.Table
                        where item.SiteId == siteId
                        & item.PluginDefinition.TypeId == (byte)pluginType
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        public virtual List<LocalizedPlugin> GetLocalizedList(int pluginId)
        {
            var query = from item in _localizedPluginRepository.Table
                        where item.PluginId == pluginId
                        select item;
            return query.ToList();
        }

        public virtual List<Plugin> GetSitePlugins(int SiteId, PluginType pluginType, int LanguageId)
        {
            List<Plugin> list = new List<Plugin>();
            string key = string.Format(CacheKey, SiteId, (byte)pluginType, LanguageId);
            object obj = _cacheManager.Get(key);
            if (obj != null)
            {
                list = (List<Plugin>)obj;
            }
            else
            {
                list = _GetSitePlugins(SiteId, pluginType, LanguageId);
                _cacheManager.Insert(key, list, 60, CacheItemPriority.Normal);
            }
            return list;
        }

        public void Move(int SiteId, int Id, string Action)
        {
            Plugin entity1 = GetById(SiteId, Id);
            int entity1NewSortOrder;

            Plugin entity2 = new Plugin();
            int entity2NewSortOrder;

            if (Action == "UP")
            {
                entity2 = _GetBySortOrder(SiteId, entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(SiteId, entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _pluginRepository.Update(entity1);
            _pluginRepository.Update(entity2);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, SiteId));
        }

        #endregion

        #region Private Methods

        private Plugin _GetBySortOrder(int siteId, int sortOrder)
        {
            var query = from item in _pluginRepository.Table
                        where item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        private List<Plugin> _GetSitePlugins(int siteId, PluginType pluginType, int languageId)
        {
            List<Plugin> list = new List<Plugin>();

            var query = from item in _pluginRepository.Table
                        .Include("LocalizedPlugins")
                        .Include("PluginDefinition")
                        where item.SiteId == siteId
                        & item.PluginDefinition.TypeId == (byte)pluginType
                        orderby item.SortOrder
                        select item;

            list = query.ToList();

            LocalizedPlugin LocalizedPlugin = new LocalizedPlugin();

            string LocalizedDashboardText = "";
            string LocalizedDashboardTitle = "";
            string LocalizedMenuText = "";
            string LocalizedMenuTitle = "";

            foreach (var item in list)
            {
                LocalizedPlugin = null;

                LocalizedDashboardText = "";
                LocalizedDashboardTitle = "";
                LocalizedMenuText = "";
                LocalizedMenuTitle = "";

                foreach (var lp in item.LocalizedPlugins)
                {
                    if (lp.LanguageId == languageId)
                    {
                        LocalizedPlugin = lp;
                        break;
                    }
                }

                if (LocalizedPlugin == null)
                {
                    LocalizedDashboardText = Utils.Choose(item.DashboardText, item.PluginDefinition.Name);
                    LocalizedDashboardTitle = Utils.Choose(item.DashboardTitle, LocalizedDashboardText);
                    LocalizedMenuText = Utils.Choose(item.MenuText, item.PluginDefinition.Name);
                    LocalizedMenuTitle = Utils.Choose(item.MenuTitle, LocalizedMenuText);
                }
                else
                {
                    LocalizedDashboardText = Utils.Choose(LocalizedPlugin.DashboardText, item.DashboardText, item.PluginDefinition.MenuText);
                    LocalizedDashboardTitle = Utils.Choose(LocalizedPlugin.DashboardTitle, LocalizedDashboardText, item.DashboardTitle);
                    LocalizedMenuText = Utils.Choose(LocalizedPlugin.MenuText, item.MenuText, item.PluginDefinition.MenuText);
                    LocalizedMenuTitle = Utils.Choose(LocalizedPlugin.MenuTitle, LocalizedMenuText, item.MenuTitle);
                }

                item.LocalizedDashboardText = LocalizedDashboardText;
                item.LocalizedDashboardTitle = LocalizedDashboardTitle;
                item.LocalizedMenuText = LocalizedMenuText;
                item.LocalizedMenuTitle = LocalizedMenuTitle;
            }

            return list;
        }

        #endregion

	}

}