#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class SettingService : ISettingService
    {

        #region Private Members

        private readonly IRepository<Setting> _settingRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|Settings";

        #endregion

        #region Constructors

        public SettingService(IRepository<Setting> settingRepository,
            ICacheManager cacheManager)
        {
            _settingRepository = settingRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual Setting Create(Setting entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Setting");
            }
            _settingRepository.Create(entity);
            _cacheManager.Remove(CacheKey);
            return entity;
        }

        public virtual void Update(Setting entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Setting");
            }
            _settingRepository.Update(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Save<T>(T settings) where T : ISettings
        {
            string prefix = typeof(T).Name.Replace("Settings", "") + ".";
            foreach (var prop in typeof(T).GetProperties())
            {
                string name = prefix + prop.Name;
                dynamic value = prop.GetValue(settings, null);
                if (value != null)
                {
                    Set(name, value);
                }
                else
                {
                    Set(name, "");
                }
            }
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Set<T>(string name, T value)
        {
            if (string.IsNullOrWhiteSpace(name))
            {
                throw new ArgumentNullException("Setting Name");
            }

            string str = TypeDescriptor.GetConverter(typeof(T)).ConvertToInvariantString(value);

            Setting setting = GetByName(name, false);

            if (setting == null)
            {
                Setting newSetting = new Setting();
                newSetting.Name = name;
                newSetting.Value = str;
                _settingRepository.Create(newSetting);
            }
            else
            {
                setting.Value = str;
                _settingRepository.Update(setting);
            }            
        }

        public virtual void Delete(Setting entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Setting");
            }
            _settingRepository.Delete(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual Setting GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual Setting GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual Setting GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual Setting GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<Setting> GetList()
        {
            return GetList(true);
        }

        public virtual List<Setting> GetList(bool UseCache)
        {
            List<Setting> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(CacheKey);
                if (obj != null)
                {
                    list = (List<Setting>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(CacheKey, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual T GetSettings<T>() where T : ISettings
        {
            T settings = Activator.CreateInstance<T>();

            foreach (var prop in typeof(T).GetProperties())
            {
                Setting setting = GetByName(typeof(T).Name.Replace("Settings", "") + "." + prop.Name);
                if (setting != null)
                {
                    prop.SetValue(settings, TypeDescriptor.GetConverter(prop.PropertyType).ConvertFromInvariantString(setting.Value), null);
                }
                else
                {
                    string txt = "";
                    switch (prop.PropertyType.ToString().Replace("System.", ""))
                    {
                        case "Int32": txt = "0";
                            break;
                        case "Boolean": txt = "false";
                            break;
                        case "String": txt = "";
                            break;
                    }
                    prop.SetValue(settings, TypeDescriptor.GetConverter(prop.PropertyType).ConvertFromInvariantString(txt), null);
                }
            }

            return settings;
        }

        #endregion

        #region Private Methods

        private Setting _GetById(int id)
        {
            var query = from item in _settingRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private Setting _GetByName(string name)
        {
            var query = from item in _settingRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<Setting> _GetList()
        {
            var query = from item in _settingRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

    }

}