#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Linq;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Collections.Generic;
using System.Web.Configuration;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
	public class SiteService : ISiteService
    {

        #region Private Members

        private readonly IRepository<Site> _siteRepository;
        private readonly IRepository<LocalizedSite> _localizedSiteRepository;
        private readonly ILanguageService _languageService;
        private readonly IPageService _pageService;
        private readonly ICacheManager _cacheManager;

        public const string CacheKey = Consts.GlobalKey + "|Site|Name:{0}";
        public const string CacheKeyLocalized = Consts.GlobalKey + "|LocalizedSite|Id:{0}|LanguageId:{1}";

        #endregion

        #region Constructors

        public SiteService(IRepository<Site> siteRepository,
            IRepository<LocalizedSite> localizedSiteRepository,
            ILanguageService languageService,
            IPageService pageService,
            ICacheManager cacheManager)
        {
            _siteRepository = siteRepository;
            _localizedSiteRepository = localizedSiteRepository;
            _languageService = languageService;
            _pageService = pageService;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Site entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Site");
            }
            _siteRepository.Create(entity);
        }

        public virtual void CreateLocalized(LocalizedSite entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Localized Site");
            }
            _localizedSiteRepository.Create(entity);
        }

        public virtual void Update(Site entity, List<LocalizedSite> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Site");
            }

            _siteRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedSiteRepository.Update(item);
            }

            _cacheManager.Remove(string.Format(CacheKey, entity.Name));
        }

        public virtual void Delete(Site entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Site");
            }

            foreach (var item in _languageService.GetList(entity.Id, false))
            {
                _languageService.Delete(item);
            }

            foreach (var item in _pageService.GetList(entity.Id))
            {
                _pageService.Delete(item);
            }

            _siteRepository.Delete(entity);

            _cacheManager.Clear();
        }

        public virtual Site GetById(int Id)
        {
            var query = from item in _siteRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual Site Get()
        {
            return GetByName(AppSettings.SiteName, true);
        }

        public virtual Site Get(bool UseCache)
        {
            return GetByName(AppSettings.SiteName, UseCache);
        }

        public virtual Site GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual Site GetByName(string Name, bool UseCache)
        {
            Site entity;
            if (UseCache)
            {
                string key = string.Format(CacheKey, Name);
                object obj = _cacheManager.Get(key);
                if (obj != null)
                {
                    entity = (Site)obj;
                }
                else
                {
                    entity = _GetByName(Name);
                    _cacheManager.Insert(key, entity, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                entity = _GetByName(Name);
            }
            return entity;
        }

        public virtual LocalizedSite GetLocalizedBySiteId(int siteId, int languageId)
        {
            return GetLocalizedBySiteId(siteId, languageId, false);
        }

        public virtual LocalizedSite GetLocalizedBySiteId(int siteId, int languageId, bool UseCache)
        {
            LocalizedSite entity;
            if (UseCache)
            {
                string key = string.Format(CacheKeyLocalized, siteId, languageId);
                object obj = _cacheManager.Get(key);
                if (obj != null)
                {
                    entity = (LocalizedSite)obj;
                }
                else
                {
                    entity = _GetLocalizedBySiteId(siteId, languageId);
                    _cacheManager.Insert(key, entity, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                entity = _GetLocalizedBySiteId(siteId, languageId);
            }
            return entity;
        }

        public virtual LocalizedSite GetLocalizedById(int id)
        {
            var query = from item in _localizedSiteRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<Site> GetList()
        {
            var query = from item in _siteRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

        #region Private Methods

        private Site _GetByName(string name)
        {
            var query = from item in _siteRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedSite _GetLocalizedBySiteId(int siteId, int languageId)
        {
            var query = from item in _localizedSiteRepository.Table
                        where item.SiteId == siteId
                        & item.LanguageId == languageId
                        select item;
            return query.FirstOrDefault();
        }

        //private Site _GetSite(string name, int languageId)
        //{
        //    Site site = new Site();

        //    var query = from item in _siteRepository.Table
        //                .Include("LocalizedSites")
        //                where item.Name == name
        //                select item;

        //    site = query.FirstOrDefault();

        //    string LocalizedTitle = "";
        //    string LocalizedMetaTitle = "";
        //    string LocalizedMetaDescription = "";
        //    string LocalizedMetaKeywords = "";

        //    LocalizedSite localizedSite = null;

        //    foreach (var ls in site.LocalizedSites)
        //    {
        //        if (ls.LanguageId == languageId)
        //        {
        //            localizedSite = ls;
        //            break;
        //        }
        //    }

        //    if (localizedSite == null)
        //    {
        //        LocalizedTitle = site.Title;
        //        LocalizedMetaTitle = site.MetaTitle;
        //        LocalizedMetaDescription = site.MetaDescription;
        //        LocalizedMetaKeywords = site.MetaKeywords;
        //    }
        //    else
        //    {
        //        LocalizedTitle = Utils.Choose(localizedSite.Title, site.Title);
        //        LocalizedMetaTitle = Utils.Choose(localizedSite.MetaTitle, site.MetaTitle);
        //        LocalizedMetaDescription = Utils.Choose(localizedSite.MetaDescription, site.MetaDescription);
        //        LocalizedMetaKeywords = Utils.Choose(localizedSite.MetaKeywords, site.MetaKeywords);
        //    }

        //    site.LocalizedTitle = LocalizedTitle;
        //    site.LocalizedMetaTitle = LocalizedMetaTitle;
        //    site.LocalizedMetaDescription = LocalizedMetaDescription;
        //    site.LocalizedMetaKeywords = LocalizedMetaKeywords;

        //    return site;
        //}

        #endregion

	}
}