#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class TextService : ITextService
    {

        #region Private Members

        private readonly IRepository<Text> _textRepository;
        private readonly IRepository<TextVersion> _textVersionRepository;
        private readonly IRepository<LocalizedTextVersion> _localizedTextVersionRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|Text|ModuleId:{0}|LanguageId:{1}";
        private const string CacheKeyPattern = Consts.GlobalKey + "|Text|ModuleId:{0}";

        #endregion

        #region Constructors

        public TextService(IRepository<Text> textRepository,
            IRepository<TextVersion> textVersionRepository,
            IRepository<LocalizedTextVersion> localizedTextVersionRepository,
            ICacheManager cacheManager)
        {
            _textRepository = textRepository;
            _textVersionRepository = textVersionRepository;
            _localizedTextVersionRepository = localizedTextVersionRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Text text, TextVersion textVersion, List<LocalizedTextVersion> localizedTextVersions)
        {
            if (text == null)
            {
                throw new ArgumentNullException("Text");
            }

            if (textVersion == null)
            {
                throw new ArgumentNullException("TextVersion");
            }

            _textRepository.Create(text);

            textVersion.TextId = text.Id;

            _textVersionRepository.Create(textVersion);

            foreach (var item in localizedTextVersions)
            {
                //item.TextId = text.Id;
                item.TextVersionId = textVersion.Id;
                _localizedTextVersionRepository.Create(item);
            }

            Text textToUpdate = GetById(text.Id);

            textToUpdate.TextVersionId = textVersion.Id;

            Update(textToUpdate);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, text.ModuleId));
        }

        public virtual TextVersion CreateVersion(TextVersion textVersion, List<LocalizedTextVersion> localizedTextVersions, int moduleId)
        {
            if (textVersion == null)
            {
                throw new ArgumentNullException("TextVersion");
            }

            _textVersionRepository.Create(textVersion);

            foreach (var item in localizedTextVersions)
            {
                item.TextVersionId = textVersion.Id;
                _localizedTextVersionRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, moduleId));

            return textVersion;
        }

        public virtual LocalizedTextVersion CreateLocalizedVersion(LocalizedTextVersion entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedTextVersion");
            }

            _localizedTextVersionRepository.Create(entity);

            return entity;
        }

        public virtual void Update(Text entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Text");
            }

            _textRepository.Update(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.ModuleId));
        }

        public virtual void UpdateVersion(TextVersion entity, List<LocalizedTextVersion> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("TextVersion");
            }

            _textVersionRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedTextVersionRepository.Update(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.Text.ModuleId));
        }

        public virtual void Delete(Text entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Text");
            }

            _textRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.ModuleId));
        }

        public virtual void Delete(TextVersion entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("TextVersion");
            }

            _textVersionRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.Text.ModuleId));
        }

        public virtual Text GetById(int id)
        {
            var query = from item in _textRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual Text GetByModuleId(int siteId, int moduleId)
        {
            var query = from item in _textRepository.Table
                        where item.Module.SiteId == siteId
                        & item.ModuleId == moduleId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual TextVersion GetVersionById(int id)
        {
            var query = from item in _textVersionRepository.Table
                        .Include("Text")
                        .Include("LocalizedTextVersions")
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedTextVersion GetLocalizedVersionByVersionId(int versionId, int languageId)
        {
            var query = from item in _localizedTextVersionRepository.Table
                        where item.TextVersionId == versionId
                        & item.LanguageId == languageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedTextVersion GetLocalizedVersionById(int id)
        {
            var query = from item in _localizedTextVersionRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual string GetText(int SiteId, int ModuleId, int LanguageId)
        {
            string result;
            string key = string.Format(CacheKey, ModuleId, LanguageId);
            object obj = _cacheManager.Get(key);
            if (obj != null)
            {
                result = (string)obj;
            }
            else
            {
                result = _GetText(SiteId, ModuleId, LanguageId);
                _cacheManager.Insert(key, result, 60, CacheItemPriority.Normal);
            }
            return result;
        }

        public virtual List<Text> GetList(int siteId)
        {
            var query = from item in _textRepository.Table
                        where item.Module.SiteId == siteId
                        select item;
            return query.ToList();
        }

        public virtual List<TextVersion> GetVersionList(int textId)
        {
            var query = from item in _textVersionRepository.Table
                        where item.TextId == textId
                        orderby item.CreatedOnDate descending
                        select item;
            return query.ToList();
        }

        #endregion

        #region Private Methods

        private string _GetText(int SiteId, int ModuleId, int LanguageId)
        {
            Text text = GetByModuleId(SiteId, ModuleId);

            if (text != null)
            {
                LocalizedTextVersion ltv = new LocalizedTextVersion();

                string LocalizedBody = "";

                TextVersion version = GetVersionById(text.TextVersionId);

                if (version != null)
                {
                    foreach (var item in version.LocalizedTextVersions)
                    {
                        if (item.LanguageId == LanguageId)
                        {
                            ltv = item;
                            break;
                        }
                    }

                    if (ltv == null)
                    {
                        LocalizedBody = version.Body;
                    }
                    else
                    {
                        LocalizedBody = Utils.Choose(ltv.Body, version.Body);
                    }

                    return LocalizedBody;
                }
                else
                {
                    return string.Empty;
                }
            }
            else
            {
                return string.Empty;
            }
        }

        #endregion

    }

}