#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class ThemeService : IThemeService
    {

        #region Private Members

        private readonly IRepository<Theme> _themeRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|Theme|Id:{0}";

        #endregion

        #region Constructors

        public ThemeService(IRepository<Theme> themeRepository,
            ICacheManager cacheManager)
        {
            _themeRepository = themeRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Theme entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Theme");
            }
            _themeRepository.Create(entity);
        }

        public virtual void Update(Theme entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Theme");
            }
            _themeRepository.Update(entity);
            _cacheManager.Remove(string.Format(CacheKey, entity.Id));
        }

        public virtual void Delete(Theme entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Theme");
            }
            _themeRepository.Delete(entity);
            _cacheManager.Remove(string.Format(CacheKey, entity.Id));
        }

        public virtual Theme GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual Theme GetById(int Id, bool UseCache)
        {
            Theme entity;
            if (UseCache)
            {
                string key = string.Format(CacheKey, Id);
                object obj = _cacheManager.Get(key);
                if (obj != null)
                {
                    entity = (Theme)obj;
                }
                else
                {
                    entity = _GetById(Id);
                    _cacheManager.Insert(key, entity, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                entity = _GetById(Id);
            }
            return entity;
        }

        public virtual Theme GetByName(string Name)
        {
            var query = from item in _themeRepository.Table
                        where item.Name == Name
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<Theme> GetList()
        {
            var query = from item in _themeRepository.Table
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        public virtual Dictionary<string, string> GetDictionary(bool includeInherit)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            if (includeInherit)
            {
                dic.Add(Consts.NullId.ToString(), "<Inherit>");
            }
            foreach (var item in GetList())
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }
            return dic;
        }

        public void Move(int Id, string Action)
        {
            Theme entity1 = _GetById(Id);
            int entity1NewSortOrder;

            Theme entity2 = new Theme();
            int entity2NewSortOrder;

            if (Action == "UP")
            {
                entity2 = _GetBySortOrder(entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _themeRepository.Update(entity1);
            _themeRepository.Update(entity2);

            _cacheManager.RemoveByPattern(string.Format(CacheKey, entity1.Id));
            _cacheManager.RemoveByPattern(string.Format(CacheKey, entity2.Id));
        }

        #endregion

        #region Private Methods

        private Theme _GetById(int id)
        {
            var query = from item in _themeRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private Theme _GetBySortOrder(int sortOrder)
        {
            var query = from item in _themeRepository.Table
                        where item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        #endregion

    }

}