#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class WidgetService : IWidgetService
    {

        #region Private Members

        private readonly IRepository<Widget> _pluginRepository;
        private readonly IRepository<LocalizedWidget> _localizedWidgetRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|Widgets|SiteId:{0}|Type:{1}|LanguageId:{2}";
        private const string CacheKeyPattern = Consts.GlobalKey + "|Widgets|SiteId:{0}";

        #endregion

        #region Constructors

        public WidgetService(IRepository<Widget> pluginRepository,
            IRepository<LocalizedWidget> localizedWidgetRepository,
            ICacheManager cacheManager)
        {
            _pluginRepository = pluginRepository;
            _localizedWidgetRepository = localizedWidgetRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(Widget entity, List<LocalizedWidget> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Widget");
            }

            _pluginRepository.Create(entity);

            foreach (var item in localizedEntities)
            {
                item.WidgetId = entity.Id;
                _localizedWidgetRepository.Create(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual LocalizedWidget CreateLocalized(LocalizedWidget entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("LocalizedWidget");
            }

            _localizedWidgetRepository.Create(entity);

            return entity;
        }

        public virtual void Update(Widget entity, List<LocalizedWidget> localizedEntities)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Widget");
            }

            _pluginRepository.Update(entity);

            foreach (var item in localizedEntities)
            {
                _localizedWidgetRepository.Update(item);
            }

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual void Delete(Widget entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Widget");
            }

            _pluginRepository.Delete(entity);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, entity.SiteId));
        }

        public virtual Widget GetById(int SiteId, int Id)
        {
            var query = from item in _pluginRepository.Table
                        where item.SiteId == SiteId & item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedWidget GetLocalizedByWidgetId(int WidgetId, int LanguageId)
        {
            var query = from item in _localizedWidgetRepository.Table
                        where item.WidgetId == WidgetId
                        & item.LanguageId == LanguageId
                        select item;
            return query.FirstOrDefault();
        }

        public virtual LocalizedWidget GetLocalizedById(int Id)
        {
            var query = from item in _localizedWidgetRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<Widget> GetList(int siteId)
        {
            var query = from item in _pluginRepository.Table
                        where item.SiteId == siteId
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        public virtual List<Widget> GetList(int siteId, WidgetType widgetType)
        {
            var query = from item in _pluginRepository.Table
                        where item.SiteId == siteId
                        & item.WidgetDefinition.TypeId == (byte)widgetType
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        public virtual List<LocalizedWidget> GetLocalizedList(int widgetId)
        {
            var query = from item in _localizedWidgetRepository.Table
                        where item.WidgetId == widgetId
                        select item;
            return query.ToList();
        }

        public virtual List<Widget> GetSiteWidgets(int SiteId, WidgetType widgetType, int LanguageId)
        {
            List<Widget> list = new List<Widget>();
            string key = string.Format(CacheKey, SiteId, (byte)widgetType, LanguageId);
            object obj = _cacheManager.Get(key);
            if (obj != null)
            {
                list = (List<Widget>)obj;
            }
            else
            {
                list = _GetSiteWidgets(SiteId, widgetType, LanguageId);
                _cacheManager.Insert(key, list, 60, CacheItemPriority.Normal);
            }
            return list;
        }

        public void Move(int SiteId, int Id, string Action)
        {
            Widget entity1 = GetById(SiteId, Id);
            int entity1NewSortOrder;

            Widget entity2 = new Widget();
            int entity2NewSortOrder;

            if (Action == "UP")
            {
                entity2 = _GetBySortOrder(SiteId, entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(SiteId, entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _pluginRepository.Update(entity1);
            _pluginRepository.Update(entity2);

            _cacheManager.RemoveByPattern(string.Format(CacheKeyPattern, SiteId));
        }

        #endregion

        #region Private Methods

        private Widget _GetBySortOrder(int siteId, int sortOrder)
        {
            var query = from item in _pluginRepository.Table
                        where item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        private List<Widget> _GetSiteWidgets(int siteId, WidgetType widgetType, int languageId)
        {
            List<Widget> list = new List<Widget>();

            var query = from item in _pluginRepository.Table
                        .Include("LocalizedWidgets")
                        where item.SiteId == siteId
                        & item.WidgetDefinition.TypeId == (byte)widgetType
                        orderby item.SortOrder
                        select item;

            list = query.ToList();

            LocalizedWidget LocalizedWidget = new LocalizedWidget();

            string LocalizedTitle = "";

            foreach (var item in list)
            {
                LocalizedWidget = null;

                LocalizedTitle = "";

                foreach (var lp in item.LocalizedWidgets)
                {
                    if (lp.LanguageId == languageId)
                    {
                        LocalizedWidget = lp;
                        break;
                    }
                }

                if (LocalizedWidget == null)
                {
                    LocalizedTitle = Utils.Choose(item.Title, item.WidgetDefinition.Name);
                }
                else
                {
                    LocalizedTitle = Utils.Choose(LocalizedWidget.Title, item.Title, item.WidgetDefinition.Title);
                }

                item.LocalizedTitle = LocalizedTitle;
            }

            return list;
        }

        #endregion

	}

}