﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using System.Web;
using Xunit;
using Moq;
using Weapsy.Services;
using Weapsy.Entities;
using Weapsy.Data;
using Weapsy.Core.Caching;
using System.Linq;

namespace Weapsy.Tests.Services
{
    public class AppServiceTest : IDisposable
    {
        private Mock<IRepository<App>> mockAppRepository;
        private Mock<ICacheManager> mockCacheManager;
        private Mock<ISettingService> mockSettingService;

        public AppServiceTest()
        {
            mockAppRepository = new Mock<IRepository<App>>();
            mockCacheManager = new Mock<ICacheManager>();
            mockSettingService = new Mock<ISettingService>();
        }

        [Fact]
        public void AppServiceConstructorTest()
        {
            var result = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);

            Assert.NotNull(result);
            Assert.IsType<AppService>(result);
        }

        [Fact]
        public void Create_GivenApp_ReturnsApp()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            App entity = new App();

            var result = target.Create(entity) as App;

            mockAppRepository.Verify(x => x.Create(It.IsAny<App>()), Times.Once());
            Assert.NotNull(result);
            Assert.IsType<App>(result);
        }

        [Fact]
        public void Delete_DeletesAppFromRepository()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            App entity = new App();

            target.Delete(entity);

            mockAppRepository.Verify(x => x.Delete(It.IsAny<App>()), Times.Once());
        }


        [Fact]
        public void GetById_UsingCacheByDefault_ReturnsApp()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetAppListStub());
            int Id = 1;

            var result = target.GetById(Id) as App;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetById_NotUsingCache_ReturnsApp()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockAppRepository.SetupGet<IQueryable<App>>(x => x.Table).Returns(Stubs.GetAppListStub().AsQueryable<App>());
            int Id = 1;

            var result = target.GetById(Id, false) as App;

            Assert.Equal(Id, result.Id);
        }

        [Fact]
        public void GetById_IdDoesntExist_ReturnsNull()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockAppRepository.SetupGet<IQueryable<App>>(x => x.Table).Returns(Stubs.GetAppListStub().AsQueryable<App>());
            int Id = 0;

            var result = target.GetById(Id, false) as App;

            Assert.Null(result);
        }

        [Fact]
        public void GetByName_NotUsingCache_ReturnsApp()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockAppRepository.SetupGet<IQueryable<App>>(x => x.Table).Returns(Stubs.GetAppListStub().AsQueryable<App>());
            string Name = "TestApp1";

            var result = target.GetByName(Name, false) as App;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByName_UsingCacheByDefault_ReturnsApp()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetAppListStub());
            string Name = "TestApp1";

            var result = target.GetByName(Name) as App;

            Assert.Equal(1, result.Id);
        }


        [Fact]
        public void GetDictionary_ReturnsDictionaryOfIdNamePairs()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetAppListStub());

            var result = target.GetDictionary() as Dictionary<string, string>;

            Assert.Equal("TestApp1", result["1"]);
        }


        [Fact]
        public void GetList_NotUsingCache_ReturnsListOfApps()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockAppRepository.SetupGet<IQueryable<App>>(x => x.Table).Returns(Stubs.GetAppListStub().AsQueryable<App>());

            var result = target.GetList(false) as List<App>;
            App resultApp = result.Find(a => a.Id == 1);

            Assert.Equal("TestApp1", resultApp.Name);
        }


        [Fact]
        public void GetList_UsingCacheByDefault_ReturnsListOfApps()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetAppListStub());

            var result = target.GetList() as List<App>;
            App resultApp = result.Find(a => a.Id == 1);

            Assert.Equal("TestApp1", resultApp.Name);
        }

        [Fact]
        public void Update_GivenAnyApp_UpdatesRepository()
        {
            AppService target = new AppService(mockAppRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            App entity = new App();

            target.Update(entity);

            mockAppRepository.Verify(d => d.Update(It.IsAny<App>()), Times.Once());
        }

        public void Dispose()
        {
            // cleanup any resources
        }
    }
}
