﻿using Weapsy.Services;
using System;
using Weapsy.Entities;
using Weapsy.Core.Caching;
using System.Collections.Generic;
using Xunit;
using Moq;
using System.Linq;
using Weapsy.Data;

namespace Weapsy.Tests.Services
{
    public class EmailAccountServiceTest : IDisposable
    {
        private Mock<IRepository<EmailAccount>> mockEmailAccountRepository;
        private Mock<ICacheManager> mockCacheManager;
        private Mock<ISettingService> mockSettingService;

        public EmailAccountServiceTest()
        {
            mockEmailAccountRepository = new Mock<IRepository<EmailAccount>>();
            mockCacheManager = new Mock<ICacheManager>();
            mockSettingService = new Mock<ISettingService>();
        }

        [Fact]
        public void Create_GivenEmailAccount_CreatesEmailAccountInRepository()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            EmailAccount entity = new EmailAccount();

            target.Create(entity);

            mockEmailAccountRepository.Verify(x => x.Create(It.IsAny<EmailAccount>()), Times.Once());
        }

        [Fact]
        public void Delete_GivenEmailAccount_DeletesEmailAccountFromRepository()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            EmailAccount entity = new EmailAccount();

            target.Delete(entity);

            mockEmailAccountRepository.Verify(x => x.Delete(It.IsAny<EmailAccount>()), Times.Once());
        }

        [Fact]
        public void GetById_NotUsingCache_ReturnsEmailAccount()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockEmailAccountRepository.SetupGet<IQueryable<EmailAccount>>(x => x.Table)
                .Returns(Stubs.GetEmailAccountListStub().AsQueryable<EmailAccount>());

            var result = target.GetById(1, 1, false) as EmailAccount;

            Assert.Equal("test1@test.com", result.Address);
        }

        [Fact]
        public void GetById_UsingCacheByDefault_ReturnsEmailAccount()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetEmailAccountListStub());

            var result = target.GetById(1, 1) as EmailAccount;

            Assert.Equal("test1@test.com", result.Address);
        }

        [Fact]
        public void GetById_IdDoesntExist_ReturnsNull()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetEmailAccountListStub());

            var result = target.GetById(1, 0) as EmailAccount;

            Assert.Null(result);
        }

        [Fact]
        public void GetList_UsingCacheByDefault_ReturnsListOfEmailAccounts()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetEmailAccountListStub());

            List<EmailAccount> actual = target.GetList(1);

            Assert.Equal(3, actual.Count);
        }

        [Fact]
        public void GetList_NotUsingCache_ReturnsListOfEmailAccounts()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockEmailAccountRepository.SetupGet<IQueryable<EmailAccount>>(x => x.Table)
                .Returns(Stubs.GetEmailAccountListStub().AsQueryable<EmailAccount>());

            var result = target.GetList(1, false) as List<EmailAccount>;

            Assert.NotNull(result);
            Assert.Equal(3, result.Count);
        }

        [Fact]
        public void Update_GivenEmailAccount_UpdatesRepository()
        {
            EmailAccountService target = new EmailAccountService(mockEmailAccountRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            EmailAccount entity = new EmailAccount();

            target.Update(entity);

            mockEmailAccountRepository.Verify(x => x.Update(It.IsAny<EmailAccount>()), Times.Once());
        }

        public void Dispose()
        {
            // cleanup resources
        }
    }
}
