﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Weapsy.Services;
using Moq;
using Weapsy.Entities;
using Weapsy.Data;
using Weapsy.Core.Caching;
using Xunit;

namespace Weapsy.Tests.Services
{
    public class LanguageServiceTest : IDisposable
    {
        private Mock<IRepository<Language>> mockLanguageRepository;
        private Mock<ICacheManager> mockCacheManager;
        private Mock<ISettingService> mockSettingService;

        public LanguageServiceTest()
        {
            mockLanguageRepository = new Mock<IRepository<Language>>();
            mockCacheManager = new Mock<ICacheManager>();
            mockSettingService = new Mock<ISettingService>();
        }

        [Fact]
        public void Create_GivenLanguage_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            Language entity = new Language();

            var result = target.Create(entity) as Language;

            mockLanguageRepository.Verify(x => x.Create(It.IsAny<Language>()), Times.Once());
            Assert.NotNull(result);
            Assert.IsType<Language>(result);
        }

        [Fact]
        public void Update_GivenLanguage_UpdatesRepository()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            Language entity = new Language();

            target.Update(entity);

            mockLanguageRepository.Verify(d => d.Update(It.IsAny<Language>()), Times.Once());
        }

        [Fact]
        public void Delete_GivenLanguage_DeletesLanguageFromRepository()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            Language entity = new Language();

            target.Delete(entity);

            mockLanguageRepository.Verify(d => d.Delete(It.IsAny<Language>()), Times.Once());
        }

        [Fact]
        public void GetById_NotUsingCache_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockLanguageRepository.SetupGet<IQueryable<Language>>(x => x.Table).Returns(Stubs.GetLanguageListStub().AsQueryable<Language>());
            int id = 1;

            var result = target.GetById(1, id, false) as Language;

            Assert.Equal("English", result.Name);
        }

        [Fact]
        public void GetById_UsingCacheByDefault_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetLanguageListStub());
            int id = 2;

            var result = target.GetById(1, id) as Language;

            Assert.Equal("Spanish", result.Name);
        }

        [Fact]
        public void GetByName_NotUsingCache_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockLanguageRepository.SetupGet<IQueryable<Language>>(x => x.Table).Returns(Stubs.GetLanguageListStub().AsQueryable<Language>());
            string Name = "English";

            var result = target.GetByName(1, Name, false) as Language;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByName_UsingCacheByDefault_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetLanguageListStub());
            string Name = "Spanish";

            var result = target.GetByName(1, Name) as Language;

            Assert.Equal(2, result.Id);
        }

        [Fact]
        public void GetByCultureName_NotUsingCache_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockLanguageRepository.SetupGet<IQueryable<Language>>(x => x.Table).Returns(Stubs.GetLanguageListStub().AsQueryable<Language>());
            string cultureName = "en";

            var result = target.GetByCultureName(1, cultureName, false) as Language;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByCultureName_UsingCacheByDefault_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetLanguageListStub());
            string cultureName = "es";

            var result = target.GetByCultureName(1, cultureName) as Language;

            Assert.Equal(2, result.Id);
        }

        [Fact]
        public void GetByUrl_NotUsingCache_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockLanguageRepository.SetupGet<IQueryable<Language>>(x => x.Table).Returns(Stubs.GetLanguageListStub().AsQueryable<Language>());
            string url = "weapsy.com/en";

            var result = target.GetByUrl(1, url, false) as Language;

            Assert.Equal(1, result.Id);
        }

        [Fact]
        public void GetByUrl_UsingCacheByDefault_ReturnsLanguage()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetLanguageListStub());
            string url = "weapsy.com/es";

            var result = target.GetByUrl(1, url) as Language;

            Assert.Equal(2, result.Id);
        }

        [Fact]
        public void Move_ActionUp_DecrementsSortOrderOfLanguageAndUpdatesRepositoryTwice()
        {
            LanguageService target = new LanguageService(mockLanguageRepository.Object, mockCacheManager.Object, mockSettingService.Object);
            mockCacheManager.Setup(x => x.Get(It.IsAny<string>())).Returns(Stubs.GetLanguageListStub());
            int id = 2;
            string action = "UP";
            Language language = target.GetById(1, id);
            int sortOrder = language.SortOrder;
            mockLanguageRepository.Setup(x => x.Update(It.Is<Language>(l => l.Id == id))).Callback(() => sortOrder--);
            mockLanguageRepository.SetupGet<IQueryable<Language>>(x => x.Table).Returns(Stubs.GetLanguageListStub().AsQueryable<Language>());

            target.Move(1, id, action);
            
            Assert.Equal(language.SortOrder - 1, sortOrder);
            mockLanguageRepository.Verify(x => x.Update(It.IsAny<Language>()), Times.Exactly(2));
        }

        public void Dispose()
        {
            // clean up resources
        }
    }
}
