#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class MailTemplateController : BaseAdminController
    {

        #region Private Members

        private readonly IAppService _appService;
        private readonly ILanguageService _languageService;
        private readonly IMailTemplateService _mailTemplateService;

        #endregion

        #region Constructors

        public MailTemplateController(IAppService appService,
            ILanguageService languageService,
            IMailTemplateService mailTemplateService)
        {
            _appService = appService;
            _languageService = languageService;
            _mailTemplateService = mailTemplateService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            ViewBag.LanguageId = WeapsyGlobal.Language.Id;

            return View(new List<MailTemplateModel>());
        }

        public PartialViewResult Grid(int languageId)
        {
            if (languageId == 0) 
            {
                languageId = WeapsyGlobal.Language.Id;
            }

            return PartialView("_Grid", _mailTemplateService.GetSiteMailTemplates(SiteId, languageId).ToModels());
        }

        public ActionResult Create()
        {
            List<LocalizedMailTemplateModel> localizedMailTemplateModels = new List<LocalizedMailTemplateModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedMailTemplateModel lm = new LocalizedMailTemplateModel();
                lm.LanguageId = Language.Id;
                lm.LanguageName = Language.Name;
                localizedMailTemplateModels.Add(lm);
            }

            MailTemplateModel model = new MailTemplateModel();
            model.LocalizedMailTemplateModels = localizedMailTemplateModels;

            return View(model);
        }

        [ValidateInput(false)]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(MailTemplateModel model)
        {
            if (ModelState.IsValid)
            {
                MailTemplate entity = model.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                _mailTemplateService.Create(entity, model.LocalizedMailTemplateModels.ToEntities());

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            MailTemplate entity = _mailTemplateService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Not Found");
            }

            List<LocalizedMailTemplateModel> localizedMailTemplateModels = new List<LocalizedMailTemplateModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedMailTemplateModel lpm = _mailTemplateService.GetLocalizedByMailTemplateId(entity.Id, Language.Id).ToModel();
                if (lpm == null)
                {
                    lpm = new LocalizedMailTemplateModel();
                    lpm.MailTemplateId = id;
                    lpm.LanguageId = Language.Id;
                    _mailTemplateService.CreateLocalized(lpm.ToEntity());
                    lpm = _mailTemplateService.GetLocalizedByMailTemplateId(entity.Id, Language.Id).ToModel();
                    lpm.LanguageName = Language.Name;
                }
                localizedMailTemplateModels.Add(lpm);
            }

            MailTemplateModel model = entity.ToModel();
            model.LocalizedMailTemplateModels = localizedMailTemplateModels;

            string tokenPath = "";

            if (model.MailTemplateTypeAppId > 0)
            {
                tokenPath = "/" 
                    + _appService.GetById(model.MailTemplateTypeAppId).Folder
                    + "/" + model.MailTemplateTypeTokenController
                    + "/" + model.MailTemplateTypeTokenAction;
            }
            else
            {
                tokenPath = "/"
                    + "Admin"
                    + "/" + model.MailTemplateTypeTokenController
                    + "/" + model.MailTemplateTypeTokenAction;
            }

            model.TokenPath = tokenPath;

            return View(model);
        }

        [HttpParamAction]
        [ValidateInput(false)]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(MailTemplateModel model)
        {
            MailTemplate entity = _mailTemplateService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Not Found");
            }

            if (ModelState.IsValid)
            {
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                List<LocalizedMailTemplate> localizedMailTemplateEntities = new List<LocalizedMailTemplate>();

                foreach (var item in model.LocalizedMailTemplateModels)
                {
                    LocalizedMailTemplate lp = _mailTemplateService.GetLocalizedById(item.Id);
                    lp = item.ToEntity(lp);
                    localizedMailTemplateEntities.Add(lp);
                }

                _mailTemplateService.Update(entity, localizedMailTemplateEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [ValidateInput(false)]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Copy(MailTemplateModel model)
        {
            MailTemplate entity = _mailTemplateService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Not Found");
            }

            if (ModelState.IsValid)
            {
                MailTemplate newEntity = new MailTemplate();

                newEntity.SiteId = SiteId;
                newEntity.MailTemplateTypeId = entity.MailTemplateTypeId;
                newEntity.Name = entity.Name + " - Copy";
                newEntity.Description = entity.Description;
                newEntity.Subject = entity.Subject;
                newEntity.BodyHtml = entity.BodyHtml;
                newEntity.BodyPlain = entity.BodyPlain;
                newEntity.SetAuditForCreate();

                List<LocalizedMailTemplate> newLocalizedEntities = new List<LocalizedMailTemplate>();

                foreach (var item in model.LocalizedMailTemplateModels)
                {
                    LocalizedMailTemplate newLocalizedEntity = new LocalizedMailTemplate();
                    newLocalizedEntity.LanguageId = item.LanguageId;
                    newLocalizedEntity.Subject = item.Subject;
                    newLocalizedEntity.BodyHtml = item.BodyHtml;
                    newLocalizedEntity.BodyPlain = item.BodyPlain;
                    newLocalizedEntities.Add(newLocalizedEntity);
                }

                _mailTemplateService.Create(newEntity, newLocalizedEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [ValidateInput(false)]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(MailTemplateModel model)
        {
            MailTemplate entity = _mailTemplateService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Not Found");
            }

            if (ModelState.IsValid)
            {
                _mailTemplateService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        public PartialViewResult RegisterTokens()
        {
            return PartialView("_RegisterTokens");
        }

        public PartialViewResult PasswordRecoveryTokens()
        {
            return PartialView("_PasswordRecoveryTokens");
        }

        #endregion
    
    }

}