#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{
    [Authorize(Roles="SuperAdmin")]
	public class ModuleController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;
        private readonly IModuleService _moduleService;
        private readonly IModuleTemplateService _moduleTemplateService;
        private readonly IPageService _pageService;
        private readonly IPageTemplateService _pageTemplateService;
        private readonly IRoleService _roleService;

        #endregion

        #region Constructors

        public ModuleController(ILanguageService languageService,
            IModuleService moduleService,
            IModuleTemplateService moduleTemplateService,
            IPageService pageService,
            IPageTemplateService pageTemplateService,
            IRoleService roleService)
        {
            _languageService = languageService;
            _moduleService = moduleService;
            _moduleTemplateService = moduleTemplateService;
            _pageService = pageService;
            _pageTemplateService = pageTemplateService;
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(new List<ModuleModel>());
        }

        public PartialViewResult Grid(int id, string action)
        {
            if (id > 0 && action != "")
            {
                //BusinessLogic.Modules.Move(id, action);
            }

            List<ModuleModel> model = new List<ModuleModel>();

            foreach (var item in _moduleService.GetList(SiteId))
            { 
                ModuleModel m = item.ToModel();
                m.PageModuleCount = item.PageModules.Count;
                model.Add(m);
            }

            return PartialView("_Grid", model);
        }

        public ActionResult Create()
        {
            ModuleSettingsModel model = new ModuleSettingsModel();

            model.LocalizedModules = GetLocalizedModules();

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(ModuleSettingsModel model)
        {
            if (ModelState.IsValid)
            {
                model.SharedSettings.Authorized = GetPermissions();

                Module entity = model.SharedSettings.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                List<LocalizedModule> localizedEntities = model.LocalizedModules.ToEntities();
                foreach (var item in localizedEntities)
                {
                    //item.SiteId = SiteId;
                }

                _moduleService.Create(entity, localizedEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Module entity = _moduleService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Module Not Found");
            }

            ModuleSettingsModel model = new ModuleSettingsModel();

            model.SharedSettings = entity.ToModel();
            model.LocalizedModules = GetLocalizedModules(model.SharedSettings.Id);

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(ModuleSettingsModel model)
        {
            Module entity = _moduleService.GetById(SiteId, model.SharedSettings.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Module Not Found");
            }

            if (ModelState.IsValid)
            {
                model.SharedSettings.Authorized = GetPermissions();

                entity = model.SharedSettings.ToEntity(entity);
                entity.SetAuditForUpdate();

                List<LocalizedModule> localizedEntities = new List<LocalizedModule>();

                foreach (var item in model.LocalizedModules)
                {
                    LocalizedModule lm = _moduleService.GetLocalizedById(item.Id);
                    lm = item.ToEntity(lm);
                    localizedEntities.Add(lm);
                }

                _moduleService.Update(entity, localizedEntities);

                return RedirectToAction("List");            
            }

            return View(model);
        }

        public ActionResult ModulePages(int id)
        {
            Module entity = _moduleService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Module Not Found");
            }

            ModulePagesModel model = new ModulePagesModel();

            model.Module = entity.ToModel();
            model.PageModules = _moduleService.GetListForModule(entity.Id).ToModels();
                
            return View(model);
        }

        public ActionResult PageModules(int id)
        {
            Page entity = _pageService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            List<PageModuleModel> pageModuleModels = new List<PageModuleModel>();

            foreach (var item in _moduleService.GetListForPage(SiteId, entity.Id))
            {
                PageModuleModel pmm = item.ToModel();

                if (item.ModuleTemplateId > 0)
                {
                    pmm.ModuleTemplateName = _moduleTemplateService.GetById(item.ModuleTemplateId).Name;
                }

                pageModuleModels.Add(pmm);
            }

            PageModulesModel model = new PageModulesModel();

            model.Page = entity.ToModel();
            model.PageModules = pageModuleModels;

            return View(model);
        }

        public ActionResult AddNew(int id)
        {
            Page entity = _pageService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            ModuleSettingsModel model = new ModuleSettingsModel();

            model.PageId = entity.Id;
            model.PageName = entity.Name;
            model.SharedSettings = new ModuleModel();
            model.PageSettings = new PageModuleModel();
            model.Zones = GetZones(entity);
            model.LocalizedModules = GetLocalizedModules();
            model.LocalizedPageModules = GetLocalizedPageModules();

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult AddNew(ModuleSettingsModel model)
        {
            if (ModelState.IsValid)
            { 
                model.SharedSettings.Authorized = GetPermissions();

                Module moduleEntity = model.SharedSettings.ToEntity();
                moduleEntity.SiteId = SiteId;
                moduleEntity.SetAuditForCreate();

                List<LocalizedModule> localizedModuleEntities = new List<LocalizedModule>();

                foreach (var item in model.LocalizedModules)
                {
                    LocalizedModule lm = new LocalizedModule();
                    lm = item.ToEntity(lm);
                    localizedModuleEntities.Add(lm);
                }

                PageModule pageModuleEntity = model.PageSettings.ToEntity();
                pageModuleEntity.PageId = model.PageId;
                pageModuleEntity.SetAuditForCreate();

                List<LocalizedPageModule> localizedPageModuleEntities = new List<LocalizedPageModule>();

                foreach (var item in model.LocalizedPageModules)
                {
                    LocalizedPageModule lpm = new LocalizedPageModule();
                    lpm = item.ToEntity(lpm);
                    localizedPageModuleEntities.Add(lpm);
                }

                _moduleService.AddNew(moduleEntity, localizedModuleEntities, pageModuleEntity, localizedPageModuleEntities);

                return RedirectToAction("PageModules", new { id = model.PageId });
            }

            model.Zones = GetZones(_pageService.GetById(SiteId, model.PageId));

            return View(model);
        }

        public ActionResult AddExisting(int id)
        {
            Page entity = _pageService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            ModuleSettingsModel model = new ModuleSettingsModel();

            model.PageId = entity.Id;
            model.PageName = entity.Name;
            model.PageSettings = new PageModuleModel();
            model.Zones = GetZones(entity);
            model.LocalizedPageModules = GetLocalizedPageModules();

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult AddExisting(ModuleSettingsModel model)
        {
            if (ModelState.IsValid)
            {
                PageModule pageModuleEntity = model.PageSettings.ToEntity();
                pageModuleEntity.PageId = model.PageId;
                pageModuleEntity.SetAuditForCreate();

                List<LocalizedPageModule> localizedPageModuleEntities = new List<LocalizedPageModule>();

                foreach (var item in model.LocalizedPageModules)
                {
                    LocalizedPageModule lpm = new LocalizedPageModule();
                    lpm = item.ToEntity(lpm);
                    localizedPageModuleEntities.Add(lpm);
                }

                _moduleService.AddExisting(pageModuleEntity, localizedPageModuleEntities);

                return RedirectToAction("PageModules", new { id = model.PageId });
            }

            model.Zones = GetZones(_pageService.GetById(SiteId, model.PageId));

            return View(model);
        }

        public ActionResult Settings(int id)
        {
            PageModule pageModule = _moduleService.GetForPageById(SiteId, id);

            if (pageModule == null)
            {
                throw new HttpException(404, "Page Module Not Found");
            }

            Page page = _pageService.GetById(SiteId, pageModule.PageId);

            if (page == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            ModuleSettingsModel model = new ModuleSettingsModel();

            model.PageId = page.Id;
            model.PageName = page.Name;
            model.SharedSettings = pageModule.Module.ToModel();
            model.PageSettings = pageModule.ToModel();
            model.Zones = GetZones(page);
            model.LocalizedModules = GetLocalizedModules(pageModule.ModuleId);
            model.LocalizedPageModules = GetLocalizedPageModules(pageModule.Id, pageModule.ModuleId, page.Id);

            return View(model);    
        }

        [HttpParamAction]
        [HttpPost]
        public ActionResult Settings(ModuleSettingsModel model)
        {
            Module moduleEntity = _moduleService.GetById(SiteId, model.SharedSettings.Id);
            PageModule pageModuleEntity = _moduleService.GetForPageById(SiteId, model.PageSettings.Id);

            if (ModelState.IsValid)
            {
                // SHARED SETTINGS

                model.SharedSettings.Authorized = GetPermissions();

                moduleEntity = model.SharedSettings.ToEntity(moduleEntity);
                moduleEntity.SetAuditForUpdate();

                List<LocalizedModule> localizedModuleEntities = new List<LocalizedModule>();

                foreach (var item in model.LocalizedModules)
                {
                    LocalizedModule lm = _moduleService.GetLocalizedById(item.Id);
                    lm = item.ToEntity(lm);
                    localizedModuleEntities.Add(lm);
                }

                _moduleService.Update(moduleEntity, localizedModuleEntities);

                // PAGE SETTINGS

                pageModuleEntity = model.PageSettings.ToEntity(pageModuleEntity);
                pageModuleEntity.SetAuditForUpdate();

                List<LocalizedPageModule> localizedPageModuleEntities = new List<LocalizedPageModule>();

                foreach (var item in model.LocalizedPageModules)
                {
                    LocalizedPageModule lpm = _moduleService.GetLocalizedForPageById(item.Id);
                    lpm = item.ToEntity(lpm);
                    localizedPageModuleEntities.Add(lpm);
                }

                _moduleService.Update(pageModuleEntity, localizedPageModuleEntities);

                return RedirectToAction("PageModules", new { id = model.PageId });
            }

            model.Zones = GetZones(_pageService.GetById(SiteId, model.PageId));

            return View(model);
        }

        [HttpParamAction]
        [HttpPost]
        public ActionResult Delete(ModuleSettingsModel model)
        {
            Module entity = _moduleService.GetById(SiteId, model.SharedSettings.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Module Not Found");
            }

            if (ModelState.IsValid)
            {
                _moduleService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Settings", new { id = model.SharedSettings.Id });
        }

        [HttpParamAction]
        [HttpPost]
        public ActionResult DeleteForPage(ModuleSettingsModel model)
        {
            PageModule entity = _moduleService.GetForPageById(SiteId, model.PageSettings.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Module Not Found");
            }

            if (ModelState.IsValid)
            {
                _moduleService.DeleteForPage(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("ModuleSettings", new { id = model.SharedSettings.Id });
        }

        #endregion

        #region Private Methods

        private List<LocalizedModuleModel> GetLocalizedModules()
        {
            List<LocalizedModuleModel> models = new List<LocalizedModuleModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedModuleModel model = new LocalizedModuleModel();
                model.LanguageId = Language.Id;
                model.LanguageName = Language.Name;
                models.Add(model);
            }

            return models;
        }

        private List<LocalizedModuleModel> GetLocalizedModules(int ModuleId)
        {
            List<LocalizedModuleModel> models = new List<LocalizedModuleModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedModuleModel model = _moduleService.GetLocalizedByModuleId(ModuleId, Language.Id).ToModel();
                if (model == null)
                {
                    model = new LocalizedModuleModel();
                    //model.SiteId = SiteId;
                    model.ModuleId = ModuleId;
                    model.LanguageId = Language.Id;
                    LocalizedModule entity = model.ToEntity();
                    _moduleService.CreateLocalized(entity);
                    model = _moduleService.GetLocalizedByModuleId(ModuleId, Language.Id).ToModel();
                    model.LanguageName = Language.Name;
                }
                models.Add(model);
            }

            return models;
        }

        private List<LocalizedPageModuleModel> GetLocalizedPageModules()
        {
            List<LocalizedPageModuleModel> models = new List<LocalizedPageModuleModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedPageModuleModel model = new LocalizedPageModuleModel();
                model.LanguageId = Language.Id;
                model.LanguageName = Language.Name;
                models.Add(model);
            }

            return models;
        }

        private List<LocalizedPageModuleModel> GetLocalizedPageModules(int PageModuleId, int ModuleId, int PageId)
        {
            List<LocalizedPageModuleModel> models = new List<LocalizedPageModuleModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedPageModuleModel model = _moduleService.GetLocalizedForPageByPageModuleId(PageModuleId, Language.Id).ToModel();
                if (model == null)
                {
                    model = new LocalizedPageModuleModel();
                    model.PageModuleId = PageModuleId;
                    model.LanguageId = Language.Id;
                    LocalizedPageModule entity = model.ToEntity();
                    _moduleService.CreateLocalizedForPage(entity);
                    model = _moduleService.GetLocalizedForPageByPageModuleId(PageModuleId, Language.Id).ToModel();
                    model.LanguageName = Language.Name;
                }
                models.Add(model);
            }

            return models;
        }

        private string[] GetZones(Page entity)
        {
            PageTemplateModel pageTemplate = new PageTemplateModel();

            if (entity.PageTemplateId > 0)
            {
                pageTemplate = _pageTemplateService.GetById(entity.PageTemplateId).ToModel();
            }
            else
            {
                pageTemplate = _pageTemplateService.GetById(WeapsyGlobal.Site.PageTemplateId).ToModel();
            }

            if (pageTemplate == null)
            {
                pageTemplate = new PageTemplateModel();
            }

            return pageTemplate.Zones.Split(",".ToCharArray());
        }

        public string GetPermissions()
        {
            string result = "";

            List<string> roles = _roleService.GetList();

            foreach (var role in roles)
            {
                if (Request.Form["SharedSettings.Authorized-" + role].Contains("true"))
                {
                    if (result == "") { result = role; } else { result += "," + role; }
                }
            }

            if (result == "") { result = AppSettings.RoleAdmin; } else { result += "," + AppSettings.RoleAdmin; }
            if (result == "") { result = AppSettings.RoleSuperAdmin; } else { result += "," + AppSettings.RoleSuperAdmin; }

            return result;
        }

        #endregion

    }
}