#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class PageController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;
        private readonly IModuleService _moduleService;
        private readonly IModuleTemplateService _moduleTemplateService;
        private readonly IPageService _pageService;
        private readonly IPageTemplateService _pageTemplateService;
        private readonly IRoleService _roleService;

        #endregion

        #region Constructors

        public PageController(ILanguageService languageService,
            IModuleService moduleService,
            IModuleTemplateService moduleTemplateService,
            IPageService pageService,
            IPageTemplateService pageTemplateService,
            IRoleService roleService)
        {
            _languageService = languageService;
            _moduleService = moduleService;
            _moduleTemplateService = moduleTemplateService;
            _pageService = pageService;
            _pageTemplateService = pageTemplateService;
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            ViewBag.LanguageId = WeapsyGlobal.Language.Id;

            return View(new List<PageModel>());
        }

        public PartialViewResult Grid(int LanguageId, int PageId, string Action)
        {
            if (PageId > 0 && Action != "")
            {
                _pageService.Move(SiteId, PageId, Action);
            }

            if (LanguageId == 0)
            {
                LanguageId = WeapsyGlobal.Language.Id;
            }

            //List<PageModel> model = new List<PageModel>();

            //foreach (var item in _pageService.GetSitePagesForTree(SiteId, LanguageId))
            //{
            //    PageModel p = item.ToModel();
            //    p.PageModuleCount = item.PageModules.Count;
            //    model.Add(p);
            //}

            List<PageModel> pageModels = new List<PageModel>();

            foreach (var item in _pageService.GetSitePagesForTree(SiteId, LanguageId))
            {
                PageModel pm = item.ToModel();

                if (item.PageTemplateId > 0)
                {
                    pm.PageTemplateName = _pageTemplateService.GetById(item.PageTemplateId).Name;
                }

                if (item.ModuleTemplateId > 0)
                {
                    pm.ModuleTemplateName = _moduleTemplateService.GetById(item.ModuleTemplateId).Name;
                }

                pageModels.Add(pm);
            }

            return PartialView("_Grid", pageModels);
        }

        public ActionResult Create()
        {
            List<LocalizedPageModel> localizedModels = new List<LocalizedPageModel>();

            foreach (var Language in _languageService.GetList(SiteId))
            {
                LocalizedPageModel localizedModel = new LocalizedPageModel();
                localizedModel.LanguageId = Language.Id;
                localizedModel.LanguageName = Language.Name;
                localizedModels.Add(localizedModel);
            }

            PageModel model = new PageModel();
            model.LocalizedPageModels = localizedModels;

            model.SortOrder = _pageService.GetList(SiteId).Count + 1;
            model.Authorized = _roleService.GetDefaultRoles(true, false, false);

            return View(model);
        }

        [HttpPost()]
        public ActionResult Create(PageModel model)
        {
            if (ModelState.IsValid)
            {
                model.Authorized = GetPermissions();

                if (string.IsNullOrWhiteSpace(model.Url))
                {
                    model.Url = Utils.ParseTextForUrl(model.Name);
                }

                Page entity = model.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                _pageService.Create(entity, model.LocalizedPageModels.ToEntities());

                List<Module> modulesForAllPages = _moduleService.GetListForAllPages(SiteId);

                if (modulesForAllPages.Count > 0)
                { 
                    foreach (var item in modulesForAllPages)
                    {
                        PageModule pageModule = new PageModule();

                        pageModule.PageId = entity.Id;
                        pageModule.ModuleId = item.Id;
                        pageModule.Zone = item.Zone;
                        pageModule.ModuleTemplateId = Consts.NullId;
                        pageModule.SortOrder = 1;
                        pageModule.Deleted = false;
                        pageModule.SetAuditForCreate();

                        List<LocalizedPageModule> localizedPageModules = new List<LocalizedPageModule>();

                        foreach (var lang in _languageService.GetList(SiteId))
                        {
                            LocalizedPageModule lpm = new LocalizedPageModule();

                            lpm.LanguageId = lang.Id;

                            localizedPageModules.Add(lpm);
                        }

                        _moduleService.CreateForPage(pageModule, localizedPageModules);
                    }                
                }

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Page entity = _pageService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            List<LocalizedPageModel> localizedModels = new List<LocalizedPageModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedPageModel lpm = _pageService.GetLocalizedByPageId(entity.Id, Language.Id).ToModel();
                if (lpm == null)
                {
                    lpm = new LocalizedPageModel();
                    lpm.PageId = id;
                    lpm.LanguageId = Language.Id;
                    LocalizedPage lpe = lpm.ToEntity();
                    _pageService.CreateLocalized(lpe);
                    lpm = _pageService.GetLocalizedByPageId(entity.Id, Language.Id).ToModel();
                    lpm.LanguageName = Language.Name;
                }
                //lpm.Body = HttpUtility.HtmlDecode(lpm.Body);
                localizedModels.Add(lpm);
            }

            PageModel model = entity.ToModel();
            model.LocalizedPageModels = localizedModels;

            return View(model);
        }

        [HttpParamAction]
        [HttpPost]
        public ActionResult Update(PageModel model)
        {
            Page entity = _pageService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            if (ModelState.IsValid)
            {
                model.Authorized = GetPermissions();

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                List<LocalizedPage> localizedEntities = new List<LocalizedPage>();

                foreach (var item in model.LocalizedPageModels)
                {
                    LocalizedPage lp = _pageService.GetLocalizedById(item.Id);
                    lp = item.ToEntity(lp);
                    localizedEntities.Add(lp);
                }

                _pageService.Update(entity, localizedEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [HttpPost]
        public ActionResult Delete(PageModel model)
        {
            Page entity = _pageService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Not Found");
            }

            if (ModelState.IsValid)
            {
                _pageService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

        #region Private Methods

        private string[] GetZones(Page entity)
        {
            PageTemplateModel pageTemplate = new PageTemplateModel();

            if (entity.PageTemplateId > 0)
            {
                pageTemplate = _pageTemplateService.GetById(entity.PageTemplateId).ToModel();
            }
            else
            {
                pageTemplate = _pageTemplateService.GetById(WeapsyGlobal.Site.PageTemplateId).ToModel();
            }

            if (pageTemplate == null)
            {
                pageTemplate = new PageTemplateModel();
            }

            return pageTemplate.Zones.Split(",".ToCharArray());
        }

        public string GetPermissions()
        {
            string result = "";

            List<string> roles = _roleService.GetList();

            foreach (var role in roles)
            {
                if (Request.Form["Authorized-" + role].Contains("true"))
                {
                    if (result == "") { result = role; } else { result += "," + role; }
                }
            }

            if (result == "") { result = AppSettings.RoleAdmin; } else { result += "," + AppSettings.RoleAdmin; }
            if (result == "") { result = AppSettings.RoleSuperAdmin; } else { result += "," + AppSettings.RoleSuperAdmin; }

            return result;
        }

        #endregion

    }

}