#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class PluginController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;
        private readonly IPluginService _pluginService;
        private readonly IRoleService _roleService;

        #endregion

        #region Constructors

        public PluginController(ILanguageService languageService,
            IPluginService pluginService,
            IRoleService roleService)
        {
            _languageService = languageService;
            _pluginService = pluginService;
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        public ActionResult List()
        {
            ViewBag.TypeId = 1;
            ViewBag.LanguageId = WeapsyGlobal.Language.Id;

            return View(new List<PluginModel>());
        }

        public PartialViewResult Grid(byte typeId, int languageId, int id, string action)
        {
            if (id > 0 && action != "")
            {
                _pluginService.Move(SiteId, id, action);
            }

            if (typeId == 0)
            {
                typeId = 1;
            }

            if (languageId == 0) 
            {
                languageId = WeapsyGlobal.Language.Id;
            }

            return PartialView("_Grid", _pluginService.GetSitePlugins(SiteId, (PluginType)typeId, languageId).ToModels());
        }

        public ActionResult Create()
        {
            List<LocalizedPluginModel> localizedPluginModels = new List<LocalizedPluginModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedPluginModel lm = new LocalizedPluginModel();
                lm.LanguageId = Language.Id;
                lm.LanguageName = Language.Name;
                localizedPluginModels.Add(lm);
            }

            PluginModel model = new PluginModel();
            model.LocalizedPluginModels = localizedPluginModels;

            model.Authorized = _roleService.GetDefaultRoles(true, false, false);

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(PluginModel model)
        {
            if (ModelState.IsValid)
            {
                model.Authorized = GetPermissions();

                Plugin entity = model.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                List<LocalizedPlugin> localizedPluginEntities = model.LocalizedPluginModels.ToEntities();
                foreach (var item in localizedPluginEntities)
                {
                    //item.SiteId = SiteId;
                }

                _pluginService.Create(entity, localizedPluginEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Plugin entity = _pluginService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Plugin Not Found");
            }

            List<LocalizedPluginModel> localizedPluginModels = new List<LocalizedPluginModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedPluginModel lpm = _pluginService.GetLocalizedByPluginId(entity.Id, Language.Id).ToModel();
                if (lpm == null)
                {
                    lpm = new LocalizedPluginModel();
                    lpm.PluginId = id;
                    lpm.LanguageId = Language.Id;
                    _pluginService.CreateLocalized(lpm.ToEntity());
                    lpm = _pluginService.GetLocalizedByPluginId(entity.Id, Language.Id).ToModel();
                    lpm.LanguageName = Language.Name;
                }
                localizedPluginModels.Add(lpm);
            }

            PluginModel model = entity.ToModel();
            model.LocalizedPluginModels = localizedPluginModels;

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(PluginModel model)
        {
            Plugin entity = _pluginService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Plugin Not Found");
            }

            if (ModelState.IsValid)
            {
                model.Authorized = GetPermissions();

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                List<LocalizedPlugin> localizedPluginEntities = new List<LocalizedPlugin>();

                foreach (var item in model.LocalizedPluginModels)
                {
                    LocalizedPlugin lp = _pluginService.GetLocalizedById(item.Id);
                    lp = item.ToEntity(lp);
                    localizedPluginEntities.Add(lp);
                }

                _pluginService.Update(entity, localizedPluginEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(PluginModel model)
        {
            Plugin entity = _pluginService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Plugin Not Found");
            }

            if (ModelState.IsValid)
            {
                _pluginService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

        #region Private Methods

        public string GetPermissions()
        {
            string result = "";

            List<string> roles = _roleService.GetList();

            foreach (var role in roles)
            {
                if (Request.Form["Authorized-" + role].Contains("true"))
                {
                    if (result == "") { result = role; } else { result += "," + role; }
                }
            }

            if (result == "") { result = AppSettings.RoleAdmin; } else { result += "," + AppSettings.RoleAdmin; }
            if (result == "") { result = AppSettings.RoleSuperAdmin; } else { result += "," + AppSettings.RoleSuperAdmin; }

            return result;
        }

        #endregion
    }

}