#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class SiteController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;
        private readonly ISiteService _siteService;

        #endregion

        #region Constructors

        public SiteController(ILanguageService languageService, ISiteService siteService)
        {
            _languageService = languageService;
            _siteService = siteService;
        }

        #endregion

        #region Public Methods

        public ActionResult Settings()
        {
            SiteModel model = _siteService.GetById(SiteId).ToModel();

            List<LocalizedSiteModel> localizedModels = new List<LocalizedSiteModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedSiteModel lsm = _siteService.GetLocalizedBySiteId(SiteId, Language.Id).ToModel();
                if (lsm == null)
                {
                    lsm = new LocalizedSiteModel();
                    lsm.SiteId = SiteId;
                    lsm.LanguageId = Language.Id;
                    LocalizedSite lse = lsm.ToEntity();
                    _siteService.CreateLocalized(lse);
                    lsm = _siteService.GetLocalizedBySiteId(SiteId, Language.Id).ToModel();
                    lsm.LanguageName = Language.Name;
                }

                model.LocalizedSiteModels = localizedModels;
                model.FolderForFiles = AppSettings.FolderForFiles.Replace("~", "");

                if (!string.IsNullOrWhiteSpace(model.Logo))
                {
                    model.LogoImage = model.FolderForFiles + model.Logo;
                }
                else
                {
                    model.LogoImage = "/Content/images/noimage.png";
                }

                localizedModels.Add(lsm);
            }
            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(SiteModel model)
        {
            Site entity = _siteService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Site Not Found");
            }

            if (ModelState.IsValid)
            {
                //HttpPostedFileBase logoFile = Request.Files["Logo"];

                //if (logoFile != null)
                //{
                //    var fileName = Path.GetFileName(logoFile.FileName);
                //    var physicalPath = Path.Combine(Server.MapPath("~/App_Data/"), fileName);
                //    logoFile.SaveAs(physicalPath);
                //}

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                List<LocalizedSite> localizedEntities = new List<LocalizedSite>();

                foreach (var item in model.LocalizedSiteModels)
                {
                    LocalizedSite ls = _siteService.GetLocalizedById(item.Id);
                    ls = item.ToEntity(ls);
                    localizedEntities.Add(ls);
                }

                _siteService.Update(entity, localizedEntities);

                TempData[Consts.MessageKeySuccess] = Resources.Admin.Site.Settings_ConfirmUpdate;

                return RedirectToAction("Index", "Home");
            }

            return View(model);
        }

        [HttpParamAction]
        [HttpPost]
        public ActionResult Delete(SiteModel model)
        {
            Site entity = _siteService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Site Not Found");
            }

            if (ModelState.IsValid)
            {
                _siteService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        //public ActionResult FirstLook(bool? autoUpload, bool? multiple)
        //{
        //    ViewData["autoUpload"] = autoUpload ?? true;
        //    ViewData["multiple"] = multiple ?? true;

        //    return View();
        //}

        public ActionResult SaveLogo(IEnumerable<HttpPostedFileBase> Logo)
        {
            foreach (var file in Logo)
            {
                var fileName = Path.GetFileName(file.FileName);
                var physicalPath = Path.Combine(Server.MapPath(AppSettings.FolderForFiles), fileName);
                file.SaveAs(physicalPath);
                Site site = _siteService.GetById(SiteId);
                site.Logo = fileName;
                site.SetAuditForUpdate();
                _siteService.Update(site, new List<LocalizedSite>());
            }
            return Content("");
        }

        public ActionResult RemoveLogo(string[] fileNames)
        {
            foreach (var fullName in fileNames)
            {
                var fileName = Path.GetFileName(fullName);
                var physicalPath = Path.Combine(Server.MapPath(AppSettings.FolderForFiles), fileName);
                if (System.IO.File.Exists(physicalPath))
                {
                    System.IO.File.Delete(physicalPath);
                }
            }
            Site site = _siteService.GetById(SiteId);
            site.Logo = string.Empty;
            site.SetAuditForUpdate();
            _siteService.Update(site, new List<LocalizedSite>());
            return Content("");
        }

        public ActionResult DeleteLogo()
        {
            Site site = _siteService.GetById(SiteId);

            var fileName = site.Logo;
            var physicalPath = Path.Combine(Server.MapPath(AppSettings.FolderForFiles), fileName);
            if (System.IO.File.Exists(physicalPath))
            {
                System.IO.File.Delete(physicalPath);
            }

            site.Logo = string.Empty;
            site.SetAuditForUpdate();
            _siteService.Update(site, new List<LocalizedSite>());
            return Content("");
        }

        #endregion

    }

}