#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class WidgetController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;
        private readonly IWidgetService _widgetService;
        private readonly IRoleService _roleService;

        #endregion

        #region Constructors

        public WidgetController(ILanguageService languageService,
            IWidgetService widgetService,
            IRoleService roleService)
        {
            _languageService = languageService;
            _widgetService = widgetService;
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        public ActionResult List()
        {
            ViewBag.TypeId = 1;
            ViewBag.LanguageId = WeapsyGlobal.Language.Id;

            return View(new List<WidgetModel>());
        }

        public PartialViewResult Grid(byte typeId, int languageId, int id, string action)
        {
            if (id > 0 && action != "")
            {
                _widgetService.Move(SiteId, id, action);
            }

            if (typeId == 0)
            {
                typeId = 1;
            }

            if (languageId == 0) 
            {
                languageId = WeapsyGlobal.Language.Id;
            }

            return PartialView("_Grid", _widgetService.GetSiteWidgets(SiteId, (WidgetType)typeId, languageId).ToModels());
        }

        public ActionResult Create()
        {
            List<LocalizedWidgetModel> localizedWidgetModels = new List<LocalizedWidgetModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedWidgetModel lm = new LocalizedWidgetModel();
                lm.LanguageId = Language.Id;
                lm.LanguageName = Language.Name;
                localizedWidgetModels.Add(lm);
            }

            WidgetModel model = new WidgetModel();
            model.LocalizedWidgetModels = localizedWidgetModels;

            model.Authorized = _roleService.GetDefaultRoles(true, false, false);

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(WidgetModel model)
        {
            if (ModelState.IsValid)
            {
                model.Authorized = GetPermissions();

                Widget entity = model.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                List<LocalizedWidget> localizedWidgetEntities = model.LocalizedWidgetModels.ToEntities();
                foreach (var item in localizedWidgetEntities)
                {
                    //item.SiteId = SiteId;
                }

                _widgetService.Create(entity, localizedWidgetEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Widget entity = _widgetService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Widget Not Found");
            }

            List<LocalizedWidgetModel> localizedWidgetModels = new List<LocalizedWidgetModel>();

            foreach (Language Language in _languageService.GetList(SiteId))
            {
                LocalizedWidgetModel lwm = _widgetService.GetLocalizedByWidgetId(entity.Id, Language.Id).ToModel();
                if (lwm == null)
                {
                    lwm = new LocalizedWidgetModel();
                    lwm.WidgetId = id;
                    lwm.LanguageId = Language.Id;
                    _widgetService.CreateLocalized(lwm.ToEntity());
                    lwm = _widgetService.GetLocalizedByWidgetId(entity.Id, Language.Id).ToModel();
                    lwm.LanguageName = Language.Name;
                }
                localizedWidgetModels.Add(lwm);
            }

            WidgetModel model = entity.ToModel();
            model.LocalizedWidgetModels = localizedWidgetModels;

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(WidgetModel model)
        {
            Widget entity = _widgetService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Widget Not Found");
            }

            if (ModelState.IsValid)
            {
                model.Authorized = GetPermissions();

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                List<LocalizedWidget> localizedWidgetEntities = new List<LocalizedWidget>();

                foreach (var item in model.LocalizedWidgetModels)
                {
                    LocalizedWidget lp = _widgetService.GetLocalizedById(item.Id);
                    lp = item.ToEntity(lp);
                    localizedWidgetEntities.Add(lp);
                }

                _widgetService.Update(entity, localizedWidgetEntities);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(WidgetModel model)
        {
            Widget entity = _widgetService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Widget Not Found");
            }

            if (ModelState.IsValid)
            {
                _widgetService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

        #region Private Methods

        public string GetPermissions()
        {
            string result = "";

            List<string> roles = _roleService.GetList();

            foreach (var role in roles)
            {
                if (Request.Form["Authorized-" + role].Contains("true"))
                {
                    if (result == "") { result = role; } else { result += "," + role; }
                }
            }

            if (result == "") { result = AppSettings.RoleAdmin; } else { result += "," + AppSettings.RoleAdmin; }
            if (result == "") { result = AppSettings.RoleSuperAdmin; } else { result += "," + AppSettings.RoleSuperAdmin; }

            return result;
        }

        #endregion

    }

}