#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Services;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using Weapsy.Apps.Forum.Entities;
using Weapsy.Apps.Forum.Services;
using Weapsy.Apps.Forum.Extensions;
using Weapsy.Apps.Forum.Models.AdminModels;
using AutoMapper;

namespace Weapsy.Apps.Forum.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class AdminController : BaseController
    {
        // TEMP
        protected override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            var action = filterContext.Result as ViewResult;
            if (action != null)
            {
                action.MasterName = "_ForumAdminLayout";
            }
            base.OnActionExecuted(filterContext);
        }

        private ForumConfiguration _forumConfiguration = null;

        public ForumConfiguration ForumConfiguration
        {
            get
            {
                if (_forumConfiguration == null)
                {
                    _forumConfiguration = new ForumConfiguration();
                    _forumConfiguration = _configurationService.GetBySiteId(SiteId);
                    // default configuration not found
                    if (_forumConfiguration == null)
                    {
                        // initialize forum
                        _installService.Initialize(SiteId);
                        _forumConfiguration = new ForumConfiguration();
                        _forumConfiguration = _configurationService.GetBySiteId(SiteId);
                    }
                }
                return _forumConfiguration;
            }
        }

        #region Private Members

        private readonly ICategoryService _categoryService;
        private readonly IConfigurationService _configurationService;
        private readonly IPermissionSetService _permissionSetService;
        private readonly ISectionService _sectionService;
        private readonly ISettingSetService _settingSetService;
        private readonly Apps.Forum.Services.IInstallService _installService;
        private readonly IRoleService _roleService;

        #endregion

        #region Constructors

        public AdminController(ICategoryService categoryService,
            IConfigurationService configurationService,
            IPermissionSetService permissionSetService,
            ISectionService sectionService,
            ISettingSetService settingSetService,
            Apps.Forum.Services.IInstallService installService,
            IRoleService roleService)
        {
            _categoryService = categoryService;
            _configurationService = configurationService;
            _permissionSetService = permissionSetService;
            _sectionService = sectionService;
            _settingSetService = settingSetService;
            _installService = installService;
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        #region Index

        public ActionResult Index()
        {
            return View();
        }

        #endregion

        #region Categories

        public ActionResult Categories()
        {
            return View(new List<ForumCategoryModel>());
        }

        public PartialViewResult CategoryGrid(int id, string action)
        {
            if (id > 0 && action != "")
            {
                _categoryService.Move(this.ForumConfiguration.Id, id, action);
            }

            return PartialView("_CategoryGrid", _categoryService.GetList(this.ForumConfiguration.Id).ToModels());
        }

        public ActionResult CreateCategory()
        {
            ForumCategoryModel model = new ForumCategoryModel();

            model.ConfigurationId = ForumConfiguration.Id;
            model.SortOrder = _categoryService.GetList(ForumConfiguration.Id).Count + 1;
            model.Authorized = _roleService.GetDefaultRoles(true, false, false);

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult CreateCategory(ForumCategoryModel model)
        {
            if (ModelState.IsValid)
            {
                if (_categoryService.GetByName(model.ConfigurationId, model.Name, false) != null)
                {
                    ModelState.AddModelError("", Resources.Apps.Forum.Admin.Category_Error_NameExists);
                    return View(model);
                }

                model.Authorized = GetPermissions("Authorized-");

                ForumCategory entity = model.ToEntity();

                if (string.IsNullOrEmpty(entity.Url))
                {
                    entity.Url = Utils.ParseTextForUrl(entity.Name);
                }

                entity.SetAuditForCreate();

                _categoryService.Create(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.CreateCategory_Confirm, entity.Name);

                return RedirectToAction("Categories");
            }

            return View(model);
        }

        public ActionResult EditCategory(int id)
        {
            ForumCategory entity = _categoryService.GetById(this.ForumConfiguration.Id, id);

            if (entity == null)
            {
                throw new HttpException(404, "Category Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateCategory(ForumCategoryModel model)
        {
            ForumCategory entity = _categoryService.GetById(this.ForumConfiguration.Id, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Category Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_categoryService.GetByName(model.ConfigurationId, model.Name, false) != null)
                    {
                        ModelState.AddModelError("", Resources.Apps.Forum.Admin.Category_Error_NameExists);
                        return View(model);
                    }
                }

                model.Authorized = GetPermissions("Authorized-");

                entity = model.ToEntity(entity);

                if (string.IsNullOrEmpty(entity.Url))
                {
                    entity.Url = Utils.ParseTextForUrl(entity.Name);
                }

                entity.SetAuditForUpdate();

                _categoryService.Update(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.EditCategory_Confirm, entity.Name);

                return RedirectToAction("Categories");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeleteCategory(ForumCategoryModel model)
        {
            ForumCategory entity = _categoryService.GetById(this.ForumConfiguration.Id, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Category Not Found");
            }

            if (ModelState.IsValid)
            {
                _categoryService.Delete(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.DeleteCategory_Confirm, entity.Name);

                return RedirectToAction("Categories");
            }

            return RedirectToAction("EditCategory", new { id = model.Id });
        }

        #endregion

        #region Configurations

        public ActionResult Configuration()
        {
            ForumConfiguration entity = _configurationService.GetBySiteId(this.SiteId);

            if (entity == null)
            {
                throw new HttpException(404, "Configuration Not Found");
            }

            return View(entity.ToModel());
        }

        //public ActionResult Configurations()
        //{
        //    List<ForumConfiguration> list = new List<ForumConfiguration>();

        //    list = _configurationService.GetList(SiteId);

        //    return View(list.ToModels());
        //}

        //public ActionResult CreateConfiguration()
        //{
        //    return View(new ForumConfigurationModel());
        //}

        //[AcceptVerbs(HttpVerbs.Post)]
        //public ActionResult CreateConfiguration(ForumConfigurationModel model)
        //{
        //    if (ModelState.IsValid)
        //    {
        //        //if (_configurationService.GetByName(model.Name, false) != null)
        //        //{
        //        //    ModelState.AddModelError("", "A Configuration with the same Name already exists");
        //        //    return View(model);
        //        //}

        //        ForumConfiguration entity = model.ToEntity();
        //        entity.SetAuditForCreate();
        //        _configurationService.Create(entity);

        //        return RedirectToAction("Configurations");
        //    }

        //    return View(model);
        //}

        //public ActionResult EditConfiguration(int id)
        //{
        //    ForumConfiguration entity = _configurationService.GetById(id);

        //    if (entity == null)
        //    {
        //        throw new HttpException(404, "Configuration Not Found");
        //    }

        //    return View(entity.ToModel());
        //}

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateConfiguration(ForumConfigurationModel model)
        {
            ForumConfiguration entity = _configurationService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Configuration Not Found");
            }

            if (ModelState.IsValid)
            {
                //if (entity.Name != model.Name)
                //{
                //    if (_configurationService.GetByName(model.Name, false) != null)
                //    {
                //        return RedirectToAction("EditConfiguration", new { id = entity.Id });
                //    }
                //}

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _configurationService.Update(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.Configuration_Confirm, entity.Name);

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //[HttpParamAction]
        //[AcceptVerbs(HttpVerbs.Post)]
        //public ActionResult DeleteConfiguration(ForumConfigurationModel model)
        //{
        //    ForumConfiguration entity = _configurationService.GetById(model.Id);

        //    if (entity == null)
        //    {
        //        throw new HttpException(404, "Configuration Not Found");
        //    }

        //    if (ModelState.IsValid)
        //    {
        //        _configurationService.Delete(entity);

        //        return RedirectToAction("Configurations");
        //    }

        //    return RedirectToAction("EditConfiguration", new { id = model.Id });
        //}

        #endregion

        #region PermissionSets

        public ActionResult PermissionSets()
        {
            List<ForumPermissionSet> list = new List<ForumPermissionSet>();

            list = _permissionSetService.GetList(this.ForumConfiguration.Id);

            return View(list.ToModels());
        }

        public ActionResult CreatePermissionSet()
        {
            ForumPermissionSetModel model = new ForumPermissionSetModel();

            model.ConfigurationId = this.ForumConfiguration.Id;

            string roles = _roleService.GetDefaultRoles(false, false, false);

            model.ViewSection = roles;
            model.ViewTopics = roles;
            model.Read = roles;
            model.Start = roles;
            model.Reply = roles;

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult CreatePermissionSet(ForumPermissionSetModel model)
        {
            if (ModelState.IsValid)
            {
                if (_permissionSetService.GetByName(this.ForumConfiguration.Id, model.Name) != null)
                {
                    ModelState.AddModelError("", Resources.Apps.Forum.Admin.PermissionSet_Error_NameExists);
                    return View(model);
                }

                model.ViewSection = GetPermissions("Auth-ViewSection-");
                model.ViewTopics = GetPermissions("Auth-ViewTopics-");
                model.Read = GetPermissions("Auth-Read-");
                model.Start = GetPermissions("Auth-Start-");
                model.Reply = GetPermissions("Auth-Reply-");

                ForumPermissionSet entity = model.ToEntity();
                entity.SetAuditForCreate();
                _permissionSetService.Create(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.CreatePermissionSet_Confirm, entity.Name);

                return RedirectToAction("PermissionSets");
            }

            return View(model);
        }

        public ActionResult EditPermissionSet(int id)
        {
            ForumPermissionSet entity = _permissionSetService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "PermissionSet Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdatePermissionSet(ForumPermissionSetModel model)
        {
            ForumPermissionSet entity = _permissionSetService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "PermissionSet Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_permissionSetService.GetByName(this.ForumConfiguration.Id, model.Name) != null)
                    {
                        ModelState.AddModelError("", Resources.Apps.Forum.Admin.PermissionSet_Error_NameExists);
                        return View(model);
                    }
                }

                model.ViewSection = GetPermissions("Auth-ViewSection-");
                model.ViewTopics = GetPermissions("Auth-ViewTopics-");
                model.Read = GetPermissions("Auth-Read-");
                model.Start = GetPermissions("Auth-Start-");
                model.Reply = GetPermissions("Auth-Reply-");

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _permissionSetService.Update(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.EditPermissionSet_Confirm, entity.Name);

                return RedirectToAction("PermissionSets");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeletePermissionSet(ForumPermissionSetModel model)
        {
            ForumPermissionSet entity = _permissionSetService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "PermissionSet Not Found");
            }

            if (ModelState.IsValid)
            {
                _permissionSetService.Delete(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.DeletePermissionSet_Confirm, entity.Name);

                return RedirectToAction("PermissionSets");
            }

            return RedirectToAction("EditPermissionSet", new { id = model.Id });
        }

        #endregion

        #region Sections

        public ActionResult Sections(int? id)
        {
            ViewBag.ConfigurationId = this.ForumConfiguration.Id;

            if (id == null)
            { 
                List<ForumCategory> list = _categoryService.GetList(this.ForumConfiguration.Id);
                ViewBag.CategoryId = list[0].Id;
            }
            else
            {
                ViewBag.CategoryId = id;
            }
            
            return View(new List<ForumSectionModel>());
        }

        public PartialViewResult SectionGrid(int categoryId, int id, string action)
        {
            if (id > 0 && action != "")
            {
                _sectionService.Move(categoryId, id, action);
            }

            return PartialView("_SectionGrid", _sectionService.GetList(categoryId).ToModels());
        }

        public ActionResult CreateSection(int? id)
        {
            ViewBag.ConfigurationId = this.ForumConfiguration.Id;

            ForumSectionModel model = new ForumSectionModel();

            if (id == null)
            {
                List<ForumCategory> list = _categoryService.GetList(this.ForumConfiguration.Id);
                model.CategoryId = list[0].Id;
            }
            else
            {
                model.CategoryId = (int)id;
            }

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult CreateSection(ForumSectionModel model)
        {
            if (ModelState.IsValid)
            {
                if (_sectionService.GetByName(model.CategoryId, model.Name) != null)
                {
                    ModelState.AddModelError("", Resources.Apps.Forum.Admin.Section_Error_NameExists);
                    return View(model);
                }

                ForumSection entity = model.ToEntity();

                if (string.IsNullOrEmpty(entity.Url))
                {
                    entity.Url = Utils.ParseTextForUrl(entity.Name);
                }

                entity.SetAuditForCreate();

                _sectionService.Create(entity);

                return RedirectToAction("Sections");
            }

            return View(model);
        }

        public ActionResult EditSection(int id)
        {
            ViewBag.ConfigurationId = this.ForumConfiguration.Id;

            ForumSection entity = _sectionService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Section Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateSection(ForumSectionModel model)
        {
            ForumSection entity = _sectionService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Section Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_sectionService.GetByName(model.CategoryId, model.Name) != null)
                    {
                        ModelState.AddModelError("", Resources.Apps.Forum.Admin.Section_Error_NameExists);
                        return View(model);
                    }
                }

                entity = model.ToEntity(entity);

                if (string.IsNullOrEmpty(entity.Url))
                {
                    entity.Url = Utils.ParseTextForUrl(entity.Name);
                }

                entity.SetAuditForUpdate();

                _sectionService.Update(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.EditSection_Confirm, entity.Name);

                return RedirectToAction("Sections");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeleteSection(ForumSectionModel model)
        {
            ForumSection entity = _sectionService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Section Not Found");
            }

            if (ModelState.IsValid)
            {
                _sectionService.Delete(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.DeleteSection_Confirm, entity.Name);

                return RedirectToAction("Sections");
            }

            return RedirectToAction("EditSection", new { id = model.Id });
        }

        #endregion

        #region SettingSets

        public ActionResult SettingSets()
        {
            List<ForumSettingSet> list = new List<ForumSettingSet>();

            list = _settingSetService.GetList(this.ForumConfiguration.Id);

            return View(list.ToModels());
        }

        public ActionResult CreateSettingSet()
        {
            ForumSettingSetModel model = new ForumSettingSetModel();

            model.ConfigurationId = this.ForumConfiguration.Id;

            string roles = _roleService.GetDefaultRoles(false, false, false);

            model.ModerateTopicsExcluded = roles;
            model.ModerateRepliesExcluded = roles;

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult CreateSettingSet(ForumSettingSetModel model)
        {
            if (ModelState.IsValid)
            {
                if (_settingSetService.GetByName(this.ForumConfiguration.Id, model.Name) != null)
                {
                    ModelState.AddModelError("", Resources.Apps.Forum.Admin.SettingSet_Error_NameExists);
                    return View(model);
                }

                model.ModerateTopicsExcluded = GetPermissions("ModerateTopicsExcluded");
                model.ModerateRepliesExcluded = GetPermissions("ModerateRepliesExcluded");

                ForumSettingSet entity = model.ToEntity();
                entity.SetAuditForCreate();
                _settingSetService.Create(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.CreateSettingSet_Confirm, entity.Name);

                return RedirectToAction("SettingSets");
            }

            return View(model);
        }

        public ActionResult EditSettingSet(int id)
        {
            ForumSettingSet entity = _settingSetService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "SettingSet Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateSettingSet(ForumSettingSetModel model)
        {
            ForumSettingSet entity = _settingSetService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "SettingSet Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_settingSetService.GetByName(this.ForumConfiguration.Id, model.Name) != null)
                    {
                        ModelState.AddModelError("", Resources.Apps.Forum.Admin.SettingSet_Error_NameExists);
                        return View(model);
                    }
                }

                model.ModerateTopicsExcluded = GetPermissions("ModerateTopicsExcluded");
                model.ModerateRepliesExcluded = GetPermissions("ModerateRepliesExcluded");

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _settingSetService.Update(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.EditSettingSet_Confirm, entity.Name);

                return RedirectToAction("SettingSets");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeleteSettingSet(ForumSettingSetModel model)
        {
            ForumSettingSet entity = _settingSetService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "SettingSet Not Found");
            }

            if (ModelState.IsValid)
            {
                _settingSetService.Delete(entity);

                TempData[Consts.MessageKeySuccess] = string.Format(Resources.Apps.Forum.Admin.DeleteSettingSet_Confirm, entity.Name);

                return RedirectToAction("SettingSets");
            }

            return RedirectToAction("EditSettingSet", new { id = model.Id });
        }

        #endregion

        #endregion

        #region Private Methods

        public string GetPermissions(string name)
        {
            string result = "";

            List<string> roles = _roleService.GetList();

            foreach (var role in roles)
            {
                if (Request.Form[name + role].Contains("true"))
                {
                    if (result == "") { result = role; } else { result += "," + role; }
                }
            }

            if (result == "") { result = AppSettings.RoleAdmin; } else { result += "," + AppSettings.RoleAdmin; }
            if (result == "") { result = AppSettings.RoleSuperAdmin; } else { result += "," + AppSettings.RoleSuperAdmin; }

            return result;
        }

        #endregion

    }

}