#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Apps.Forum.Entities;

namespace Weapsy.Apps.Forum.Services
{
    public class CategoryService : ICategoryService
    {

        #region Private Members

        private readonly IRepository<ForumCategory> _categoryRepository;
        private readonly ICacheManager _cacheManager;

        private const string Key = Consts.GlobalKey + "|Forum|Categories|ConfigurationId:{0}";

        #endregion

        #region Constructors

        public CategoryService(IRepository<ForumCategory> appRepository,
            ICacheManager cacheManager)
        {
            _categoryRepository = appRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ForumCategory entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Category");
            }
            _categoryRepository.Create(entity);
            _cacheManager.Remove(Key);
        }

        public virtual void Update(ForumCategory entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Category");
            }
            _categoryRepository.Update(entity);
            _cacheManager.Remove(Key);
        }

        public virtual void Delete(ForumCategory entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Category");
            }

            _categoryRepository.Delete(entity);
            _cacheManager.Remove(Key);
        }

        public virtual ForumCategory GetById(int configurationId, int id)
        {
            return GetById(configurationId, id, true);
        }

        public virtual ForumCategory GetById(int configurationId, int id, bool useCache)
        {
            if (useCache)
            {
                return GetList(configurationId, true).Find(o => o.Id == id);
            }
            else
            {
                return _GetById(configurationId, id);
            }
        }

        public virtual ForumCategory GetByUrl(int configurationId, string url)
        {
            return GetByUrl(configurationId, url, true);
        }

        public virtual ForumCategory GetByUrl(int configurationId, string url, bool useCache)
        {
            if (useCache)
            {
                return GetList(configurationId, true).Find(o => o.Url == url);
            }
            else
            {
                return _GetByUrl(configurationId, url);
            }
        }

        public virtual ForumCategory GetByName(int configurationId, string name)
        {
            return GetByName(configurationId, name, true);
        }

        public virtual ForumCategory GetByName(int configurationId, string name, bool useCache)
        {
            if (useCache)
            {
                return GetList(configurationId, true).Find(o => o.Name == name);
            }
            else
            {
                return _GetByName(configurationId, name);
            }
        }

        public virtual List<ForumCategory> GetList(int configurationId)
        {
            return GetList(configurationId, true);
        }

        public virtual List<ForumCategory> GetList(int configurationId, bool useCache)
        {
            List<ForumCategory> list;
            if (useCache)
            {
                string cacheKey = string.Format(Key, configurationId.ToString());
                object obj = _cacheManager.Get(cacheKey);
                if (obj != null)
                {
                    list = (List<ForumCategory>)obj;
                }
                else
                {
                    list = _GetList(configurationId);
                    _cacheManager.Insert(Key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList(configurationId);
            }
            return list;
        }

        public virtual Dictionary<string, string> GetDictionary(int configurationId)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add(Consts.NullId.ToString(), "");
            foreach (var item in GetList(configurationId))
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }
            return dic;
        }

        public virtual void Move(int configurationId, int Id, string Action)
        {
            ForumCategory entity1 = GetById(configurationId, Id, false);
            int entity1NewSortOrder;

            ForumCategory entity2 = new ForumCategory();
            int entity2NewSortOrder;

            if (Action == "UP")
            {
                entity2 = _GetBySortOrder(configurationId, entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(configurationId, entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _categoryRepository.Update(entity1);
            _categoryRepository.Update(entity2);

            _cacheManager.Remove(string.Format(Key, configurationId));
        }

        #endregion

        #region Private Methods

        private ForumCategory _GetById(int configurationId, int id)
        {
            var query = from item in _categoryRepository.Table
                        where item.ConfigurationId == configurationId && item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private ForumCategory _GetByUrl(int configurationId, string url)
        {
            var query = from item in _categoryRepository.Table
                        where item.ConfigurationId == configurationId && item.Url == url
                        select item;
            return query.FirstOrDefault();
        }

        private ForumCategory _GetByName(int configurationId, string name)
        {
            var query = from item in _categoryRepository.Table
                        where item.ConfigurationId == configurationId && item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private ForumCategory _GetBySortOrder(int configurationId, int sortOrder)
        {
            var query = from item in _categoryRepository.Table
                        where item.ConfigurationId == configurationId && item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        private List<ForumCategory> _GetList(int configurationId)
        {
            var query = from item in _categoryRepository.Table
                        where item.ConfigurationId == configurationId
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        #endregion

    }
}