#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Services;
using Weapsy.Apps.Forum.Entities;

namespace Weapsy.Apps.Forum.Services
{
    public class InstallService : IInstallService
    {

        #region Private Members

        private readonly ICategoryService _categoryService;
        private readonly IConfigurationService _configurationService;
        private readonly IPermissionSetService _permissionSetService;
        private readonly ISectionService _sectionService;
        private readonly ISettingSetService _settingSetService;
        private readonly IRoleService _roleService;

        private string _IpAddress = string.Empty;

        private string GetIpAddress()
        {
            if (_IpAddress == string.Empty)
            {
                if (HttpContext.Current != null)
                {
                    _IpAddress = HttpContext.Current.Request.UserHostAddress;
                }
                else
                {
                    _IpAddress = Consts.NullIpAddress;
                }
            }
            return _IpAddress;
        }

        #endregion

        #region Constructors

        public InstallService(ICategoryService categoryService,
            IConfigurationService configurationService,
            IPermissionSetService permissionSetService,
            ISectionService sectionService,
            ISettingSetService settingSetService,
            IRoleService roleService)
        {
            _categoryService = categoryService;
            _configurationService = configurationService;
            _permissionSetService = permissionSetService;
            _sectionService = sectionService;
            _settingSetService = settingSetService;
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        public void Initialize(int siteId)
        {
            string publicRoles = _roleService.GetDefaultRoles(true, false, false);
            string registeredRoles = _roleService.GetDefaultRoles(false, true, false);
            string adminRoles = _roleService.GetDefaultRoles(false, false, false);

            // ==============================
            //  CONFIGURATION
            // ==============================

            ForumConfiguration configuration = InstallConfiguration(
                new ForumConfiguration() { 
                    SiteId = siteId,
                    Name = "Default",
                    Enabled = true,
                    PermissionSetId = 0,
                    SettingSetId = 0
                }
            );

            // ==============================
            //  PERMISSION SETS
            // ==============================

            ForumPermissionSet defaultPermissionSet = InstallPermissionSet(
                new ForumPermissionSet() { 
                    ConfigurationId = configuration.Id,
                    Name = "Default",
                    Description = "Default Permission Set",
                    ViewSection = publicRoles,
                    ViewTopics = publicRoles,
                    Read = publicRoles,
                    Start = registeredRoles,
                    Reply = registeredRoles,
                    Upload = adminRoles,
                    Download = adminRoles,
                    SubscribeSection = registeredRoles,
                    SubscribeTopic = registeredRoles,
                    Edit = registeredRoles,
                    Delete = adminRoles,
                    Pin = adminRoles,
                    Lock = adminRoles,
                    Moderate = adminRoles,
                    VotePoll = registeredRoles,
                    CreatePoll = adminRoles,
                    SelectStatus = registeredRoles,
                    Announce = registeredRoles,
                    RateTopics = registeredRoles
                }
            );

            // ==============================
            //  SETTING SETS
            // ==============================

            ForumSettingSet defaultSettingSet = InstallSettingSet(
                new ForumSettingSet()
                {
                    ConfigurationId = configuration.Id,
                    Name = "Default",
                    Description = "Default Setting Set",
                    ModerateTopics = false,
                    ModerateTopicsExcluded = adminRoles,
                    ModerateReplies = false,
                    ModerateRepliesExcluded = adminRoles,
                    //SubscribeSection = true,
                    //SubscribeTopic = true,
                    //Emoticons = true,
                    //PostIcon = true,
                    //Attachments = false,
                    //AttachMaxNumber = 3,
                    //AttachMaxSize = 1000,
                    //AttachTypes = "jpg,jpeg,png,bmp",
                    //AttachMaxImgHeight = 400,
                    //AttachMaxImgWidth = 400,
                    //AttachMaxImgThumbHeight = 40,
                    //AttachMaxImgThumbWidth = 40,
                    //TopicRating = false,
                    //RSS = false,
                    //Polls = false,
                    //StatusSelection = false,
                    //StatusRequired = false,
                    //QuickReply = true,
                    //PostIconDisplayDefault = false,
                    //PostIconRequired = false,
                    //TopicsSortOrder = 1,
                    //TopicsSortField = 1,
                    //TopicsSortFilter = 1,
                    //PostEditRestrictionEnabled = false,
                    //PostEditRestrictionInterval = 600,
                    //PostEditRestrictionExcluded = adminRoles,
                    //PostEditReasonEnabled = true,
                    //PostEditReasonRequired = true,
                    //PostEditReasonExcluded = adminRoles
                }
            );

            // ==============================
            //  CATEGORIES
            // ==============================

            ForumCategory Category1 = InstallCategory(
                new ForumCategory()
                {
                    ConfigurationId = configuration.Id,
                    Name = "Category 1",
                    Active = true,
                    PermissionSetId = Consts.NullId,
                    SettingSetId = Consts.NullId,
                    SortOrder = 1,
                    Authorized = publicRoles
                }
            );

            ForumCategory Category2 = InstallCategory(
                new ForumCategory()
                {
                    ConfigurationId = configuration.Id,
                    Name = "Category 2",
                    Active = true,
                    PermissionSetId = Consts.NullId,
                    SettingSetId = Consts.NullId,
                    SortOrder = 2,
                    Authorized = publicRoles
                }
            );

            // ==============================
            //  SECTIONS
            // ==============================

            ForumSection Section1A = InstallSection(
                new ForumSection()
                {
                    CategoryId = Category1.Id,
                    Name = "Section 1A",
                    Active = true,
                    PermissionSetId = Consts.NullId,
                    SettingSetId = Consts.NullId,
                    SortOrder = 1
                }
            );

            ForumSection Section1B = InstallSection(
                new ForumSection()
                {
                    CategoryId = Category1.Id,
                    Name = "Section 1B",
                    Active = true,
                    PermissionSetId = Consts.NullId,
                    SettingSetId = Consts.NullId,
                    SortOrder = 2
                }
            );

            ForumSection Section2A = InstallSection(
                new ForumSection()
                {
                    CategoryId = Category2.Id,
                    Name = "Section 2A",
                    Active = true,
                    PermissionSetId = Consts.NullId,
                    SettingSetId = Consts.NullId,
                    SortOrder = 1
                }
            );

            ForumSection Section2B = InstallSection(
                new ForumSection()
                {
                    CategoryId = Category2.Id,
                    Name = "Section 2B",
                    Active = true,
                    PermissionSetId = Consts.NullId,
                    SettingSetId = Consts.NullId,
                    SortOrder = 2
                }
            );

            // ==============================
            //  UPDATE CONFIGURATION
            // ==============================

            ForumConfiguration configurationToUpdate = new ForumConfiguration();

            configurationToUpdate.PermissionSetId = defaultPermissionSet.Id;
            configurationToUpdate.SettingSetId = defaultSettingSet.Id;

            _configurationService.Update(configurationToUpdate);
        }

        #endregion

        #region Private Methods

        private ForumCategory InstallCategory(ForumCategory obj)
        {
            ForumCategory entity = _categoryService.GetByName(obj.ConfigurationId, obj.Name);

            if (entity == null)
            {
                entity = new ForumCategory();

                obj.Url = Utils.ParseTextForUrl(obj.Name);

                obj.CreatedByUserId = Consts.NullId;
                obj.CreatedByIpAddress = GetIpAddress();
                obj.CreatedOnDate = Utils.GetDateForDB();

                obj.ModifiedByUserId = Consts.NullId;
                obj.ModifiedByIpAddress = GetIpAddress();
                obj.ModifiedOnDate = Utils.GetDateForDB();

                _categoryService.Create(obj);

                entity = _categoryService.GetByName(obj.ConfigurationId, obj.Name);
            }

            return entity;
        }

        private ForumConfiguration InstallConfiguration(ForumConfiguration obj)
        {
            ForumConfiguration entity = _configurationService.GetBySiteId(obj.SiteId, false);

            if (entity == null)
            {
                entity = new ForumConfiguration();

                obj.CreatedByUserId = Consts.NullId;
                obj.CreatedByIpAddress = GetIpAddress();
                obj.CreatedOnDate = Utils.GetDateForDB();

                obj.ModifiedByUserId = Consts.NullId;
                obj.ModifiedByIpAddress = GetIpAddress();
                obj.ModifiedOnDate = Utils.GetDateForDB();

                _configurationService.Create(obj);

                entity = _configurationService.GetBySiteId(obj.SiteId, false);
            }

            return entity;
        }

        private ForumPermissionSet InstallPermissionSet(ForumPermissionSet obj)
        {
            ForumPermissionSet entity = _permissionSetService.GetByName(obj.ConfigurationId, obj.Name);

            if (entity == null)
            {
                entity = new ForumPermissionSet();

                obj.CreatedByUserId = Consts.NullId;
                obj.CreatedByIpAddress = GetIpAddress();
                obj.CreatedOnDate = Utils.GetDateForDB();

                obj.ModifiedByUserId = Consts.NullId;
                obj.ModifiedByIpAddress = GetIpAddress();
                obj.ModifiedOnDate = Utils.GetDateForDB();

                _permissionSetService.Create(obj);

                entity = _permissionSetService.GetByName(obj.ConfigurationId, obj.Name);
            }

            return entity;
        }

        private ForumSection InstallSection(ForumSection obj)
        {
            ForumSection entity = _sectionService.GetByName(obj.CategoryId, obj.Name);

            if (entity == null)
            {
                entity = new ForumSection();

                obj.Url = Utils.ParseTextForUrl(obj.Name);

                obj.CreatedByUserId = Consts.NullId;
                obj.CreatedByIpAddress = GetIpAddress();
                obj.CreatedOnDate = Utils.GetDateForDB();

                obj.ModifiedByUserId = Consts.NullId;
                obj.ModifiedByIpAddress = GetIpAddress();
                obj.ModifiedOnDate = Utils.GetDateForDB();

                _sectionService.Create(obj);

                entity = _sectionService.GetByName(obj.CategoryId, obj.Name);
            }

            return entity;
        }

        private ForumSettingSet InstallSettingSet(ForumSettingSet obj)
        {
            ForumSettingSet entity = _settingSetService.GetByName(obj.ConfigurationId, obj.Name);

            if (entity == null)
            {
                entity = new ForumSettingSet();

                obj.CreatedByUserId = Consts.NullId;
                obj.CreatedByIpAddress = GetIpAddress();
                obj.CreatedOnDate = Utils.GetDateForDB();

                obj.ModifiedByUserId = Consts.NullId;
                obj.ModifiedByIpAddress = GetIpAddress();
                obj.ModifiedOnDate = Utils.GetDateForDB();

                _settingSetService.Create(obj);

                entity = _settingSetService.GetByName(obj.ConfigurationId, obj.Name);
            }

            return entity;
        }

        #endregion

    }
}