#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Apps.Forum.Entities;

namespace Weapsy.Apps.Forum.Services
{
    public class SectionService : ISectionService
    {

        #region Private Members

        private readonly IRepository<ForumSection> _sectionRepository;
        private readonly ICacheManager _cacheManager;

        private const string listKey = Consts.GlobalKey + "|Forum|Sections|CategoryId:{0}";
        private const string singleKey = Consts.GlobalKey + "|Forum|Section|CategoryId:{0}|SectionUrl:{1}";

        #endregion

        #region Constructors

        public SectionService(IRepository<ForumSection> sectionRepository,
            ICacheManager cacheManager)
        {
            _sectionRepository = sectionRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ForumSection entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Section");
            }
            _sectionRepository.Create(entity);
            _cacheManager.Remove(string.Format(listKey, entity.CategoryId));
        }

        public virtual void Update(ForumSection entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Section");
            }
            _sectionRepository.Update(entity);
            _cacheManager.Remove(string.Format(listKey, entity.CategoryId));
            _cacheManager.Remove(string.Format(singleKey, entity.Id, entity.Url));
        }

        public virtual void Delete(ForumSection entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Section");
            }
            _sectionRepository.Delete(entity);
            _cacheManager.Remove(string.Format(listKey, entity.CategoryId));
            _cacheManager.Remove(string.Format(singleKey, entity.Id, entity.Url));
        }

        public virtual ForumSection GetById(int id)
        {
            var query = from item in _sectionRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual ForumSection GetByName(int categoryId, string name)
        {
            var query = from item in _sectionRepository.Table
                        where item.CategoryId == categoryId && item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        public virtual ForumSection GetByUrl(int categoryId, string url)
        {
            return GetByUrl(categoryId, url, true);
        }

        public virtual ForumSection GetByUrl(int categoryId, string url, bool useCache)
        {
            ForumSection entity;
            if (useCache)
            {
                string cacheKey = string.Format(singleKey, categoryId.ToString(), url);
                object obj = _cacheManager.Get(cacheKey);
                if (obj != null)
                {
                    entity = (ForumSection)obj;
                }
                else
                {
                    entity = _GetByUrl(categoryId, url);
                    _cacheManager.Insert(cacheKey, entity, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                entity = _GetByUrl(categoryId, url);
            }
            return entity;
        }

        public virtual List<ForumSection> GetList(int categoryId)
        {
            return GetList(categoryId, true);
        }

        public virtual List<ForumSection> GetList(int categoryId, bool useCache)
        {
            List<ForumSection> list;
            if (useCache)
            {
                string cacheKey = string.Format(listKey, categoryId.ToString());
                object obj = _cacheManager.Get(cacheKey);
                if (obj != null)
                {
                    list = (List<ForumSection>)obj;
                }
                else
                {
                    list = _GetList(categoryId);
                    _cacheManager.Insert(cacheKey, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList(categoryId);
            }
            return list;
        }

        public virtual void Move(int categoryId, int id, string action)
        {
            ForumSection entity1 = GetById(id);
            int entity1NewSortOrder;

            ForumSection entity2 = new ForumSection();
            int entity2NewSortOrder;

            if (action == "UP")
            {
                entity2 = _GetBySortOrder(categoryId, entity1.SortOrder - 1);
                entity1NewSortOrder = entity1.SortOrder - 1;
                entity2NewSortOrder = entity2.SortOrder + 1;
            }
            else
            {
                entity2 = _GetBySortOrder(categoryId, entity1.SortOrder + 1);
                entity1NewSortOrder = entity1.SortOrder + 1;
                entity2NewSortOrder = entity2.SortOrder - 1;
            }

            entity1.SortOrder = entity1NewSortOrder;
            entity2.SortOrder = entity2NewSortOrder;

            _sectionRepository.Update(entity1);
            _sectionRepository.Update(entity2);

            _cacheManager.Remove(string.Format(listKey, categoryId));
        }

        #endregion

        #region Private Methods

        private ForumSection _GetByUrl(int categoryId, string url)
        {
            var query = from item in _sectionRepository.Table
                        where item.CategoryId == categoryId && item.Url == url
                        select item;
            return query.FirstOrDefault();
        }

        private ForumSection _GetBySortOrder(int categoryId, int sortOrder)
        {
            var query = from item in _sectionRepository.Table
                        where item.CategoryId == categoryId && item.SortOrder == sortOrder
                        select item;
            return query.FirstOrDefault();
        }

        private List<ForumSection> _GetList(int categoryId)
        {
            var query = from item in _sectionRepository.Table
                        where item.CategoryId == categoryId
                        orderby item.SortOrder
                        select item;
            return query.ToList();
        }

        #endregion

    }
}