#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using Weapsy.Core.Net;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Extensions;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using Weapsy.Web.Models;
using Weapsy.Web.Plugins.Models.Account;

namespace Weapsy.Web.Plugins.Controllers
{
    public class AccountController : BaseAccountController
    {

        #region Private Members

        private readonly IEmailAccountService _eMailAccountService;
        private readonly IMailManager _mailManager;
        private readonly IMailTemplateService _mailTemplateService;
        private readonly IUserService _userService;

        #endregion

        #region Constructors

        public AccountController(IEmailAccountService eMailAccountService,
            IMailManager mailManager,
            IMailTemplateService mailTemplateService, 
            IUserService userService)
        {
            _eMailAccountService = eMailAccountService;
            _mailManager = mailManager;
            _mailTemplateService = mailTemplateService;
            _userService = userService;
        }

        #endregion

        #region Public Methods

        public ActionResult ChangePassword()
        {
            return View(new ChangePasswordModel());
        }

        [Authorize]
        [HttpPost]
        public ActionResult ChangePassword(ChangePasswordModel model)
        {
            if (ModelState.IsValid)
            {
                bool changePasswordSucceeded;
                try
                {
                    MembershipUser currentUser = Membership.GetUser(User.Identity.Name, true /* userIsOnline */);
                    changePasswordSucceeded = currentUser.ChangePassword(model.OldPassword, model.NewPassword);
                }
                catch (Exception)
                {
                    changePasswordSucceeded = false;
                }

                if (changePasswordSucceeded)
                {

                    MailTemplate mailTemplate = _mailTemplateService.GetSiteMailTemplate(
                        SiteId,
                        WeapsyGlobal.Site.ChangePasswordMailTemplateId,
                        WeapsyGlobal.Language.Id);

                    EmailAccount emailAccount = _eMailAccountService.GetById(SiteId,
                        WeapsyGlobal.Site.ChangePasswordEmailAccountId);

                    if (mailTemplate != null && emailAccount != null)
                    {
                        _mailManager.Send(new Mail()
                        {
                            Address = WeapsyGlobal.User.MembershipEmail,
                            DisplayName = WeapsyGlobal.User.DisplayName,
                            Subject = ReplaceChangePasswordTokens(mailTemplate.LocalizedSubject, model.NewPassword),
                            BodyHTML = ReplaceChangePasswordTokens(mailTemplate.LocalizedBodyHtml, model.NewPassword),
                            BodyPlain = ReplaceChangePasswordTokens(mailTemplate.LocalizedBodyPlain, model.NewPassword)
                        }, emailAccount);
                    }

                    TempData[Consts.MessageKeySuccess] = Resources.Plugins.Account.ChangePassword_ConfirmUpdate;

                    return RedirectToAction("Index", "Account", new { area = "" });
                }
                else
                {
                    ModelState.AddModelError("", Resources.Plugins.Account.ChangePassword_ErrorUpdate);
                }
            }

            return View(model);
        }

        public ActionResult EditProfile()
        {
            User entity = _userService.GetById(WeapsyGlobal.User.Id);

            if (entity == null)
            {
                throw new HttpException(404, "ERROR");
            }

            EditProfileModel model = entity.ToModel();

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateProfile(EditProfileModel model)
        {
            User entity = _userService.GetById(WeapsyGlobal.User.Id);

            if (entity == null)
            {
                throw new HttpException(404, "ERROR");
            }

            if (ModelState.IsValid)
            {
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _userService.Update(entity);

                TempData[Consts.MessageKeySuccess] = Resources.Plugins.Account.EditProfile_ConfirmUpdate;

                return RedirectToAction("Index", "Account", new { area = "" });
            }

            return View(model);
        }

        public ActionResult ChangeEmail()
        {
            User entity = _userService.GetById(UserId);

            if (entity == null)
            {
                throw new HttpException(404, "ERROR");
            }

            ChangeEmailModel model = new ChangeEmailModel();

            model.Email = entity.MembershipEmail;

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateEmail(ChangeEmailModel model)
        {
            User entity = _userService.GetById(UserId);

            if (entity == null)
            {
                throw new HttpException(404, "ERROR");
            }

            if (ModelState.IsValid)
            {
                entity.MembershipEmail = model.Email;
                entity.SetAuditForUpdate();
                _userService.Update(entity);

                MembershipUser mUser = Membership.GetUser(entity.MembershipName);

                if (mUser != null)
                {
                    mUser.Email = model.Email;
                    Membership.UpdateUser(mUser);
                }

                TempData[Consts.MessageKeySuccess] = Resources.Plugins.Account.ChangeEmail_ConfirmUpdate;

                return RedirectToAction("Index", "Account", new { area = "" });
            }

            return View(model);
        }

        #endregion

        #region Private Methods

        private string ReplaceChangePasswordTokens(string text, string newPassword)
        {
            Site site = WeapsyGlobal.Site;
            User user = WeapsyGlobal.User;

            text = text.Replace("{SiteTitle}", site.Title);
            text = text.Replace("{SiteUrl}", site.Url);

            text = text.Replace("{Password}", newPassword);

            text = text.Replace("{UserName}", user.MembershipName);
            text = text.Replace("{Email}", user.MembershipEmail);

            return text;
        }

        #endregion

    }
}