#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using Weapsy.Core.Net;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Framework.Controllers;
using Weapsy.Web.Models.Auth;

namespace Weapsy.Web.Controllers
{
    public class AuthController : BaseAuthController
    {
        #region Private Members

        private readonly IEmailAccountService _eMailAccountService;
        private readonly IMailManager _mailManager;
        private readonly IMailTemplateService _mailTemplateService;
        private readonly ISettingService _settingService;
        private readonly IUserService _userService;

        #endregion

        #region Constructors

        public AuthController(IEmailAccountService eMailAccountService, 
            IMailManager mailManager,
            IMailTemplateService mailTemplateService,
            ISettingService settingService,
            IUserService userService)
        {
            _eMailAccountService = eMailAccountService;
            _mailManager = mailManager;
            _mailTemplateService = mailTemplateService;
            _settingService = settingService;
            _userService = userService;
        }

        #endregion

        #region Public Methods

        #region Login/Logout

        public ActionResult Login()
        {
            LoginModel model = new LoginModel();

            if (HttpContext.Request.IsAuthenticated)
            {
                return RedirectToAction("Index", "Home");
            }

            MemberSettings memberSettings = _settingService.GetSettings<MemberSettings>();
            SecuritySettings securitySettings = _settingService.GetSettings<SecuritySettings>();

            if (memberSettings.LoginName == LoginNameType.UserName)
            {
                model.DisplayUserName = true;
            }

            if (memberSettings.RegistrationType == RegistrationType.Public)
            {
                model.DisplayRegister = true;
            }

            if (securitySettings.LoginCaptcha)
            {
                model.DisplayCaptcha = true;
            }

            return View(model);
        }

        [ReCaptchaValidator(ReCaptchaValidatorAttribute.CaptchaType.Login)]
        [HttpPost]
        public ActionResult Login(LoginModel model, string returnUrl, bool captchaValid)
        {
            MemberSettings memberSettings = _settingService.GetSettings<MemberSettings>();
            SecuritySettings securitySettings = _settingService.GetSettings<SecuritySettings>();

            bool captchaError = false;

            if (securitySettings.LoginCaptcha && !captchaValid)
            {
                captchaError = true;
            }

            if (!captchaError)
            {
                string membershipName = "";

                if (_settingService.GetSettings<MemberSettings>().LoginName == LoginNameType.UserName)
                {
                    membershipName = model.UserName;
                }
                else
                {
                    membershipName = model.Email;
                }

                if (Membership.ValidateUser(membershipName, model.Password))
                {
                    FormsAuthentication.SetAuthCookie(membershipName, model.RememberMe);

                    SessionService.RemoveObject(Consts.SessionKeyUser);

                    if (Url.IsLocalUrl(returnUrl) && returnUrl.Length > 1 && returnUrl.StartsWith("/")
                        && !returnUrl.StartsWith("//") && !returnUrl.StartsWith("/\\"))
                    {
                        return Redirect(returnUrl);
                    }
                    else
                    {
                        return RedirectToAction("Index", "Home");
                    }
                }
                else
                {
                    if (_settingService.GetSettings<MemberSettings>().LoginName == LoginNameType.UserName)
                    {
                        ModelState.AddModelError("", Resources.Auth.Login_Error_UserNameOrPassword);
                    }
                    else
                    {
                        ModelState.AddModelError("", Resources.Auth.Login_Error_EmailOrPassword);
                    }
                }
            }
            else
            {
                ModelState.AddModelError("", Resources.Auth.CaptchaError);
            }

            if (memberSettings.LoginName == LoginNameType.UserName)
            {
                model.DisplayUserName = true;
            }

            if (memberSettings.RegistrationType == RegistrationType.Public)
            {
                model.DisplayRegister = true;
            }

            if (securitySettings.LoginCaptcha)
            {
                model.DisplayCaptcha = true;
            }

            return View(model);
        }

        public ActionResult Logout()
        {
            FormsAuthentication.SignOut();
            SessionService.RemoveObject(Consts.SessionKeyUser);

            return RedirectToAction("Index", "Home");
        }

        #endregion

        #region Register

        public ActionResult Register()
        {
            if (HttpContext.Request.IsAuthenticated)
            {
                return RedirectToAction("Index", "Home");
            }

            MemberSettings memberSettings = _settingService.GetSettings<MemberSettings>();
            SecuritySettings securitySettings = _settingService.GetSettings<SecuritySettings>();

            if (memberSettings.RegistrationType == RegistrationType.Private)
            {
                return RedirectToAction("Index", "Home");
            }

            RegisterModel model = new RegisterModel();

            if (memberSettings.LoginName == LoginNameType.UserName)
            {
                model.DisplayUserName = true;
            }

            if (securitySettings.RegisterCaptcha)
            {
                model.DisplayCaptcha = true;
            }

            return View(model);
        }

        [ReCaptchaValidator(ReCaptchaValidatorAttribute.CaptchaType.Register)]
        [HttpPost]
        public ActionResult Register(RegisterModel model, bool captchaValid)
        {
            MemberSettings memberSettings = _settingService.GetSettings<MemberSettings>();
            SecuritySettings securitySettings = _settingService.GetSettings<SecuritySettings>();

            bool captchaError = false;

            if (securitySettings.RegisterCaptcha && !captchaValid)
            {
                captchaError = true;
            }

            if (!captchaError)
            {
                string membershipName = "";

                if (memberSettings.LoginName == LoginNameType.UserName)
                {
                    membershipName = model.UserName;
                }
                else
                {
                    membershipName = model.Email;
                }

                MembershipCreateStatus createStatus;
                Membership.CreateUser(membershipName, model.Password, model.Email, null, null, true, null, out createStatus);

                if (createStatus == MembershipCreateStatus.Success)
                {
                    FormsAuthentication.SetAuthCookie(membershipName, false);

                    MembershipUser mUser = Membership.GetUser(membershipName);

                    if (_userService.GetByMembershipName(membershipName) == null)
                    {
                        User user = new User();

                        user.MembershipName = mUser.UserName;
                        user.MembershipEmail = mUser.Email;
                        user.MembershipDate = mUser.CreationDate;

                        user.DisplayName = mUser.UserName;
                        user.ProfileName = Utils.ParseTextForUrl(mUser.UserName);

                        user.CreatedByUserId = Consts.NullId;
                        user.CreatedByIpAddress = HttpContext.Request.UserHostAddress;
                        user.CreatedOnDate = Utils.GetDateForDB();

                        user.ModifiedByUserId = Consts.NullId;
                        user.ModifiedByIpAddress = HttpContext.Request.UserHostAddress;
                        user.ModifiedOnDate = Utils.GetDateForDB();

                        _userService.Create(user);
                    }

                    MailTemplate mailTemplate = _mailTemplateService.GetSiteMailTemplate(
                        SiteId, 
                        WeapsyGlobal.Site.RegisterMailTemplateId,
                        WeapsyGlobal.Language.Id);

                    EmailAccount emailAccount = _eMailAccountService.GetById(SiteId, 
                        WeapsyGlobal.Site.RegisterEmailAccountId);

                    if (mailTemplate != null && emailAccount != null)
                    {
                        _mailManager.Send(new Mail()
                        {
                            Address = mUser.Email,
                            DisplayName = mUser.UserName,
                            Subject = ReplaceRegisterTokens(mailTemplate.LocalizedSubject, model),
                            BodyHTML = ReplaceRegisterTokens(mailTemplate.LocalizedBodyHtml, model),
                            BodyPlain = ReplaceRegisterTokens(mailTemplate.LocalizedBodyPlain, model)
                        }, emailAccount);                    
                    }

                    return RedirectToAction("Index", "Home");
                }
                else
                {
                    ModelState.AddModelError("", ErrorCodeToString(createStatus));
                }
            }
            else
            {
                ModelState.AddModelError("", Resources.Auth.CaptchaError);
            }

            if (memberSettings.LoginName == LoginNameType.UserName)
            {
                model.DisplayUserName = true;
            }

            if (securitySettings.RegisterCaptcha
                && !string.IsNullOrWhiteSpace(securitySettings.ReCaptchaPublicKey)
                && !string.IsNullOrWhiteSpace(securitySettings.ReCaptchaPrivateKey))
            {
                model.DisplayCaptcha = true;
            }

            return View(model);
        }

        #endregion

        #region Password Recovery

        public ActionResult PasswordRecovery()
        {
            if (HttpContext.Request.IsAuthenticated)
            {
                return RedirectToAction("Home", "Index");
            }

            PasswordRecoveryModel model = new PasswordRecoveryModel();

            SecuritySettings securitySettings = _settingService.GetSettings<SecuritySettings>();

            if (securitySettings.PasswordRecoveryCaptcha)
            {
                model.DisplayCaptcha = true;
            }

            return View(model);
        }

        [ReCaptchaValidator(ReCaptchaValidatorAttribute.CaptchaType.PasswordRecovery)]
        [HttpPost]
        public ActionResult PasswordRecovery(PasswordRecoveryModel model, bool captchaValid)
        {
            SecuritySettings securitySettings = _settingService.GetSettings<SecuritySettings>();

            bool captchaError = false;

            if (securitySettings.PasswordRecoveryCaptcha && !captchaValid)
            {
                captchaError = true;
            }

            if (!captchaError)
            {
                string membershipName = Membership.GetUserNameByEmail(model.Email);

                if (!string.IsNullOrWhiteSpace(membershipName))
                {
                    MembershipUser mUser = Membership.GetUser(membershipName);

                    string newPassword = mUser.ResetPassword();

                    if (!string.IsNullOrWhiteSpace(newPassword))
                    {
                        MailTemplate mailTemplate = _mailTemplateService.GetSiteMailTemplate(
                            SiteId,
                            WeapsyGlobal.Site.PasswordRecoveryMailTemplateId,
                            WeapsyGlobal.Language.Id);

                        EmailAccount emailAccount = _eMailAccountService.GetById(SiteId,
                            WeapsyGlobal.Site.PasswordRecoveryEmailAccountId);

                        if (mailTemplate != null && emailAccount != null)
                        {
                            User user = _userService.GetByMembershipName(mUser.UserName);

                            _mailManager.Send(new Mail()
                            {
                                Address = model.Email,
                                DisplayName = model.Email,
                                Subject = ReplacePasswordRecoveryTokens(mailTemplate.LocalizedSubject, newPassword, user),
                                BodyHTML = ReplacePasswordRecoveryTokens(mailTemplate.LocalizedBodyHtml, newPassword, user),
                                BodyPlain = ReplacePasswordRecoveryTokens(mailTemplate.LocalizedBodyPlain, newPassword, user)
                            }, emailAccount);
                        }

                        return RedirectToAction("Index", "Home");
                    }
                }
                else
                {
                    ModelState.AddModelError("", "Email address not found");
                }
            }
            else
            {
                ModelState.AddModelError("", Resources.Auth.CaptchaError);
            }

            if (securitySettings.PasswordRecoveryCaptcha
                && !string.IsNullOrWhiteSpace(securitySettings.ReCaptchaPublicKey)
                && !string.IsNullOrWhiteSpace(securitySettings.ReCaptchaPrivateKey))
            {
                model.DisplayCaptcha = true;
            }

            return View(model);
        }

        #endregion

        #endregion

        #region Private Methods

        private string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            // See http://go.microsoft.com/fwlink/?LinkID=177550 for
            // a full list of status codes.
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    if (_settingService.GetSettings<MemberSettings>().LoginName == LoginNameType.Email)
                    {
                        return Resources.Auth.Register_Error_DuplicateEmail;
                    }

                    return Resources.Auth.Register_Error_DuplicateUserName;

                case MembershipCreateStatus.DuplicateEmail:
                    return Resources.Auth.Register_Error_DuplicateEmail;

                case MembershipCreateStatus.InvalidPassword:
                    return Resources.Auth.Register_Error_InvalidPassword;

                case MembershipCreateStatus.InvalidEmail:
                    return Resources.Auth.Register_Error_InvalidEmail;

                case MembershipCreateStatus.InvalidAnswer:
                    return Resources.Auth.Register_Error_InvalidAnswer;

                case MembershipCreateStatus.InvalidQuestion:
                    return Resources.Auth.Register_Error_InvalidQuestion;

                case MembershipCreateStatus.InvalidUserName:
                    return Resources.Auth.Register_Error_InvalidUserName;

                case MembershipCreateStatus.ProviderError:
                    return Resources.Auth.Register_Error_ProviderError;

                case MembershipCreateStatus.UserRejected:
                    return Resources.Auth.Register_Error_UserRejected;

                default:
                    return Resources.Auth.Register_Error_Unknown;
            }
        }

        private string ReplaceRegisterTokens(string text, RegisterModel model)
        {
            Site site = WeapsyGlobal.Site;

            text = text.Replace("{SiteTitle}", site.Title);
            text = text.Replace("{SiteUrl}", site.Url);

            text = text.Replace("{UserName}", model.DisplayUserName ? model.UserName : model.Email);
            text = text.Replace("{Password}", model.Password);
            text = text.Replace("{Email}", model.Email);

            return text;
        }

        private string ReplacePasswordRecoveryTokens(string text, string newPassword, User user)
        {
            Site site = WeapsyGlobal.Site;

            text = text.Replace("{SiteTitle}", site.Title);
            text = text.Replace("{SiteUrl}", site.Url);

            text = text.Replace("{Password}", newPassword);

            text = text.Replace("{UserName}", user.MembershipName);
            text = text.Replace("{Email}", user.MembershipEmail);

            return text;
        }

        #endregion

    }
}