#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Extensions;
using Weapsy.Web.Framework.Controllers;
using Weapsy.Web.Models;
using Weapsy.Web.Models.Home;

namespace Weapsy.Web.Controllers
{

	public class HomeController : BaseController
	{

        #region Private Members

        private readonly IModuleService _moduleService;
        private readonly IModuleTemplateService _moduleTemplateService;
        private readonly IPageService _pageService;
        private readonly IPageTemplateService _pageTemplateService;
        private readonly ISiteService _siteService;
        
        #endregion

        #region Constructors

        public HomeController(IModuleService moduleService,
            IModuleTemplateService moduleTemplateService,
            IPageService pageService,
            IPageTemplateService pageTemplateService,
            ISiteService siteService)
        {
            _moduleService = moduleService;
            _moduleTemplateService = moduleTemplateService;
            _pageService = pageService;
            _pageTemplateService = pageTemplateService;
            _siteService = siteService;
        }

        #endregion

        #region Public Methods

		public ActionResult Index()
		{
            Page page = new Page();

            int pageId = 0;

            if (RouteData.DataTokens.Keys.Count > 0)
            {
                if (RouteData.DataTokens["pageId"] != null)
                {
                    pageId = (int)RouteData.DataTokens["pageId"];
                }
            }

            if (pageId > 0)
            {
                page = WeapsyGlobal.Pages.Find(x => x.Id == pageId);
            }
            else
            {
                page = WeapsyGlobal.Pages.Find(x => x.Id == WeapsyGlobal.Site.HomePageId);

                if (page == null)
                {
                    // DO SOMETHING
                }
            }

            if (page != null)
            {
                if (Utils.HasPermission(page.Authorized))
                {
                    Site currentSite = WeapsyGlobal.Site;

                    string localizedMetaTitle = currentSite.MetaTitle;
                    string localizedMetaDescription = currentSite.MetaDescription;
                    string localizedMetaKeywords = currentSite.MetaKeywords;

                    LocalizedSite localizedSite = 
                        _siteService.GetLocalizedBySiteId(currentSite.Id, 
                        WeapsyGlobal.Language.Id,
                        true);

                    if (localizedSite != null)
                    {
                        if (!string.IsNullOrWhiteSpace(localizedSite.MetaTitle))
                        {
                            localizedMetaTitle = localizedSite.MetaTitle;
                        }

                        if (!string.IsNullOrWhiteSpace(localizedSite.MetaDescription))
                        {
                            localizedMetaDescription = localizedSite.MetaDescription;
                        }

                        if (!string.IsNullOrWhiteSpace(localizedSite.MetaKeywords))
                        {
                            localizedMetaKeywords = localizedSite.MetaKeywords;
                        }
                    }

                    if (string.IsNullOrWhiteSpace(localizedMetaTitle))
                    {
                        localizedMetaTitle = currentSite.Title;
                    }

                    if (!string.IsNullOrWhiteSpace(page.LocalizedMetaTitle))
                    {
                        ViewBag.MetaTitle = page.LocalizedMetaTitle;
                    }
                    else
                    {
                        ViewBag.MetaTitle = localizedMetaTitle;
                    }

                    if (!string.IsNullOrWhiteSpace(page.LocalizedMetaDescription))
                    {
                        ViewBag.MetaDescription = page.LocalizedMetaDescription;
                    }
                    else
                    {
                        ViewBag.MetaDescription = localizedMetaDescription;
                    }

                    if (!string.IsNullOrWhiteSpace(page.LocalizedMetaKeywords))
                    {
                        ViewBag.MetaKeywords = page.LocalizedMetaKeywords;
                    }
                    else
                    {
                        ViewBag.MetaKeywords = localizedMetaKeywords;
                    }

                    PageTemplate pageTemplate = new PageTemplate();
                    ModuleTemplate moduleTemplate = new ModuleTemplate();

                    if (page.PageTemplateId > 0)
                    {
                        pageTemplate = _pageTemplateService.GetById(page.PageTemplateId);
                    }
                    else
                    {
                        pageTemplate = _pageTemplateService.GetById(WeapsyGlobal.Site.PageTemplateId);
                    }

                    if (page.ModuleTemplateId > 0)
                    {
                        moduleTemplate = _moduleTemplateService.GetById(page.ModuleTemplateId);
                    }
                    else
                    {
                        moduleTemplate = _moduleTemplateService.GetById(WeapsyGlobal.Site.ModuleTemplateId);
                    }

                    List<PageModule> AllModulesForPage = new List<PageModule>();

                    foreach (var item in _moduleService.GetPageModules(page.SiteId, page.Id, WeapsyGlobal.Language.Id))
                    {
                        bool canAdd = true;

                        if (item.Module.StartDate.HasValue)
                        {
                            if (item.Module.StartDate.Value.ToLocalTime() > DateTime.Now.ToLocalTime())
                            {
                                canAdd = false;
                            }
                        }

                        if (item.Module.EndDate.HasValue)
                        {
                            if (item.Module.EndDate.Value.ToLocalTime() < DateTime.Now.ToLocalTime())
                            {
                                canAdd = false;
                            }
                        }

                        if (canAdd) { AllModulesForPage.Add(item); }
                    }

                    List<PageModule> AllModulesForUser = new List<PageModule>();

                    foreach (var item in AllModulesForPage)
                    {
                        if (item.Module.InheritPermissions || Utils.HasPermission(item.Module.Authorized))
                        {
                            AllModulesForUser.Add(item);
                        }
                    }

                    List<ZoneModel> Zones = new List<ZoneModel>();

                    foreach (string Name in pageTemplate.Zones.Split(",".ToCharArray()))
                    {
                        ZoneModel zone = new ZoneModel();

                        List<ModuleModel> Modules = new List<ModuleModel>();

                        List<PageModule> ModulesForZone = AllModulesForUser.FindAll(m => m.Zone == Name);

                        foreach (var item in ModulesForZone)
                        {
                            ModuleModel settings = new ModuleModel();

                            if (item.ModuleTemplateId > 0)
                            {
                                settings.ModuleTemplate = _moduleTemplateService.GetById(item.ModuleTemplateId).ToModel();
                            }
                            else
                            {
                                settings.ModuleTemplate = moduleTemplate.ToModel();
                            }

                            settings.SharedSettings = item.Module.ToModel();
                            settings.PageSettings = item.ToModel();
                            settings.Definition = item.Module.ModuleDefinition.ToModel();
                            //settings.ModuleTemplate
                            settings.Page = page.ToModel();
                            settings.Context = WeapsyGlobal;
                            settings.LocalizedTitle = item.LocalizedTitle;

                            Modules.Add(settings);
                        }

                        zone.Name = Name;
                        zone.Modules = Modules;

                        Zones.Add(zone);
                    }

                    IndexModel model = new IndexModel();

                    model.Page = page.ToModel();
                    model.Zones = Zones;
                    model.PageTemplate = pageTemplate.ToModel();

                    return View("../Shared/PageTemplates/" + pageTemplate.ViewName, model);
                }
                else
                {
                    //return RedirectToAction("AccessDenied", "Error", new { area = "" });
                    throw new HttpException(404, "Page Not Found");
                }
            }
            else
            {
                throw new HttpException(404, "Page Not Found");
            }
		}

        public ActionResult ChangeLanguage(int id, string ReturnUrl)
        {
            CookieService.SetCookieValue(Consts.CookieKeyLanguage, id.ToString(), 30);
            //return Redirect(ReturnUrl);
            return RedirectToAction("Index");
        }

        //public ActionResult SendFeedback(string Text)
        //{
        //    try {
        //        if (!string.IsNullOrEmpty(Text)) {
        //            Mail mail = new Mail();

        //            mail.FromDisplay = "My Site";
        //            mail.FromEmail = "feedback@mysite.com";
        //            mail.ToDisplay = "Luca";
        //            mail.ToEmail = "info@mysite.com";
        //            mail.Subject = "Feedback";
        //            mail.BodyHTML = Text;
        //            mail.BodyPlain = Text;

        //            mail.Send();

        //            return Content("Thanks!");
        //        } else {
        //            return Content("Please, write something.");
        //        }
        //    } catch (Exception ex) {
        //        return Content("Ooops!");
        //    }
        //}

		public ActionResult SiteMap()
		{

			return View();
		}

		public ActionResult Privacy()
		{

			return View();
		}

		public ActionResult Terms()
		{

			return View();
        }

        #endregion

    }

}