#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Caching;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Entities;
using Weapsy.Services;

namespace Weapsy.Web.Framework
{

	public class WeapsyGlobal
	{
        private readonly IInstallService _installService;
        private readonly ILanguageService _languageService;
        private readonly IPageService _pageService;
        private readonly ISiteService _siteService;
        private readonly IThemeService _themeService;
        private readonly IUserService _userService;

        private const string ContextKey = Consts.GlobalKey + "|WeapsyGlobal";

        public Site Site { get; set; }
        public Theme Theme { get; set; }
        public Language Language { get; set; }
        public User User { get; set; }
        public List<Page> Pages { get; set; }

		public WeapsyGlobal()
		{
            _installService = DependencyResolver.Current.GetService<IInstallService>();
            _languageService = DependencyResolver.Current.GetService<ILanguageService>();
            _pageService = DependencyResolver.Current.GetService<IPageService>();
            _siteService = DependencyResolver.Current.GetService<ISiteService>();
            _themeService = DependencyResolver.Current.GetService<IThemeService>();
            _userService = DependencyResolver.Current.GetService<IUserService>();

            Site = GetSite();
            User = GetUser();
            Language = GetLanguage();
            Theme = GetTheme();
            Pages = GetPages();
		}

		public static WeapsyGlobal Current {
			get {
                if (HttpContext.Current.Items[ContextKey] == null)
                {
                    HttpContext.Current.Items.Add(ContextKey, new WeapsyGlobal());
                }
                return (WeapsyGlobal)HttpContext.Current.Items[ContextKey];
			}
		}

        private Site GetSite()
        {
            Site site = _siteService.Get();
            if (site == null)
            {
                _installService.Initialize();
                site = new Site();
                site = _siteService.Get();
            }
            return site;
        }

        private Theme GetTheme()
        {
            Theme theme = _themeService.GetById(Site.ThemeId);
            if (theme == null)
            {
                theme = new Theme();
                theme.Name = "Default";
                theme.Folder = "Default";
            }
            return theme;
        }

        private Language GetLanguage()
        {
            Language language = new Language();

            if (Site.LanguageSelectionEnabled)
            {
                string lang = "";

                if (HttpContext.Current.Request.RequestContext.RouteData.Values["lang"] != null &&
                    !string.IsNullOrWhiteSpace(HttpContext.Current.Request.RequestContext.RouteData.Values["lang"].ToString()))
                {
                    lang = HttpContext.Current.Request.RequestContext.RouteData.Values["lang"].ToString();
                }

                if (lang == "")
                {
                    int langId = CookieService.GetCookieAsInt(Consts.CookieKeyLanguage, 0);
                    if (langId > 0)
                    {
                        language = _languageService.GetById(Site.Id, langId);
                    }
                    else
                    {
                        language = null;
                    }
                }
                else
                {
                    language = _languageService.GetByUrl(Site.Id, lang);
                }
            }
            else
            {
                language = null;
            }

            if (language == null)
            {
                language = new Language();

                //if (false)
                //{
                //    language = _languageService.GetByCultureName(Site.Id, HttpContext.Current.Request.UserLanguages[0]);
                //}
                //else
                //{
                //    language = _languageService.GetById(Site.Id, Site.LanguageId);
                //}

                language = _languageService.GetById(Site.Id, Site.LanguageId);
            }
            else
            {
                if (language.Active == false)
                {
                    language = null;
                }
            }

            if (language == null)
            {
                language = new Language();
                language.CultureName = AppSettings.DefaultLanguage;
                //language = _languageService.GetDefault(Site.Id);
            }

            return language;
        }

        private User GetUser()
        {
            User user = new User();

            if (HttpContext.Current.Request.IsAuthenticated)
            {
                if (HttpContext.Current.Session != null)
                {
                    object obj = HttpContext.Current.Session[Consts.SessionKeyUser];

                    if (obj != null)
                    {
                        user = (User)obj;
                    }
                    else
                    {
                        user = _userService.GetByMembershipName(HttpContext.Current.User.Identity.Name, false);

                        if (user == null)
                        {
                            user = new User();

                            MembershipUser mUser = Membership.GetUser(HttpContext.Current.User.Identity.Name);

                            if (mUser != null)
                            {
                                User newUser = new User();

                                newUser.MembershipName = mUser.UserName;
                                newUser.MembershipEmail = mUser.Email;
                                newUser.MembershipDate = mUser.CreationDate;

                                newUser.DisplayName = mUser.UserName;
                                newUser.ProfileName = Utils.ParseTextForUrl(mUser.UserName);

                                newUser.CreatedByUserId = Consts.NullId;
                                newUser.CreatedByIpAddress = HttpContext.Current.Request.UserHostAddress;
                                newUser.CreatedOnDate = Utils.GetDateForDB();

                                newUser.ModifiedByUserId = Consts.NullId;
                                newUser.ModifiedByIpAddress = HttpContext.Current.Request.UserHostAddress;
                                newUser.ModifiedOnDate = Utils.GetDateForDB();

                                _userService.Create(newUser);

                                user = _userService.GetByMembershipName(HttpContext.Current.User.Identity.Name, false);                            
                            }
                        }

                        HttpContext.Current.Session[Consts.SessionKeyUser] = user;
                    }

                    user.IsSuperAdmin = Roles.IsUserInRole(AppSettings.RoleSuperAdmin);
                    user.IsAdmin = Roles.IsUserInRole(AppSettings.RoleAdmin);
                }
                else
                {
                    user.IsSuperAdmin = false;
                    user.IsAdmin = false;
                }
            }
            else
            {
                user.IsSuperAdmin = false;
                user.IsAdmin = false;
            }

            return user;
        }

        private List<Page> GetPages()
        {
            List<Page> result = new List<Page>();

            foreach (var item in _pageService.GetSitePages(Site.Id, Language.Id))
            {
                bool canAdd = true;

                if (item.StartDate.HasValue)
                {
                    if (item.StartDate.Value.ToLocalTime() > DateTime.Now.ToLocalTime())
                    {
                        canAdd = false;
                    }
                }

                if (item.EndDate.HasValue)
                {
                    if (item.EndDate.Value.ToLocalTime() < DateTime.Now.ToLocalTime())
                    {
                        canAdd = false;
                    }
                }

                if (canAdd && item.Active && Utils.HasPermission(item.Authorized))
                {
                    result.Add(item);
                }
            }

            return result;
        }

	}

}