﻿#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Weapsy.Core.Tasks;
using Weapsy.Data;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Tasks;
using Weapsy.Web.Framework;
using Weapsy.Web.Models;
using Autofac;
using Autofac.Core;
using Autofac.Integration.Mvc;

namespace Weapsy
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : HttpApplication
    {
        private Assembly ExecutingAssembly { get; set; }

        public MvcApplication()
        {
            // cache executing assembly for future use
            ExecutingAssembly = Assembly.GetExecutingAssembly();
        }

        public void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public void RegisterRoutes(RouteCollection routes)
        {
            var _siteService = DependencyResolver.Current.GetService<ISiteService>();
            var _languageService = DependencyResolver.Current.GetService<ILanguageService>();
            var _pageService = DependencyResolver.Current.GetService<IPageService>();

            // Can't use for compatibility with IIS 7.0
            // WeapsyGlobal context = WeapsyGlobal.Current;
            // Define instead current Site and default Language
            Site currentSite = _siteService.Get();
            Language defaultLanguage = _languageService.GetById(currentSite.Id, currentSite.LanguageId);

            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            const string Namespace = "Weapsy.Web.Controllers";

            routes.MapRoute(
                "Install",
                "install",
                new { controller = "Install", action = "Index" },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "SiteMap",
                "sitemap",
                new { controller = "Home", action = "SiteMap" },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Login",
                "login",
                new { controller = "Auth", action = "Login" },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Logout",
                "logout",
                new { controller = "Auth", action = "Logout" },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Register",
                "register",
                new { controller = "Auth", action = "Register" },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Auth",
                "auth/{action}/{id}",
                new { controller = "Auth", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Account",
                "account/{action}/{id}",
                new { controller = "Account", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Group",
                "group/{action}/{id}",
                new { controller = "Group", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { Namespace }
            );

            routes.MapRoute(
                "Profile",
                "profile/{action}/{id}",
                new { controller = "Profile", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { Namespace }
            );

            //routes.MapRoute(
            //    "Text",
            //    "Text/{action}/{id}",
            //    new { controller = "Text", action = "Index", id = UrlParameter.Optional },
            //    null,
            //    new string[] { Namespace }
            //);

            routes.MapRoute(
                "ChangeLanguage",
                "ChangeLanguage/{id}",
                new { controller = "Home", action = "ChangeLanguage" },
                null,
                new string[] { Namespace }
            );

            RouteValueDictionary defaults = new RouteValueDictionary();
            RouteValueDictionary constraints = new RouteValueDictionary();
            RouteValueDictionary tokens = new RouteValueDictionary();

            defaults.Add("controller", "home");
            defaults.Add("action", "index");
            defaults.Add("data", string.Empty);
            constraints.Add("data", @"[a-zA-Z0-9\-]*");
            tokens.Add("namespaces", new[] { Namespace });
            tokens.Add("pageId", 0);

            routes.Add(new Route("", defaults, constraints, tokens, new MyRouteHandler()));

            // LOCALIZED ROUTES

            List<Language> Languages = _languageService.GetList(currentSite.Id);

            bool useLocalizedUrls = Languages.Count > 1
                && currentSite.LanguageSelectionEnabled
                && currentSite.LanguageUrlEnabled;

            if (useLocalizedUrls)
            {
                foreach (Language l in Languages)
                {
                    foreach (Page p in _pageService.GetSitePages(currentSite.Id, l.Id))
                    {
                        if (p.Active == true && p.Deleted == false)
                        {
                            string data = "/{*data}";

                            foreach (string url in Consts.ReservedUrls)
                            {
                                if (url == p.Url)
                                {
                                    data = "";
                                    break;
                                }
                            }

                            defaults = new RouteValueDictionary();
                            constraints = new RouteValueDictionary();
                            tokens = new RouteValueDictionary();

                            defaults.Add("lang", l.Url);
                            defaults.Add("controller", "home");
                            defaults.Add("action", "index");
                            defaults.Add("data", string.Empty);
                            constraints.Add("data", @"[a-zA-Z0-9\-]*");
                            tokens.Add("namespaces", new[] { Namespace });
                            tokens.Add("pageId", p.Id);

                            routes.Add(new Route(l.Url + "/" + p.LocalizedUrl + data, defaults, constraints, tokens, new MyRouteHandler()));
                        }
                    }
                }

                routes.MapRoute(
                    "Default_Localized",
                    "{lang}/{controller}/{action}/{id}",
                    new { controller = "Home", action = "Index", id = UrlParameter.Optional },
                    null,
                    new string[] { Namespace }
                );

            }

            // STANDARD ROUTES

            foreach (Page p in _pageService.GetSitePages(currentSite.Id, defaultLanguage.Id))
            {
                if (p.Active == true && p.Deleted == false) {

                    defaults = new RouteValueDictionary();
                    constraints = new RouteValueDictionary();
                    tokens = new RouteValueDictionary();

                    string data = "/{*data}";

                    foreach (string url in Consts.ReservedUrls)
                    {
                        if (url == p.Url)
                        {
                            data = "";
                            break;
                        }
                    }

                    defaults.Add("controller", "home");
                    defaults.Add("action", "index");
                    defaults.Add("data", string.Empty);
                    constraints.Add("data", @"[a-zA-Z0-9\-]*");
                    tokens.Add("namespaces",  new[] {Namespace});
                    tokens.Add("pageId", p.Id);

                    routes.Add(new Route(p.Url + data, defaults, constraints, tokens, new MyRouteHandler()));
                }
            }

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional }, // Parameter defaults
                null,
                new string[] { Namespace }
            );

        }

        protected void Application_Start()
        {
            ExecuteStartupTasks();

            var _installService = DependencyResolver.Current.GetService<IInstallService>();

            _installService.EnsureInstalled();

            AreaRegistration.RegisterAllAreas();

            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);

            ViewEngines.Engines.Clear();
            ViewEngines.Engines.Add(new RazorViewEngine());
        }

        protected void Application_AcquireRequestState(object sender, EventArgs e)
        {
            CultureInfo ci = new CultureInfo(WeapsyGlobal.Current.Language.CultureName);

            System.Threading.Thread.CurrentThread.CurrentUICulture = ci;
            System.Threading.Thread.CurrentThread.CurrentCulture = CultureInfo.CreateSpecificCulture(ci.Name);
        }

        #region Private Methods

        //=============================================================================
        //  Created by Michael Hornfeck
        //=============================================================================
        //  Modified by Luca Cammarata Briguglia on Jun 22 2012
        //  Added Autofac Modules registration
        //=============================================================================
        /// <summary>
        /// Reflects on all assemblies in bin to find types derived from IStartupTask and executes them.
        /// </summary>
        private void ExecuteStartupTasks()
        {
            var builder = new ContainerBuilder();

            List<IStartupTask> startupTasks = new List<IStartupTask>();
            List<IModule> modules = new List<IModule>();

            Assembly asm = this.ExecutingAssembly;

            // get path of executing (bin) folder from the current executing assembly
            string codeBase = this.ExecutingAssembly.CodeBase;
            UriBuilder uri = new UriBuilder(codeBase);
            string path = Uri.UnescapeDataString(uri.Path);
            string bin = Path.GetDirectoryName(path);
            string[] assemblies = Directory.GetFiles(bin, "*.dll");

            foreach (String file in assemblies)
            {
                try
                {
                    if (File.Exists(file))
                    {
                        // load the assembly
                        asm = Assembly.LoadFrom(file);

                        // get all types from the assembly that inherit IStartupTask
                        var iStartUpTaskQuery = from t in asm.GetTypes()
                                    where t.IsClass && t.GetInterface(typeof(IStartupTask).FullName) != null
                                    select t;

                        // add types to list of startup tasks
                        foreach (Type type in iStartUpTaskQuery)
                        {
                            startupTasks.Add((IStartupTask)Activator.CreateInstance(type));
                        }

                        // get all types from the assembly that inherit Autofac.Core.IModule
                        var iModuleQuery = from t in asm.GetTypes()
                                    where t.IsClass && t.GetInterface(typeof(IModule).FullName) != null
                                    select t;

                        // add types to list of modules
                        foreach (Type type in iModuleQuery)
                        {
                            modules.Add((IModule)Activator.CreateInstance(type));
                        }

                        // register model binders and controllers
                        builder.RegisterModelBinders(asm);
                        builder.RegisterControllers(asm);
                    }
                }
                catch (Exception ex)
                {
                    Exceptions.LogException(ex);
                }
            }

            // execute each startup task
            foreach (IStartupTask task in startupTasks)
            {
                task.Execute();
            }

            // register each module
            foreach (IModule module in modules)
            {
                builder.RegisterModule(module);
            }

            builder.RegisterModelBinderProvider();
            var container = builder.Build();
            DependencyResolver.SetResolver(new AutofacDependencyResolver(container));
        }

        #endregion Private Methods
    }

    #region Handlers

    //=============================================================================
    //  Thanks to maxnk: http://stackoverflow.com/questions/379558/mvcnet-routing
    //=============================================================================

    public class MyRouteHandler : IRouteHandler
    {
	    public IHttpHandler GetHttpHandler(RequestContext requestContext)
	    {
		    return new MyHttpHandler(requestContext);
	    }
    }

    public class MyHttpHandler : MvcHandler
    {
	    public MyHttpHandler(RequestContext requestContext) : base(requestContext)
	    {
	    }

	    protected override void ProcessRequest(HttpContextBase httpContext)
	    {
            //IController controller = new Web.Controllers.HomeController();
            //(controller as Controller).ActionInvoker = new MyActionInvoker();
            //controller.Execute(RequestContext);
	    }
    }

    public class MyActionInvoker : ControllerActionInvoker
    {
	    protected override System.Web.Mvc.ActionResult InvokeActionMethod(System.Web.Mvc.ControllerContext controllerContext, System.Web.Mvc.ActionDescriptor actionDescriptor, System.Collections.Generic.IDictionary<string, object> parameters)
	    {
		    string path = controllerContext.RouteData.GetRequiredString("data");
		    string[] tokens = path.Split('/');

		    Dictionary<string, string> dic = new Dictionary<string, string>();
		    for (int i = 0; i <= tokens.Length - 1; i++) {
			    dic.Add(tokens[i], tokens[i++]);
		    }

		    parameters["params"] = dic;

		    return base.InvokeActionMethod(controllerContext, actionDescriptor, parameters);
	    }

    }

    #endregion

}